@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // ecasound.cpp: Console mode user interface to ecasound.
-// Copyright (C) 2002,2003 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 2002-2005 Kai Vehmanen
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -132,6 +132,7 @@ int main(int argc, char *argv[])
     ECA_LOGGER::instance().set_log_level(ECA_LOGGER::info, true);
     ECA_LOGGER::instance().set_log_level(ECA_LOGGER::subsystems, true);
     ECA_LOGGER::instance().set_log_level(ECA_LOGGER::eiam_return_values, true);
+    ECA_LOGGER::instance().set_log_level(ECA_LOGGER::module_names, true);
     
     /* 6. create eca objects */
     ecasound_create_eca_objects(state, *clineout);
@@ -283,6 +284,7 @@ void ecasound_main_loop(struct ecasound_state* state)
 	if (cmd == "quit" || cmd == "q") {
 	  state->console->print("---\necasound: Exiting...");
 	  state->exit_request = 1;
+	  ECA_LOGGER::instance().flush();
 	}
       }
     }
@@ -295,7 +297,10 @@ void ecasound_main_loop(struct ecasound_state* state)
       DBC_CHECK(res == 0);
     }
 
-    ctrl->connect_chainsetup();
+    if (ctrl->is_valid() == true) {
+      ctrl->connect_chainsetup();
+    }
+
     if (ctrl->is_connected() == true) {
       ctrl->run();
     }
@@ -396,7 +401,7 @@ void ecasound_print_usage(void)
 void ecasound_print_version_banner(void)
 {
   cout << "ecasound v" << ecasound_library_version << endl;
-  cout << "Copyright (C) 1997-2003 Kai Vehmanen" << endl;
+  cout << "Copyright (C) 1997-2005 Kai Vehmanen" << endl;
   cout << "Ecasound comes with ABSOLUTELY NO WARRANTY." << endl;
   cout << "You may redistribute copies of ecasound under the terms of the GNU" << endl;
   cout << "General Public License. For more information about these matters, see" << endl; 
