@@ -1,7 +1,7 @@
 // ------------------------------------------------------------------------
 // ecasignalview.cpp: A simple command-line tools for monitoring
 //                    signal amplitude.
-// Copyright (C) 1999-2002 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 1999-2005 Kai Vehmanen
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -94,7 +94,7 @@ void ecasv_signal_handler(int signum);
  * Static global variables
  */
 
-static const string ecatools_signalview_version = "20021028-5";
+static const string ecatools_signalview_version = "20050401-7";
 static const double ecasv_clipped_threshold_const = 1.0f - 1.0f / 16384.0f;
 static const int ecasv_bar_length_const = 32;
 static const int ecasv_header_height_const = 10;
@@ -109,6 +109,9 @@ static int ecasv_chcount = 0;
 
 static ECA_CONTROL_INTERFACE* ecasv_eci_repp = 0;
 
+static sig_atomic_t done = 0;
+static sig_atomic_t reset_stats = 0;
+
 /**
  * Function definitions
  */
@@ -124,6 +127,7 @@ int main(int argc, char *argv[])
   sigaction(SIGINT, &es_handler, 0);
   sigaction(SIGQUIT, &es_handler, 0);
   sigaction(SIGABRT, &es_handler, 0);
+  sigaction(SIGHUP, &es_handler, 0);
 
   struct sigaction ign_handler;
   ign_handler.sa_handler = SIG_IGN;
@@ -182,12 +186,15 @@ int main(int argc, char *argv[])
 
   eci.command("start");
 
-  while(true) {
+  while(! done ) {
     kvu_sleep(secs, msecs * 1000000);
     ecasv_print_vu_meters(&eci, &chstats);
   }
 
   ecasv_output_cleanup();
+#ifdef ECASV_USE_CURSES
+  endwin();
+#endif
   
   return 0;
 }
@@ -264,7 +271,7 @@ void ecasv_output_init(void)
     erase();
 
     mvprintw(0, 0, "******************************************************\n");
-    mvprintw(1, 0, "* ecasignalview v%s (C) 1999-2002 Kai Vehmanen  \n", ecatools_signalview_version.c_str());
+    mvprintw(1, 0, "* ecasignalview v%s (C) 1999-2005 Kai Vehmanen  \n", ecatools_signalview_version.c_str());
     mvprintw(2, 0, "******************************************************\n\n");
 
     mvprintw(4, 0, "input=\"%s\"\noutput=\"%s\"\naudio_format=\"%s\", refresh_rate_ms='%ld', buffersize='%ld'\n", 
@@ -299,6 +306,15 @@ void ecasv_output_cleanup(void)
 
 void ecasv_print_vu_meters(ECA_CONTROL_INTERFACE* eci, vector<struct ecasv_channel_stats>* chstats)
 {
+  /* check wheter to reset peaks */
+  if (reset_stats) {
+    reset_stats = 0;
+    for(int n = 0; n < ecasv_chcount; n++) {
+      (*chstats)[n].max_peak = 0;
+      (*chstats)[n].clipped_samples = 0;
+    }
+  }
+
 #ifdef ECASV_USE_CURSES
   for(int n = 0; n < ecasv_chcount; n++) {
     eci->command("copp-select " + kvu_numtostr(n + 1));
@@ -359,8 +375,8 @@ void ecasv_create_bar(double value, int barlen, unsigned char* barbuf)
 void ecasv_print_usage(void)
 {
   cerr << "****************************************************************************\n";
-  cerr << "* ecasignalview, v" << ecatools_signalview_version << "\n";
-  cerr << "* (C) 1999-2002 Kai Vehmanen, released under GPL licence\n";
+  cerr << "* ecasignalview, v" << ecatools_signalview_version << " (" << VERSION << ")\n";
+  cerr << "* (C) 1999-2005 Kai Vehmanen, released under GPL licence\n";
   cerr << "****************************************************************************\n";
 
   cerr << "\nUSAGE: ecasignalview [options] [input] [output] \n";
@@ -374,7 +390,11 @@ void ecasv_print_usage(void)
 
 void ecasv_signal_handler(int signum)
 {
-  cerr << "Unexpected interrupt... cleaning up.\n";
-  ecasv_output_cleanup();
-  exit(1);
+  if (signum == SIGHUP) {
+    reset_stats = 1;
+  }
+  else {
+    cerr << "Interrupted... cleaning up.\n";
+    done=1;
+  }
 }
