@@ -1,6 +1,9 @@
 // ------------------------------------------------------------------------
 // audiofx_filter.cpp: Routines for filter effects.
-// Copyright (C) 1999 Kai Vehmanen (kaiv@wakkanet.fi)
+// Copyright (C) 1999,2004 Kai Vehmanen
+//
+// Attributes:
+//     eca-style-version: 2
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,29 +22,42 @@
 
 #include <cmath>
 
-#include <kvutils.h>
+#include <kvu_utils.h>
 
 #include "samplebuffer_iterators.h"
-#include "eca-debug.h"
+#include "sample-ops_impl.h"
+#include "eca-logger.h"
 #include "audiofx_filter.h"
 
-EFFECT_BANDPASS::EFFECT_BANDPASS (CHAIN_OPERATOR::parameter_type centerf, CHAIN_OPERATOR::parameter_type w) {
+
+EFFECT_FILTER::~EFFECT_FILTER(void)
+{
+}
+
+EFFECT_BANDPASS::EFFECT_BANDPASS (CHAIN_OPERATOR::parameter_t centerf, CHAIN_OPERATOR::parameter_t w)
+{
+  /* to avoid accessing uninitialized data */
+  width = 1;
+  center = 1;
+  C = 1;
+
   set_parameter(1, centerf);
   set_parameter(2, w);
 }
 
-void EFFECT_BANDPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_BANDPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1: 
     center = value;
-    D = 2 * cos(2 * M_PI * center / (CHAIN_OPERATOR::parameter_type)samples_per_second());
+    D = 2 * cos(2 * M_PI * center / (CHAIN_OPERATOR::parameter_t)samples_per_second());
     b[0] = -C * D * a[0];
     break;
   case 2: 
     if (value != 0) width = value;
     else width = center / 2;
-    C = 1.0 / tan(M_PI * width / (CHAIN_OPERATOR::parameter_type)samples_per_second());
-    D = 2 * cos(2 * M_PI * center / (CHAIN_OPERATOR::parameter_type)samples_per_second());
+    C = 1.0 / tan(M_PI * width / (CHAIN_OPERATOR::parameter_t)samples_per_second());
+    D = 2 * cos(2 * M_PI * center / (CHAIN_OPERATOR::parameter_t)samples_per_second());
     a[0] = 1.0 / (1.0 + C);
     a[1] = 0.0;
     a[2] = -a[0];
@@ -51,34 +67,36 @@ void EFFECT_BANDPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_type va
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_BANDPASS::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_BANDPASS::get_parameter(int param) const
+{ 
   switch (param) {
   case 1: 
-    return(center);
+    return center;
   case 2: 
-    return(width);
+    return width;
   }
-  return(0.0);
+  return 0.0;
 }
 
-EFFECT_BANDREJECT::EFFECT_BANDREJECT (CHAIN_OPERATOR::parameter_type centerf, CHAIN_OPERATOR::parameter_type w) 
+EFFECT_BANDREJECT::EFFECT_BANDREJECT (CHAIN_OPERATOR::parameter_t centerf, CHAIN_OPERATOR::parameter_t w)
 {
   set_parameter(1, centerf);
   set_parameter(2, w);
 }
 
-void EFFECT_BANDREJECT::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_BANDREJECT::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1: 
     center = value;
-    D = 2 * cos(2 * M_PI * center / (CHAIN_OPERATOR::parameter_type)samples_per_second());
+    D = 2 * cos(2 * M_PI * center / (CHAIN_OPERATOR::parameter_t)samples_per_second());
     a[1] = -D * a[0];
     b[0] = a[1];
     break;
   case 2: 
     if (value != 0) width = value;
     else width = center / 2;
-    C = tan(M_PI * width / (CHAIN_OPERATOR::parameter_type)samples_per_second());
+    C = tan(M_PI * width / (CHAIN_OPERATOR::parameter_t)samples_per_second());
     a[0] = 1.0 / (1.0 + C);
     a[1] = -D * a[0];
     a[2] = a[0];
@@ -88,34 +106,36 @@ void EFFECT_BANDREJECT::set_parameter(int param, CHAIN_OPERATOR::parameter_type
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_BANDREJECT::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_BANDREJECT::get_parameter(int param) const
+{ 
   switch (param) {
   case 1: 
-    return(center);
+    return center;
   case 2: 
-    return(width);
+    return width;
   }
-  return(0.0);
+  return 0.0;
 }
 
-void EFFECT_BW_FILTER::init(SAMPLE_BUFFER *insample) {
+void EFFECT_BW_FILTER::init(SAMPLE_BUFFER *insample)
+{
   i.init(insample);
 
   set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
 
-  sin.resize(insample->number_of_channels(), vector<SAMPLE_SPECS::sample_type> (2));
-  sout.resize(insample->number_of_channels(), vector<SAMPLE_SPECS::sample_type> (2));
+  sin.resize(insample->number_of_channels(), std::vector<SAMPLE_SPECS::sample_t> (2));
+  sout.resize(insample->number_of_channels(), std::vector<SAMPLE_SPECS::sample_t> (2));
 }
 
-void EFFECT_BW_FILTER::process(void) {
+void EFFECT_BW_FILTER::process(void)
+{
   i.begin();
   while(!i.end()) {
-    outputSample = a[0] * (*i.current()) + 
-                   a[1] * sin[i.channel()][0] + 
-                   a[2] * sin[i.channel()][1] - 
-                   b[0] * sout[i.channel()][0] - 
-                   b[1] * sout[i.channel()][1];
+    outputSample = ecaops_flush_to_zero(a[0] * (*i.current()) + 
+					a[1] * sin[i.channel()][0] + 
+					a[2] * sin[i.channel()][1] - 
+					b[0] * sout[i.channel()][0] - 
+					b[1] * sout[i.channel()][1]);
     sin[i.channel()][1] = sin[i.channel()][0];
     sin[i.channel()][0] = *i.current();
 
@@ -127,7 +147,8 @@ void EFFECT_BW_FILTER::process(void) {
   }
 }
 
-void EFFECT_BW_FILTER::process_notused(SAMPLE_BUFFER* sbuf) {
+void EFFECT_BW_FILTER::process_notused(SAMPLE_BUFFER* sbuf)
+{
 //    sbuf->first();
 //    while(sbuf->is_readable()) {
 //      outputSample = *sbuf->current_sample() * a[0]
@@ -147,7 +168,8 @@ void EFFECT_BW_FILTER::process_notused(SAMPLE_BUFFER* sbuf) {
 //    }
 }
 
-void EFFECT_BW_FILTER::init_values(void) {
+void EFFECT_BW_FILTER::init_values(void)
+{
 //    for(int j = 0; j < 2;j++) {
 //      sin[j].sample[SAMPLE_BUFFER::ch_left] = 0.0;
 //      sin[j].sample[SAMPLE_BUFFER::ch_right] = 0.0;
@@ -156,15 +178,17 @@ void EFFECT_BW_FILTER::init_values(void) {
 //    }
 }
 
-EFFECT_HIGHPASS::EFFECT_HIGHPASS (CHAIN_OPERATOR::parameter_type cutoff) {
+EFFECT_HIGHPASS::EFFECT_HIGHPASS (CHAIN_OPERATOR::parameter_t cutoff)
+{
   set_parameter(1, cutoff);
 }
 
-void EFFECT_HIGHPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_HIGHPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1: 
     cutOffFreq = value;
-    C = tan(M_PI * cutOffFreq / (CHAIN_OPERATOR::parameter_type)samples_per_second());
+    C = tan(M_PI * cutOffFreq / (CHAIN_OPERATOR::parameter_t)samples_per_second());
     a[0] = 1.0 / (1.0 + sqrt(2.0) * C + C * C);
     a[1] = -2.0 * a[0];
     a[2] = a[0];
@@ -174,15 +198,17 @@ void EFFECT_HIGHPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_type va
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_HIGHPASS::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_HIGHPASS::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
-    return(cutOffFreq);
+    return cutOffFreq;
   }
-  return(0.0);
+  return 0.0;
 }
 
-void EFFECT_ALLPASS_FILTER::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_ALLPASS_FILTER::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1: 
     D = value;
@@ -198,71 +224,78 @@ void EFFECT_ALLPASS_FILTER::set_parameter(int param, CHAIN_OPERATOR::parameter_t
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_ALLPASS_FILTER::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_ALLPASS_FILTER::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
-    return(D);
+    return D;
   case 2: 
-    return(feedback_gain * 100.0);
+    return feedback_gain * 100.0;
   }
-  return(0.0);
+  return 0.0;
 }
 
-void EFFECT_ALLPASS_FILTER::init(SAMPLE_BUFFER* insample) {
+void EFFECT_ALLPASS_FILTER::init(SAMPLE_BUFFER* insample)
+{
   i.init(insample);
 
   set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
 
   inbuf.resize(insample->number_of_channels());
-//  outbuf.resize(insample->number_of_channels());
+  //  outbuf.resize(insample->number_of_channels());
 }
 
-void EFFECT_ALLPASS_FILTER::process(void) {
+void EFFECT_ALLPASS_FILTER::process(void)
+{
   i.begin();
   while(!i.end()) {
     if (inbuf[i.channel()].size() >= D) {
       inbuf[i.channel()].push_back(*i.current());
-//      *i.current() = -feedback_gain * (*i.current()) +
-//	             inbuf[i.channel()].front() +
-//	             feedback_gain * outbuf[i.channel()].front();
-      *i.current() = -feedback_gain * (*i.current()) +
-	             (feedback_gain * inbuf[i.channel()].front() +
-		     *i.current()) * 
-		     (1.0 - feedback_gain * feedback_gain);
-//	             feedback_gain * outbuf[i.channel()].front();
-//      outbuf[i.channel()].push_back(*i.current());
+
+      //      *i.current() = -feedback_gain * (*i.current()) +
+      //	             inbuf[i.channel()].front() +
+      //	             feedback_gain * outbuf[i.channel()].front();
+
+      *i.current() = ecaops_flush_to_zero(-feedback_gain * (*i.current()) +
+					  (feedback_gain * inbuf[i.channel()].front() +
+					   *i.current()) * 
+					  (1.0 - feedback_gain * feedback_gain));
+
+      //      feedback_gain * outbuf[i.channel()].front();
+      //      outbuf[i.channel()].push_back(*i.current());
 
       inbuf[i.channel()].pop_front();
-//      outbuf[i.channel()].pop_front();
+      // outbuf[i.channel()].pop_front();
     } 
     else {
       inbuf[i.channel()].push_back(*i.current());
-      *i.current() = *i.current() * (1.0 - feedback_gain);
-//      outbuf[i.channel()].push_back(*i.current());
+      *i.current() = ecaops_flush_to_zero(*i.current() * (1.0 - feedback_gain));
+      // outbuf[i.channel()].push_back(*i.current());
     }
     i.next();
   }
 }
 
-EFFECT_COMB_FILTER::EFFECT_COMB_FILTER (int delay_in_samples, CHAIN_OPERATOR::parameter_type radius) {
-  set_parameter(1, (CHAIN_OPERATOR::parameter_type)delay_in_samples);
+EFFECT_COMB_FILTER::EFFECT_COMB_FILTER (int delay_in_samples, CHAIN_OPERATOR::parameter_t radius)
+{
+  set_parameter(1, (CHAIN_OPERATOR::parameter_t)delay_in_samples);
   set_parameter(2, radius);
 }
 
-void EFFECT_COMB_FILTER::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_COMB_FILTER::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1: 
     {
       C = value;
-      vector<deque<SAMPLE_SPECS::sample_type> >::iterator p = buffer.begin();
+      std::vector<std::deque<SAMPLE_SPECS::sample_t> >::iterator p = buffer.begin();
       while(p != buffer.end()) {
 	if (p->size() > C) {
 	  p->resize(static_cast<unsigned int>(C));
 	}
 	++p;
-	break;
       }
+      break;
     }
 
   case 2: 
@@ -271,26 +304,28 @@ void EFFECT_COMB_FILTER::set_parameter(int param, CHAIN_OPERATOR::parameter_type
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_COMB_FILTER::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_COMB_FILTER::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
-    return(C);
+    return C;
   case 2: 
-    return(D);
+    return D;
   }
-  return(0.0);
+  return 0.0;
 }
 
-void EFFECT_COMB_FILTER::init(SAMPLE_BUFFER* insample) {
+void EFFECT_COMB_FILTER::init(SAMPLE_BUFFER* insample)
+{
   i.init(insample);
 
   set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
 
   buffer.resize(insample->number_of_channels());
 }
 
-void EFFECT_COMB_FILTER::process(void) {
+void EFFECT_COMB_FILTER::process(void)
+{
   i.begin();
   while(!i.end()) {
     if (buffer[i.channel()].size() >= C) {
@@ -306,16 +341,18 @@ void EFFECT_COMB_FILTER::process(void) {
   }
 }
 
-EFFECT_INVERSE_COMB_FILTER::EFFECT_INVERSE_COMB_FILTER (int delay_in_samples, CHAIN_OPERATOR::parameter_type radius) {
+EFFECT_INVERSE_COMB_FILTER::EFFECT_INVERSE_COMB_FILTER (int delay_in_samples, CHAIN_OPERATOR::parameter_t radius)
+{
   // 
   // delay in number of samples
   // circle radius
   //
-  set_parameter(1, (CHAIN_OPERATOR::parameter_type)delay_in_samples);
+  set_parameter(1, (CHAIN_OPERATOR::parameter_t)delay_in_samples);
   set_parameter(2, radius);
 }
 
-void EFFECT_INVERSE_COMB_FILTER::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_INVERSE_COMB_FILTER::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1: 
     C = value;
@@ -326,27 +363,29 @@ void EFFECT_INVERSE_COMB_FILTER::set_parameter(int param, CHAIN_OPERATOR::parame
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_INVERSE_COMB_FILTER::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_INVERSE_COMB_FILTER::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
-    return(C);
+    return C;
   case 2: 
-    return(D);
+    return D;
   }
-  return(0.0);
+  return 0.0;
 }
 
-void EFFECT_INVERSE_COMB_FILTER::init(SAMPLE_BUFFER* insample) {
+void EFFECT_INVERSE_COMB_FILTER::init(SAMPLE_BUFFER* insample)
+{
   i.init(insample);
 
   set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
 
   buffer.resize(insample->number_of_channels());
-  laskuri.resize(insample->number_of_channels(), parameter_type(0.0));
+  laskuri.resize(insample->number_of_channels(), parameter_t(0.0));
 }
 
-void EFFECT_INVERSE_COMB_FILTER::process(void) {
+void EFFECT_INVERSE_COMB_FILTER::process(void)
+{
   i.begin();
   while(!i.end()) {
     buffer[i.channel()].push_back(*i.current());
@@ -363,11 +402,12 @@ void EFFECT_INVERSE_COMB_FILTER::process(void) {
   }
 }
 
-EFFECT_LOWPASS::EFFECT_LOWPASS (CHAIN_OPERATOR::parameter_type cutoff) {
+EFFECT_LOWPASS::EFFECT_LOWPASS (CHAIN_OPERATOR::parameter_t cutoff)
+{
   set_parameter(1, cutoff);
 }
 
-void EFFECT_LOWPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_LOWPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_t value) {
   switch (param) {
   case 1: 
     set_cutoff(value, samples_per_second());
@@ -375,17 +415,19 @@ void EFFECT_LOWPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_type val
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_LOWPASS::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_LOWPASS::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
-    return(cutOffFreq);
+    return cutOffFreq;
   }
-  return(0.0);
+  return 0.0;
 }
 
-void EFFECT_LOWPASS::set_cutoff(CHAIN_OPERATOR::parameter_type value, long int srate) {
+void EFFECT_LOWPASS::set_cutoff(CHAIN_OPERATOR::parameter_t value, long int srate)
+{
   cutOffFreq = value;
-  C = 1.0 / tan(M_PI * cutOffFreq / (CHAIN_OPERATOR::parameter_type)srate);
+  C = 1.0 / tan(M_PI * cutOffFreq / (CHAIN_OPERATOR::parameter_t)srate);
   a[0] = 1.0 / (1.0 + sqrt(2.0) * C + C * C);
   a[1] = 2.0 * a[0];
   a[2] = a[0];
@@ -393,11 +435,13 @@ void EFFECT_LOWPASS::set_cutoff(CHAIN_OPERATOR::parameter_type value, long int s
   b[1] = (1.0 - sqrt(2.0) * C + C * C) * a[0];
 }
 
-EFFECT_LOWPASS_SIMPLE::EFFECT_LOWPASS_SIMPLE (CHAIN_OPERATOR::parameter_type cutoff) {
+EFFECT_LOWPASS_SIMPLE::EFFECT_LOWPASS_SIMPLE (CHAIN_OPERATOR::parameter_t cutoff)
+{
   set_parameter(1, cutoff);
 }
 
-void EFFECT_LOWPASS_SIMPLE::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_LOWPASS_SIMPLE::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1: 
     cutOffFreq = value;
@@ -407,26 +451,28 @@ void EFFECT_LOWPASS_SIMPLE::set_parameter(int param, CHAIN_OPERATOR::parameter_t
   }
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_LOWPASS_SIMPLE::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_LOWPASS_SIMPLE::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
-    return(cutOffFreq);
+    return cutOffFreq;
   }
-  return(0.0);
+  return 0.0;
 }
 
-void EFFECT_LOWPASS_SIMPLE::init(SAMPLE_BUFFER *insample) {
+void EFFECT_LOWPASS_SIMPLE::init(SAMPLE_BUFFER *insample)
+{
   i.init(insample);
 
   set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
 
   outhist.resize(insample->number_of_channels());
   tempin.resize(insample->number_of_channels());
   temphist.resize(insample->number_of_channels());
 }
 
-void EFFECT_LOWPASS_SIMPLE::process(void) {
+void EFFECT_LOWPASS_SIMPLE::process(void)
+{
   i.begin();
   while(!i.end()) {
     tempin[i.channel()] = *i.current();
@@ -436,21 +482,25 @@ void EFFECT_LOWPASS_SIMPLE::process(void) {
     tempin[i.channel()] *= A * 0.5;
     temphist[i.channel()] *= B * 0.5;
 
-    *i.current() = tempin[i.channel()] + temphist[i.channel()];
+    *i.current() = ecaops_flush_to_zero(tempin[i.channel()] + temphist[i.channel()]);
 
     i.next();
   }
 }
 
-EFFECT_RESONANT_BANDPASS::EFFECT_RESONANT_BANDPASS (CHAIN_OPERATOR::parameter_type centerf,
-						    CHAIN_OPERATOR::parameter_type w) 
+EFFECT_RESONANT_BANDPASS::EFFECT_RESONANT_BANDPASS (CHAIN_OPERATOR::parameter_t centerf,
+						    CHAIN_OPERATOR::parameter_t w) 
 {
+  /* to avoid accessing uninitialized data */
+  width = 1;
+  center = 1;
 
   set_parameter(1, centerf);
   set_parameter(2, w);
 }
 
-void EFFECT_RESONANT_BANDPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_RESONANT_BANDPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1: 
     center = value;
@@ -460,43 +510,45 @@ void EFFECT_RESONANT_BANDPASS::set_parameter(int param, CHAIN_OPERATOR::paramete
     else width = center / 2.0;
     break;
   }
-  //  R = 1.0 - M_PI * width / (CHAIN_OPERATOR::parameter_type)samples_per_second();
-  //  R = 1.0 - ((width / (CHAIN_OPERATOR::parameter_type)samples_per_second()) / 2.0);
-  R = 1.0 - M_PI * (width / (CHAIN_OPERATOR::parameter_type)samples_per_second());
+  //  R = 1.0 - M_PI * width / (CHAIN_OPERATOR::parameter_t)samples_per_second();
+  //  R = 1.0 - ((width / (CHAIN_OPERATOR::parameter_t)samples_per_second()) / 2.0);
+  R = 1.0 - M_PI * (width / (CHAIN_OPERATOR::parameter_t)samples_per_second());
   c = R * R;
   pole_angle = (((2.0 * R) / (1.0 + c)) * cos((center / 
-					       (CHAIN_OPERATOR::parameter_type)samples_per_second() * 2.0 * M_PI)));
+					       (CHAIN_OPERATOR::parameter_t)samples_per_second() * 2.0 * M_PI)));
   pole_angle = acos(pole_angle);
   a = (1.0 - c) * sin(pole_angle);
   b = 2.0 * R * cos(pole_angle);
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_RESONANT_BANDPASS::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_RESONANT_BANDPASS::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
-    return(center);
+    return center;
   case 2: 
-    return(width);
+    return width;
   }
-  return(0.0);
+  return 0.0;
 }
 
-void EFFECT_RESONANT_BANDPASS::init(SAMPLE_BUFFER* insample) {
+void EFFECT_RESONANT_BANDPASS::init(SAMPLE_BUFFER* insample)
+{
   i.init(insample);
 
   set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
 
   outhist1.resize(insample->number_of_channels());
   outhist2.resize(insample->number_of_channels());
 }
 
-void EFFECT_RESONANT_BANDPASS::process(void) {
+void EFFECT_RESONANT_BANDPASS::process(void)
+{
   i.begin();
   while(!i.end()) {
-    *i.current() = a * (*i.current()) +
-                   b * outhist1[i.channel()] -
-                   c * outhist2[i.channel()];
+    *i.current() = ecaops_flush_to_zero(a * (*i.current()) +
+					b * outhist1[i.channel()] -
+					c * outhist2[i.channel()]);
   
     outhist2[i.channel()] = outhist1[i.channel()];
     outhist1[i.channel()] = *i.current();
@@ -505,8 +557,8 @@ void EFFECT_RESONANT_BANDPASS::process(void) {
   }
 }
 
-EFFECT_RESONANT_LOWPASS::EFFECT_RESONANT_LOWPASS (CHAIN_OPERATOR::parameter_type co, CHAIN_OPERATOR::parameter_type
-						  res, CHAIN_OPERATOR::parameter_type g) 
+EFFECT_RESONANT_LOWPASS::EFFECT_RESONANT_LOWPASS (CHAIN_OPERATOR::parameter_t co, CHAIN_OPERATOR::parameter_t
+						  res, CHAIN_OPERATOR::parameter_t g) 
   : ProtoCoef(2), Coef(2)
 {
   cutoff = co;
@@ -540,7 +592,8 @@ EFFECT_RESONANT_LOWPASS::EFFECT_RESONANT_LOWPASS (CHAIN_OPERATOR::parameter_type
   szxform(1);
 }
 
-void EFFECT_RESONANT_LOWPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) {
+void EFFECT_RESONANT_LOWPASS::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
+{
   switch (param) {
   case 1: 
     cutoff = value;
@@ -555,19 +608,21 @@ void EFFECT_RESONANT_LOWPASS::set_parameter(int param, CHAIN_OPERATOR::parameter
   refresh_values();
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_RESONANT_LOWPASS::get_parameter(int param) const {
+CHAIN_OPERATOR::parameter_t EFFECT_RESONANT_LOWPASS::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
-    return(cutoff);
+    return cutoff;
   case 2: 
-    return(Q);
+    return Q;
   case 3: 
-    return(gain_orig);
+    return gain_orig;
   }
-  return(0.0);
+  return 0.0;
 }
 
-void EFFECT_RESONANT_LOWPASS::refresh_values(void) {
+void EFFECT_RESONANT_LOWPASS::refresh_values(void)
+{
   if (cutoff == 0.0) cutoff = 0.1;
     
   gain = gain_orig;
@@ -590,8 +645,9 @@ void EFFECT_RESONANT_LOWPASS::refresh_values(void) {
   szxform(1);
 }
 
-void EFFECT_RESONANT_LOWPASS::szxform(int section) {
-  wp = 2.0 * (CHAIN_OPERATOR::parameter_type)samples_per_second() * tan(pi * cutoff / (CHAIN_OPERATOR::parameter_type)samples_per_second());
+void EFFECT_RESONANT_LOWPASS::szxform(int section)
+{
+  wp = 2.0 * (CHAIN_OPERATOR::parameter_t)samples_per_second() * tan(pi * cutoff / (CHAIN_OPERATOR::parameter_t)samples_per_second());
 
   // ---
   // a0 and b0 are presumed to be 1, so...
@@ -604,12 +660,12 @@ void EFFECT_RESONANT_LOWPASS::szxform(int section) {
 
   // ---
   // alpha (Numerator in s-domain)
-  ad = 4.0 * ProtoCoef[section].a2 * (CHAIN_OPERATOR::parameter_type)samples_per_second() * (CHAIN_OPERATOR::parameter_type)samples_per_second() + 2.0 * ProtoCoef[section].a1
-    * (CHAIN_OPERATOR::parameter_type)samples_per_second() + ProtoCoef[section].a0;
+  ad = 4.0 * ProtoCoef[section].a2 * (CHAIN_OPERATOR::parameter_t)samples_per_second() * (CHAIN_OPERATOR::parameter_t)samples_per_second() + 2.0 * ProtoCoef[section].a1
+    * (CHAIN_OPERATOR::parameter_t)samples_per_second() + ProtoCoef[section].a0;
   // ---
   // beta (Denominator in s-domain)
-  bd = 4.0 * ProtoCoef[section].b2 * (CHAIN_OPERATOR::parameter_type)samples_per_second() * (CHAIN_OPERATOR::parameter_type)samples_per_second() + 2.0 * ProtoCoef[section].b1
-    * (CHAIN_OPERATOR::parameter_type)samples_per_second() + ProtoCoef[section].b0;
+  bd = 4.0 * ProtoCoef[section].b2 * (CHAIN_OPERATOR::parameter_t)samples_per_second() * (CHAIN_OPERATOR::parameter_t)samples_per_second() + 2.0 * ProtoCoef[section].b1
+    * (CHAIN_OPERATOR::parameter_t)samples_per_second() + ProtoCoef[section].b0;
 
   // ---
   /* update gain constant for this section */
@@ -618,31 +674,31 @@ void EFFECT_RESONANT_LOWPASS::szxform(int section) {
   // ---
   // Denominator
   Coef[section].A = (2.0 * ProtoCoef[section].b0 - 8.0 * ProtoCoef[section].b2
-		     * (CHAIN_OPERATOR::parameter_type)samples_per_second() * (CHAIN_OPERATOR::parameter_type)samples_per_second()) / bd;
+		     * (CHAIN_OPERATOR::parameter_t)samples_per_second() * (CHAIN_OPERATOR::parameter_t)samples_per_second()) / bd;
   // ---
   // beta1
-  Coef[section].B = (4.0 * ProtoCoef[section].b2 * (CHAIN_OPERATOR::parameter_type)samples_per_second() * (CHAIN_OPERATOR::parameter_type)samples_per_second() - 2.0 * ProtoCoef[section].b1
-		     * (CHAIN_OPERATOR::parameter_type)samples_per_second() + ProtoCoef[section].b0) / bd;
+  Coef[section].B = (4.0 * ProtoCoef[section].b2 * (CHAIN_OPERATOR::parameter_t)samples_per_second() * (CHAIN_OPERATOR::parameter_t)samples_per_second() - 2.0 * ProtoCoef[section].b1
+		     * (CHAIN_OPERATOR::parameter_t)samples_per_second() + ProtoCoef[section].b0) / bd;
   // ---
   // beta2
 
   // ---
   // Nominator
   Coef[section].C = (2.0 * ProtoCoef[section].a0 - 8.0 * ProtoCoef[section].a2
-		     * (CHAIN_OPERATOR::parameter_type)samples_per_second() * (CHAIN_OPERATOR::parameter_type)samples_per_second()) / ad;
+		     * (CHAIN_OPERATOR::parameter_t)samples_per_second() * (CHAIN_OPERATOR::parameter_t)samples_per_second()) / ad;
   // ---
   // alpha1
-  Coef[section].D = (4.0 * ProtoCoef[section].a2 * (CHAIN_OPERATOR::parameter_type)samples_per_second() * (CHAIN_OPERATOR::parameter_type)samples_per_second() - 2.0
-		     * ProtoCoef[section].a1 * (CHAIN_OPERATOR::parameter_type)samples_per_second() + ProtoCoef[section].a0) / ad;
+  Coef[section].D = (4.0 * ProtoCoef[section].a2 * (CHAIN_OPERATOR::parameter_t)samples_per_second() * (CHAIN_OPERATOR::parameter_t)samples_per_second() - 2.0
+		     * ProtoCoef[section].a1 * (CHAIN_OPERATOR::parameter_t)samples_per_second() + ProtoCoef[section].a0) / ad;
   // ---
   // alpha2
 }
 
-void EFFECT_RESONANT_LOWPASS::init(SAMPLE_BUFFER* insample) {
+void EFFECT_RESONANT_LOWPASS::init(SAMPLE_BUFFER* insample)
+{
   i.init(insample);
 
   set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
 
   outhist0.resize(insample->number_of_channels());
   outhist1.resize(insample->number_of_channels());
@@ -653,7 +709,8 @@ void EFFECT_RESONANT_LOWPASS::init(SAMPLE_BUFFER* insample) {
   newhist1.resize(insample->number_of_channels());
 }
 
-void EFFECT_RESONANT_LOWPASS::process(void) {
+void EFFECT_RESONANT_LOWPASS::process(void)
+{
   i.begin();
   while(!i.end()) {
     *i.current() = (*i.current()) * gain;
@@ -663,7 +720,7 @@ void EFFECT_RESONANT_LOWPASS::process(void) {
     
     // poles:
     *i.current() =  (*i.current()) - outhist0[i.channel()] * Coef[0].A;
-    newhist0[i.channel()] = (*i.current()) - outhist1[i.channel()] * Coef[0].B;
+    newhist0[i.channel()] = ecaops_flush_to_zero((*i.current()) - outhist1[i.channel()] * Coef[0].B);
         
     // zeros:
     *i.current() = newhist0[i.channel()] + outhist0[i.channel()] * Coef[0].C;
@@ -677,7 +734,7 @@ void EFFECT_RESONANT_LOWPASS::process(void) {
     
     // poles:
     *i.current() =  (*i.current()) - outhist2[i.channel()] * Coef[1].A;
-    newhist1[i.channel()] = (*i.current()) - outhist3[i.channel()] * Coef[1].B;
+    newhist1[i.channel()] = ecaops_flush_to_zero((*i.current()) - outhist3[i.channel()] * Coef[1].B);
        
     // zeros:
     *i.current() = newhist1[i.channel()] + outhist2[i.channel()] * Coef[1].C;
@@ -696,7 +753,6 @@ void EFFECT_RESONANT_LOWPASS::process(void) {
 //  {
 //    outhist = x.outhist;
 //    newhist = x.newhist;
-
 //    for(vector<BIQUAD>::size_type p = 0; p != x.ProtoCoef.size(); p++) {
 //      ProtoCoef[p].a0 = x.ProtoCoef[p].a0;
 //      ProtoCoef[p].a1 = x.ProtoCoef[p].a1;
@@ -706,7 +762,6 @@ void EFFECT_RESONANT_LOWPASS::process(void) {
 //      ProtoCoef[p].b2 = x.ProtoCoef[p].b2;
 //      ++p;
 //    }
-
 //    for(vector<BIQUAD>::size_type p = 0; p != x.Coef.size(); p++) {
 //      Coef[p].A = x.Coef[p].A;
 //      Coef[p].B = x.Coef[p].B;
@@ -714,27 +769,29 @@ void EFFECT_RESONANT_LOWPASS::process(void) {
 //      Coef[p].D = x.Coef[p].D;
 //      ++p;
 //    }
-
 //    cutoff = x.cutoff;
 //    Q = x.Q;
 //    gain = x.gain;
 //    gain_orig = x.gain_orig;
 //    pi = x.pi;
 //    laskuri = x.laskuri;
-  
 //    ad = x.ad;
 //    bd = x.bd;
 //    wp = x.wp;
 //  }
 
-EFFECT_RESONATOR::EFFECT_RESONATOR (CHAIN_OPERATOR::parameter_type centerf, CHAIN_OPERATOR::parameter_type w) 
+EFFECT_RESONATOR::EFFECT_RESONATOR (CHAIN_OPERATOR::parameter_t centerf, CHAIN_OPERATOR::parameter_t w) 
   : cona(1), conb(2) 
 {
+  /* to avoid accessing uninitialized data */
+  width = 1;
+  center = 1;
+
   set_parameter(1, centerf);
   set_parameter(2, w);
 }
 
-void EFFECT_RESONATOR::set_parameter(int param, CHAIN_OPERATOR::parameter_type value) 
+void EFFECT_RESONATOR::set_parameter(int param, CHAIN_OPERATOR::parameter_t value)
 {
   switch (param) {
   case 1: 
@@ -745,32 +802,32 @@ void EFFECT_RESONATOR::set_parameter(int param, CHAIN_OPERATOR::parameter_type v
     else width = center / 2;
     break;
   }
-  conb[1] = exp(-(2 * M_PI) * (width / (CHAIN_OPERATOR::parameter_type)samples_per_second()));
-  conb[0] = (-4.0 * conb[1]) / (1.0 + conb[1]) * cos(2 * M_PI * (center / (CHAIN_OPERATOR::parameter_type)samples_per_second()));
+  conb[1] = exp(-(2 * M_PI) * (width / (CHAIN_OPERATOR::parameter_t)samples_per_second()));
+  conb[0] = (-4.0 * conb[1]) / (1.0 + conb[1]) * cos(2 * M_PI * (center / (CHAIN_OPERATOR::parameter_t)samples_per_second()));
   cona[0] = (1.0 - conb[1]) * sqrt(1.0 - (conb[0] * conb[0]) / (4.0 * conb[1]));
 }
 
-CHAIN_OPERATOR::parameter_type EFFECT_RESONATOR::get_parameter(int param) const { 
+CHAIN_OPERATOR::parameter_t EFFECT_RESONATOR::get_parameter(int param) const { 
   switch (param) {
   case 1: 
-    return(center);
+    return center;
   case 2: 
-    return(width);
+    return width;
   }
-  return(0.0);
+  return 0.0;
 }
 
 void EFFECT_RESONATOR::init(SAMPLE_BUFFER* insample) {
   i.init(insample);
 
   set_channels(insample->number_of_channels());
-  set_samples_per_second(insample->sample_rate());
 
   saout0.resize(insample->number_of_channels());
   saout1.resize(insample->number_of_channels());
 }
 
-void EFFECT_RESONATOR::process(void) {
+void EFFECT_RESONATOR::process(void)
+{
   i.begin();
   while(!i.end()) {
     *i.current() = cona[0] * (*i.current()) -
@@ -778,7 +835,7 @@ void EFFECT_RESONATOR::process(void) {
 		   conb[1] * saout1[i.channel()];
     
     saout1[i.channel()] = saout0[i.channel()];
-    saout0[i.channel()] = *i.current();
+    saout0[i.channel()] = ecaops_flush_to_zero(*i.current());
 				 
     i.next();
   }
