@@ -1,8 +1,12 @@
 #ifndef INCLUDED_AUDIO_GATE_H
 #define INCLUDED_AUDIO_GATE_H
 
+#include <string>
+
 #include "eca-chainop.h"
-#include "samplebuffer.h"
+#include "eca-samplerate-aware.h"
+
+class SAMPLE_BUFFER;
 
 /**
  * Interface to gate effects. Gate processes sample data, but
@@ -27,8 +31,8 @@ public:
   void process(void);
   virtual void analyze(SAMPLE_BUFFER* sbuf) = 0;
 
-  virtual GATE_BASE* clone(void) = 0;   
-  virtual ~GATE_BASE(void) { }
+  virtual GATE_BASE* clone(void) const = 0;   
+  virtual ~GATE_BASE(void);
 
   GATE_BASE(void) { close_gate(); }
 };
@@ -39,31 +43,33 @@ public:
  * 'duration' seconds. If 'duration' is 0, gate will stay open
  * forever.
  */
-class TIME_CROP_GATE : public GATE_BASE {
+class TIME_CROP_GATE : public GATE_BASE,
+		       public ECA_SAMPLERATE_AWARE
+{
 
 public:
 
   // Functions returning info about effect and its parameters.
   // ---
-  parameter_type get_parameter(int param) const;
-  void set_parameter(int param, parameter_type value);
+  parameter_t get_parameter(int param) const;
+  void set_parameter(int param, parameter_t value);
 
-  string name(void) const { return("Time crop gate"); }
+  std::string name(void) const { return("Time crop gate"); }
 
-  string parameter_names(void) const { return("open-at-sec,duration-sec"); }
+  std::string parameter_names(void) const { return("open-at-sec,duration-sec"); }
 
   void analyze(SAMPLE_BUFFER* insample);
 
-  TIME_CROP_GATE* clone(void)  { return new TIME_CROP_GATE(*this); }
-  TIME_CROP_GATE* new_expr(void)  { return new TIME_CROP_GATE(); }
-  TIME_CROP_GATE (parameter_type open_at, parameter_type duration);
+  TIME_CROP_GATE* clone(void) const { return new TIME_CROP_GATE(*this); }
+  TIME_CROP_GATE* new_expr(void) const { return new TIME_CROP_GATE(); }
+  TIME_CROP_GATE (parameter_t open_at, parameter_t duration);
   TIME_CROP_GATE (void) : curtime(0.0) {
     close_gate();
   }
 
 private:
 
-  parameter_type curtime, btime, etime; 
+  parameter_t curtime, btime, etime; 
 };
 
 /**
@@ -82,25 +88,25 @@ public:
 
   // Functions returning info about effect and its parameters.
   // ---
-  parameter_type get_parameter(int param) const;
-  void set_parameter(int param, parameter_type value);
+  parameter_t get_parameter(int param) const;
+  void set_parameter(int param, parameter_t value);
 
-  string name(void) const { return("Threshold gate"); }
+  std::string name(void) const { return("Threshold gate"); }
 
-  string parameter_names(void) const { return("threshold-openlevel-%,threshold-closelevel-%,rms-enabled"); }
+  std::string parameter_names(void) const { return("threshold-openlevel-%,threshold-closelevel-%,rms-enabled"); }
 
   void analyze(SAMPLE_BUFFER* insample);
 
-  THRESHOLD_GATE* clone(void)  { return new THRESHOLD_GATE(*this); }
-  THRESHOLD_GATE* new_expr(void)  { return new THRESHOLD_GATE(); }
-  THRESHOLD_GATE (parameter_type threshold_openlevel, parameter_type
+  THRESHOLD_GATE* clone(void) const { return new THRESHOLD_GATE(*this); }
+  THRESHOLD_GATE* new_expr(void) const { return new THRESHOLD_GATE(); }
+  THRESHOLD_GATE (parameter_t threshold_openlevel, parameter_t
 		  threshold_closelevel,  bool use_rms = false);
   THRESHOLD_GATE (void) 
     : rms(false), is_opened(false), is_closed(false) { }
   
 private:
   
-  parameter_type openlevel, closelevel, avolume;
+  parameter_t openlevel, closelevel, avolume;
   bool rms;
   bool is_opened, is_closed;
 };
