@@ -2,13 +2,14 @@
 #define INCLUDED_AUDIOIO_MIKMOD_H
 
 #include <string>
-#include "audioio-types.h"
-#include "samplebuffer.h"
+#include <cstdio>
+#include "audioio-buffered.h"
 #include "audioio-forked-stream.h"
 
 /**
- * Interface for module players that support i/o using standard streams. 
- * Defaults to MikMod.
+ * Interface to module players such as MikMod that support 
+ * UNIX pipe i/o.
+ *
  * @author Kai Vehmanen
  */
 class MIKMOD_INTERFACE : public AUDIO_IO_BUFFERED,
@@ -16,32 +17,28 @@ class MIKMOD_INTERFACE : public AUDIO_IO_BUFFERED,
 
  private:
   
-  static string default_mikmod_cmd;
+  static std::string default_mikmod_cmd;
 
  public:
 
-  static void set_mikmod_cmd(const string& value);
-
- private:
-
-  bool triggered_rep;
-  bool finished_rep;
-  long int bytes_read_rep;
-  int fd_rep;
-  FILE* f1_rep;
-  
-  void seek_position_in_samples(long pos);
-  MIKMOD_INTERFACE& operator=(const MIKMOD_INTERFACE& x) { return *this; }
-
-  void fork_mikmod(void);
-  void kill_mikmod(void);
-  
+  static void set_mikmod_cmd(const std::string& value);
+ 
  public:
 
-  virtual string name(void) const { return("MikMod tracker module"); }
-  virtual string description(void) const { return("Interface for module players that support i/o using standard streams."); }
+  MIKMOD_INTERFACE (const std::string& name = "");
+  virtual ~MIKMOD_INTERFACE(void);
+    
+  virtual MIKMOD_INTERFACE* clone(void) const { return new MIKMOD_INTERFACE(*this); }
+  virtual MIKMOD_INTERFACE* new_expr(void) const { return new MIKMOD_INTERFACE(); }
+
+  virtual std::string name(void) const { return("MikMod tracker module"); }
+  virtual std::string description(void) const { return("Interface to module players that support UNIX pipe i/o."); }
+
+  virtual void set_parameter(int param, string value);
+  virtual string get_parameter(int param) const;
 
   virtual int supported_io_modes(void) const { return(io_read); }
+  virtual string parameter_names(void) const { return("filename,opt_filename"); }
   virtual bool supports_seeking(void) const { return(false); }
 
   virtual void open(void) throw (AUDIO_IO::SETUP_ERROR &);
@@ -52,12 +49,22 @@ class MIKMOD_INTERFACE : public AUDIO_IO_BUFFERED,
 
   virtual bool finished(void) const { return(finished_rep); }
   virtual void seek_position(void);
- 
-  MIKMOD_INTERFACE (const string& name = "");
-  ~MIKMOD_INTERFACE(void);
-    
-  MIKMOD_INTERFACE* clone(void) { return new MIKMOD_INTERFACE(*this); }
-  MIKMOD_INTERFACE* new_expr(void) { return new MIKMOD_INTERFACE(); }
+
+
+ private:
+
+  std::string opt_filename_rep;
+  bool triggered_rep;
+  bool finished_rep;
+  long int bytes_read_rep;
+  int filedes_rep;
+  FILE* f1_rep;
+  
+  void seek_position_in_samples(long pos);
+  MIKMOD_INTERFACE& operator=(const MIKMOD_INTERFACE& x) { return *this; }
+
+  void fork_mikmod(void);
+  void kill_mikmod(void);
 };
 
 #endif
