@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // audioio-timidity.cpp: Interface class for Timidity++ input.
-// Copyright (C) 2000,2002 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 2000,2002,2004-2005 Kai Vehmanen
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -58,8 +58,9 @@ void TIMIDITY_INTERFACE::open(void) throw (AUDIO_IO::SETUP_ERROR &)
     }
   }
   
-  /* s16 samples, 2 channels, srate configurable */
-  set_sample_format(ECA_AUDIO_FORMAT::sfmt_s16_le);
+  /* decoder supports: s16 samples, 2 channels, srate configurable 
+     no support for: endianess (use native)  */
+  set_sample_format(ECA_AUDIO_FORMAT::sfmt_s16);
   set_channels(2);
 
   triggered_rep = false;
@@ -89,7 +90,7 @@ long int TIMIDITY_INTERFACE::read_samples(void* target_buffer,
   bytes_read_rep = std::fread(target_buffer, 1, frame_size() * samples, f1_rep);
   if (bytes_read_rep < samples * frame_size() || bytes_read_rep == 0) {
     if (position_in_samples() == 0) 
-      ECA_LOG_MSG(ECA_LOGGER::info, "(audioio-timidity) Can't start process \"" + TIMIDITY_INTERFACE::default_timidity_cmd + "\". Please check your ~/.ecasoundrc.");
+      ECA_LOG_MSG(ECA_LOGGER::info, "(audioio-timidity) Can't start process \"" + TIMIDITY_INTERFACE::default_timidity_cmd + "\". Please check your ~/.ecasound/ecasoundrc.");
     finished_rep = true;
     triggered_rep = false;
   }
@@ -123,8 +124,10 @@ void TIMIDITY_INTERFACE::fork_timidity(void)
   set_fork_sample_rate(samples_per_second());
   fork_child_for_read();
   if (child_fork_succeeded() == true) {
-    fd_rep = file_descriptor();
-    f1_rep = fdopen(fd_rep, "r"); /* not part of <cstdio> */
+    /* NOTE: the file description will be closed by 
+     *       AUDIO_IO_FORKED_STREAM::clean_child() */
+    filedes_rep = file_descriptor();
+    f1_rep = fdopen(filedes_rep, "r"); /* not part of <cstdio> */
     if (f1_rep == 0) {
       finished_rep = true;
       triggered_rep = false;
