@@ -1,8 +1,6 @@
 #ifndef INCLUDED_DYNAMIC_OBJECT_H
 #define INCLUDED_DYNAMIC_OBJECT_H
 
-#include <string>
-
 #include "dynamic-parameters.h"
 #include "eca-object.h"
 
@@ -10,6 +8,9 @@
  * Virtual class for objects supporting dynamic parameter
  * control.
  *
+ * Related design patterns:
+ *     - Factory Method / Virtual Constructor (GoF107)
+ *
  * @author Kai Vehmanen
  */
 template<class T>
@@ -18,19 +19,19 @@ class DYNAMIC_OBJECT : public DYNAMIC_PARAMETERS<T>,
 
  public:
 
+  virtual ~DYNAMIC_OBJECT (void) { }
+
   /**
    * Virtual method that clones the current object and returns 
    * a pointer to it. This must be implemented by all subclasses!
    */
-  virtual DYNAMIC_OBJECT<T>* clone(void) = 0;
+  virtual DYNAMIC_OBJECT<T>* clone(void) const = 0;
 
   /**
    * Virtual method that creates a new object of current type.
    * This must be implemented by all subclasses!
    */
-  virtual DYNAMIC_OBJECT<T>* new_expr(void) = 0;
-
-  virtual ~DYNAMIC_OBJECT (void) { }
+  virtual DYNAMIC_OBJECT<T>* new_expr(void) const = 0;
 };
 
 #endif
