@@ -1,6 +1,9 @@
 // ------------------------------------------------------------------------
 // eca-chain.cpp: Class representing an abstract audio signal chain.
-// Copyright (C) 1999-2002 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 1999-2004 Kai Vehmanen
+//
+// Attributes:
+//     eca-style-version: 3
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -58,7 +61,7 @@
 
 CHAIN::CHAIN (void)
 {
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(chain) constuctor: CHAIN");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "constuctor: CHAIN");
   muted_rep = false;
   sfx_rep = false;
   initialized_rep = false;
@@ -76,11 +79,14 @@ CHAIN::CHAIN (void)
 
 CHAIN::~CHAIN (void)
 {
-  ECA_LOG_MSG(ECA_LOGGER::system_objects,"CHAIN destructor!");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "CHAIN destructor!");
 
   for(std::vector<CHAIN_OPERATOR*>::iterator p = chainops_rep.begin(); p !=
 	chainops_rep.end(); p++) {
-    ECA_LOG_MSG(ECA_LOGGER::info, (*p)->status());
+    string tmp = (*p)->status();
+    if (tmp.size() > 0) {
+      ECA_LOG_MSG(ECA_LOGGER::info, tmp);
+    }
     delete *p;
   }
 
@@ -97,10 +103,9 @@ bool CHAIN::is_valid(void) const
 {
   if (input_id_rep == -1 ||
       output_id_rep == -1) {
-    ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chain) Chain \"" + name() + "\" not valid.");
-    return(false);
+    return false;
   }
-  return(true);
+  return true;
 }
 
 /**
@@ -225,7 +230,7 @@ string CHAIN::chain_operator_name(void) const
   // --------
   DBC_REQUIRE(selected_chain_operator() > 0);
   // --------
-  return(selected_chainop_repp->name());
+  return selected_chainop_repp->name();
 }
 
 /**
@@ -241,7 +246,7 @@ string CHAIN::chain_operator_parameter_name(void) const
   DBC_REQUIRE(selected_chain_operator() > 0);
   DBC_REQUIRE(selected_chain_operator_parameter() > 0);
   // --------
-  return(selected_chainop_repp->get_parameter_name(selected_chain_operator_parameter()));
+  return selected_chainop_repp->get_parameter_name(selected_chain_operator_parameter());
 }
 
 /**
@@ -256,7 +261,7 @@ int CHAIN::number_of_chain_operator_parameters(void) const
   // --------
   DBC_REQUIRE(selected_chain_operator() > 0);
   // --------
-  return(selected_chainop_repp->number_of_params());
+  return selected_chainop_repp->number_of_params();
 }
 
 /**
@@ -270,7 +275,7 @@ string CHAIN::controller_name(void) const
   // --------
   DBC_REQUIRE(selected_controller() > 0);
   // --------
-  return(selected_controller_repp->name());
+  return selected_controller_repp->name();
 }
 
 /**
@@ -307,7 +312,7 @@ CHAIN_OPERATOR::parameter_t CHAIN::get_parameter(void) const
   DBC_REQUIRE(selected_chain_operator_parameter() > 0);
   DBC_REQUIRE(selected_chain_operator() != 0);
   // --------
-  return(selected_chainop_repp->get_parameter(selected_chainop_parameter_rep));
+  return selected_chainop_repp->get_parameter(selected_chainop_parameter_rep);
 }
 
 /**
@@ -327,7 +332,7 @@ void CHAIN::add_controller(GENERIC_CONTROLLER* gcontroller)
   gcontroller->set_samples_per_second(samples_per_second());
 #ifndef ECA_DISABLE_EFFECTS
   gcontroller->assign_target(selected_dynobj_repp);
-  ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-chain) " + gcontroller->status());
+  ECA_LOG_MSG(ECA_LOGGER::user_objects, gcontroller->status());
 #endif
   gcontrollers_rep.push_back(gcontroller);
   selected_controller_repp = gcontroller;
@@ -547,14 +552,14 @@ void CHAIN::init(SAMPLE_BUFFER* sbuf, int in_channels, int out_channels)
   initialized_rep = true;
 
   ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		"(chain) Initialized chain " +
-		name() + 
-		" with " +
-		kvu_numtostr(chainops_rep.size()) +
-		" chainops and " +
-		kvu_numtostr(gcontrollers_rep.size()) +
-		" gcontrollers. Sbuf points to " +
-		kvu_numtostr(reinterpret_cast<long int>(audioslot_repp)) + ".");
+	      "Initialized chain " +
+	      name() + 
+	      " with " +
+	      kvu_numtostr(chainops_rep.size()) +
+	      " chainops and " +
+	      kvu_numtostr(gcontrollers_rep.size()) +
+	      " gcontrollers. Sbuf points to " +
+	      kvu_numtostr(reinterpret_cast<long int>(audioslot_repp)) + ".");
   
   // --------
   DBC_ENSURE(is_initialized() == true);
@@ -648,13 +653,19 @@ string CHAIN::to_string(void) const
     fpreset = 0;
     fpreset = dynamic_cast<FILE_PRESET*>(chainops_rep[q]);
     if (fpreset != 0) {
-      t << "-pf:" << fpreset->filename() << " ";
+      t << "-pf:" << fpreset->filename();
+      if (fpreset->number_of_params() > 0) t << ",";
+      t << operator_parameters_to_string(fpreset);
+      t << " ";
     }
     else {
       gpreset = 0;
       gpreset = dynamic_cast<GLOBAL_PRESET*>(chainops_rep[q]);
       if (gpreset != 0) {
-	t << "-pn:" << gpreset->name() << " ";
+	t << "-pn:" << gpreset->name();
+	if (gpreset->number_of_params() > 0) t << ",";
+	t << operator_parameters_to_string(gpreset);
+	t << " ";
       }
       else {
         t << chain_operator_to_string(chainops_rep[q]) << " ";
@@ -667,6 +678,10 @@ string CHAIN::to_string(void) const
   return(t.to_string());
 }
 
+/**
+ * Makes an EOS-compatible option describing the current state
+ * of chain operator 'gctrl'.
+ */
 string CHAIN::chain_operator_to_string(CHAIN_OPERATOR* chainop) const
 {
   MESSAGE_ITEM t;
@@ -689,9 +704,9 @@ string CHAIN::chain_operator_to_string(CHAIN_OPERATOR* chainop) const
     }
     if (idstring.size() == 0) {
       ECA_LOG_MSG(ECA_LOGGER::errors,
-		  "(eca-chain) Unable to save chain operator \"" +
+		  "Unable to save chain operator \"" +
 		  chainop->name() + "\".");
-      return(t.to_string());
+      return t.to_string();
     }
      
     t << "-" << idstring;
@@ -700,10 +715,7 @@ string CHAIN::chain_operator_to_string(CHAIN_OPERATOR* chainop) const
 #endif
   // --<
 
-  for(int n = 0; n < chainop->number_of_params(); n++) {
-    t << chainop->get_parameter(n + 1);
-    if (n + 1 < chainop->number_of_params()) t << ",";
-  }
+  t << operator_parameters_to_string(chainop);
 
   std::vector<GENERIC_CONTROLLER*>::size_type p = 0;
   while (p < gcontrollers_rep.size()) {
@@ -713,9 +725,13 @@ string CHAIN::chain_operator_to_string(CHAIN_OPERATOR* chainop) const
     ++p;
   } 
 
-  return(t.to_string());
+  return t.to_string();
 }
 
+/**
+ * Makes an EOS-compatible option describing the current state
+ * of controller object 'gctrl'.
+ */
 string CHAIN::controller_to_string(GENERIC_CONTROLLER* gctrl) const
 {
   MESSAGE_ITEM t;
@@ -724,17 +740,14 @@ string CHAIN::controller_to_string(GENERIC_CONTROLLER* gctrl) const
 
   if (idstring.size() == 0) {
     ECA_LOG_MSG(ECA_LOGGER::errors, 
-		"(eca-chain) Unable to save controller \"" +
+		"Unable to save controller \"" +
 		gctrl->name() + "\".");
     return(t.to_string());
   }
 
   t << "-" << idstring;
   t << ":";
-  for(int n = 0; n < gctrl->number_of_params(); n++) {
-    t << gctrl->get_parameter(n + 1);
-    if (n + 1 < gctrl->number_of_params()) t << ",";
-  }
+  t << operator_parameters_to_string(gctrl);
 
   std::vector<GENERIC_CONTROLLER*>::size_type p = 0;
   while (p < gcontrollers_rep.size()) {
@@ -744,7 +757,23 @@ string CHAIN::controller_to_string(GENERIC_CONTROLLER* gctrl) const
     ++p;
   } 
 
-  return(t.to_string());
+  return t.to_string();
+}
+
+/**
+ * Makes an EOS-compatible, comma-separated list of parameter 
+ * values for chain operator 'chainop'.
+ */
+string CHAIN::operator_parameters_to_string(const OPERATOR* chainop) const
+{
+  MESSAGE_ITEM t;
+  
+  for(int n = 0; n < chainop->number_of_params(); n++) {
+    t << chainop->get_parameter(n + 1);
+    if (n + 1 < chainop->number_of_params()) t << ",";
+  }
+
+  return t.to_string();
 }
 
 /**
@@ -757,7 +786,7 @@ void CHAIN::set_samples_per_second(SAMPLE_SPECS::sample_rate_t v)
     ECA_SAMPLERATE_AWARE* srateobj = dynamic_cast<ECA_SAMPLERATE_AWARE*>(temp);
     if (srateobj != 0) {
       ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		    "(eca-chain) sample rate change, chain '" +
+		    "sample rate change, chain '" +
 		    name() + "' object '" +
 		    temp->name() + "' rate " +
 		    kvu_numtostr(v) + ".");
@@ -767,7 +796,7 @@ void CHAIN::set_samples_per_second(SAMPLE_SPECS::sample_rate_t v)
 
   for(size_t p = 0; p != gcontrollers_rep.size(); p++) {
     ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		"(eca-chain) sample rate change, chain '" +
+		"sample rate change, chain '" +
 		name() + "' object '" +
 		gcontrollers_rep[p]->name() + "' rate " +
 		kvu_numtostr(v) + ".");
@@ -783,7 +812,7 @@ void CHAIN::set_samples_per_second(SAMPLE_SPECS::sample_rate_t v)
 void CHAIN::seek_position(void)
 {
   ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		"(eca-chain) seek position, to pos " +
+		"seek position, to pos " +
 		kvu_numtostr(position_in_seconds()) + ".");
 
   for(size_t p = 0; p != gcontrollers_rep.size(); p++) {
