@@ -1,7 +1,10 @@
 // ------------------------------------------------------------------------
 // eca-chainsetup-parser.cpp: Functionality for parsing chainsetup 
 //                            option syntax.
-// Copyright (C) 2001-2003 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 2001-2005 Kai Vehmanen
+//
+// Attributes:
+//     eca-style-version: 3
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -89,7 +92,7 @@ void ECA_CHAINSETUP_PARSER::interpret_global_option (const string& arg)
 {
   interpret_entry();
 
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup-parser) Interpreting global option \"" + arg + "\".");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "Interpreting global option \"" + arg + "\".");
   if (istatus_rep == false) interpret_general_option(arg);
   if (istatus_rep == false) interpret_processing_control(arg);
   if (istatus_rep == false) interpret_chains(arg);
@@ -111,7 +114,7 @@ void ECA_CHAINSETUP_PARSER::interpret_object_option (const string& arg)
 {
   interpret_entry();
 
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup-parser) Interpreting object option \"" + arg + "\".");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "Interpreting object option \"" + arg + "\".");
   interpret_chains(arg);
   if (istatus_rep == false) interpret_audio_format(arg);
   if (istatus_rep == false) interpret_audioio_device(arg);
@@ -138,7 +141,6 @@ void ECA_CHAINSETUP_PARSER::interpret_options(std::vector<string>& opts)
    * phase1: parse global options only */
 
   std::vector<string>::iterator p = opts.begin();
-  p = opts.begin();
   while(p != opts.end()) {
     interpret_global_option(*p);
 
@@ -188,7 +190,7 @@ void ECA_CHAINSETUP_PARSER::interpret_options(std::vector<string>& opts)
 
   if (other_matches + global_matches != optcount) {
     ECA_LOG_MSG(ECA_LOGGER::info, 
-		string("(eca-chainsetup-parser) Warning! Only ") + 
+		string("WARNING: Only ") + 
 		kvu_numtostr(other_matches) +
 		"+" +
 		kvu_numtostr(global_matches) +
@@ -218,7 +220,7 @@ void ECA_CHAINSETUP_PARSER::preprocess_options(std::vector<string>& opts) const
 
     if (p->size() > 0 && (*p)[0] != '-') {
       /* hack1: rest as "-i:file" */
-      ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Note! Interpreting option " +
+      ECA_LOG_MSG(ECA_LOGGER::info, "Note! Interpreting option " +
 		    *p +
 		    " as -i:" +
 		    *p +
@@ -294,10 +296,15 @@ void ECA_CHAINSETUP_PARSER::interpret_general_option (const string& argu)
   case 'b':
     {
       int bsize = atoi(kvu_get_argument_number(1, argu).c_str());
-      csetup_repp->set_buffersize(bsize);
-      MESSAGE_ITEM mitemb;
-      mitemb << "(eca-chainsetup-parser) Setting buffersize to (samples) " << bsize << ".";
-      ECA_LOG_MSG(ECA_LOGGER::info, mitemb.to_string()); 
+      if (bsize > 0) {
+	csetup_repp->set_buffersize(bsize);
+	MESSAGE_ITEM mitemb;
+	mitemb << "Setting buffersize to (samples) " << bsize << ".";
+	ECA_LOG_MSG(ECA_LOGGER::info, mitemb.to_string()); 
+      }
+      else {
+	ECA_LOG_MSG(ECA_LOGGER::info, "Invalid buffersize given; using default value.");
+      }
       break;
     }
 
@@ -306,23 +313,23 @@ void ECA_CHAINSETUP_PARSER::interpret_general_option (const string& argu)
       string temp = kvu_get_argument_number(1, argu);
       if (temp == "auto") {
 	csetup_repp->set_buffering_mode(ECA_CHAINSETUP::cs_bmode_auto);
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Buffering mode is selected automatically.");
+	ECA_LOG_MSG(ECA_LOGGER::info, "Buffering mode is selected automatically.");
       }
       else if (temp == "nonrt") {
 	csetup_repp->set_buffering_mode(ECA_CHAINSETUP::cs_bmode_nonrt);
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Buffering mode 'nonrt' selected.");
+	ECA_LOG_MSG(ECA_LOGGER::info, "Buffering mode 'nonrt' selected.");
       }
       else if (temp == "rt") {
 	csetup_repp->set_buffering_mode(ECA_CHAINSETUP::cs_bmode_rt);
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Buffering mode 'rt' selected.");
+	ECA_LOG_MSG(ECA_LOGGER::info, "Buffering mode 'rt' selected.");
       }
       else if (temp == "rtlowlatency") {
 	csetup_repp->set_buffering_mode(ECA_CHAINSETUP::cs_bmode_rtlowlatency);
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Buffering mode 'rtlowlatency' selected.");
+	ECA_LOG_MSG(ECA_LOGGER::info, "Buffering mode 'rtlowlatency' selected.");
       }
       else {
 	csetup_repp->set_buffering_mode(ECA_CHAINSETUP::cs_bmode_auto);
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Unknown buffering mode; 'auto' mode is used instead.");
+	ECA_LOG_MSG(ECA_LOGGER::info, "Unknown buffering mode; 'auto' mode is used instead.");
       }
       break;
     }
@@ -330,7 +337,7 @@ void ECA_CHAINSETUP_PARSER::interpret_general_option (const string& argu)
   case 'n':
     {
       csetup_repp->set_name(kvu_get_argument_number(1, argu));
-      ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Setting chainsetup name to \""
+      ECA_LOG_MSG(ECA_LOGGER::info, "Setting chainsetup name to \""
 		  + csetup_repp->name() + "\".");
       break;
     }
@@ -355,49 +362,50 @@ void ECA_CHAINSETUP_PARSER::interpret_general_option (const string& argu)
   case 's':
     {
       if (argu.size() > 2 && argu[2] == 'r') {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Option '-sr' is obsolete. Use syntax '-f:sfmt,channels,srate,ileaving' instead.");
+	ECA_LOG_MSG(ECA_LOGGER::info, "Option '-sr' is obsolete. Use syntax '-f:sfmt,channels,srate,ileaving' instead.");
       }
       break;
     }
 
   case 'x':
     {
-      ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Truncating outputs (overwrite-mode).");
+      ECA_LOG_MSG(ECA_LOGGER::info, "Truncating outputs (overwrite-mode).");
       csetup_repp->set_output_openmode(AUDIO_IO::io_write);
       break;
     }
 
   case 'X':
     {
-      ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Updating outputs (rw-mode).");
+      ECA_LOG_MSG(ECA_LOGGER::info, "Updating outputs (rw-mode).");
       csetup_repp->set_output_openmode(AUDIO_IO::io_readwrite);
       break;
     }
 
   case 'z':
     {
-      if (kvu_get_argument_number(1, argu) == "db") {
+      string first_arg (kvu_get_argument_number(1, argu));
+      if (first_arg == "db") {
 	long int bufs = atol(kvu_get_argument_number(2, argu).c_str());
 	if (bufs == 0) bufs = 100000;
 	csetup_repp->set_double_buffer_size(bufs);
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Using double-buffer of " + 
+	ECA_LOG_MSG(ECA_LOGGER::info, "Using double-buffer of " + 
 		    kvu_numtostr(bufs) + " sample frames.");
 	csetup_repp->toggle_double_buffering(true);
       }
-      else if (kvu_get_argument_number(1, argu) == "nodb") {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Double-buffering disabled.");
+      else if (first_arg == "nodb") {
+	ECA_LOG_MSG(ECA_LOGGER::info, "Double-buffering disabled.");
 	csetup_repp->toggle_double_buffering(false);
       }
-      else if (kvu_get_argument_number(1, argu) == "intbuf") {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Enabling extra buffering on realtime devices.");
+      else if (first_arg == "intbuf") {
+	ECA_LOG_MSG(ECA_LOGGER::info, "Enabling extra buffering on realtime devices.");
 	csetup_repp->toggle_max_buffers(true);
       }
-      else if (kvu_get_argument_number(1, argu) == "nointbuf") {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Disabling extra buffering on realtime devices.");
+      else if (first_arg == "nointbuf") {
+	ECA_LOG_MSG(ECA_LOGGER::info, "Disabling extra buffering on realtime devices.");
 	csetup_repp->toggle_max_buffers(false);
       }
-      else if (kvu_get_argument_number(1, argu) == "multitrack") {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Enabling multitrack-mode (override).");
+      else if (first_arg == "multitrack") {
+	ECA_LOG_MSG(ECA_LOGGER::info, "Enabling multitrack-mode (override).");
 	long int samples = -1;
 	if (kvu_get_number_of_arguments(argu) > 1) {
 	  /* -z:multitrack,XXX */
@@ -407,28 +415,38 @@ void ECA_CHAINSETUP_PARSER::interpret_general_option (const string& argu)
 	csetup_repp->multitrack_mode_override_rep = true;
 	csetup_repp->multitrack_mode_rep = true;
       }
-      else if (kvu_get_argument_number(1, argu) == "nomultitrack") {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Disabling multitrack-mode (override).");
+      else if (first_arg == "nomultitrack") {
+	ECA_LOG_MSG(ECA_LOGGER::info, "Disabling multitrack-mode (override).");
 	csetup_repp->multitrack_mode_override_rep = true;
 	csetup_repp->multitrack_mode_offset_rep = 0;
 	csetup_repp->multitrack_mode_rep = false;
       }
-      else if (kvu_get_argument_number(1, argu) == "psr") {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Enabling precise-sample-rates with OSS audio devices.");
+      else if (first_arg == "psr") {
+	ECA_LOG_MSG(ECA_LOGGER::info, "Enabling precise-sample-rates with OSS audio devices.");
 	csetup_repp->toggle_precise_sample_rates(true);
       }
-      else if (kvu_get_argument_number(1, argu) == "nopsr") {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Disabling precise-sample-rates with OSS audio devices.");
+      else if (first_arg == "nopsr") {
+	ECA_LOG_MSG(ECA_LOGGER::info, "Disabling precise-sample-rates with OSS audio devices.");
 	csetup_repp->toggle_precise_sample_rates(false);
       }
-      else if (kvu_get_argument_number(1, argu) == "xruns") {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Processing is stopped if an xrun occurs.");
+      else if (first_arg == "xruns") {
+	ECA_LOG_MSG(ECA_LOGGER::info, "Processing is stopped if an xrun occurs.");
 	csetup_repp->toggle_ignore_xruns(false);
       }
-      else if (kvu_get_argument_number(1, argu) == "noxruns") {
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Ignoring xruns during processing.");
+      else if (first_arg == "noxruns") {
+	ECA_LOG_MSG(ECA_LOGGER::info, "Ignoring xruns during processing.");
 	csetup_repp->toggle_ignore_xruns(true);
       }
+      else if (first_arg == "mixmode") {
+	if (kvu_get_argument_number(2, argu) == "sum") {
+	  ECA_LOG_MSG(ECA_LOGGER::info, "Enabling 'sum' mixmode.");
+	  csetup_repp->set_mix_mode(ECA_CHAINSETUP::cs_mmode_sum);
+	}
+	else {
+	  ECA_LOG_MSG(ECA_LOGGER::info, "Enabling 'avg' mixmode.");
+	  csetup_repp->set_mix_mode(ECA_CHAINSETUP::cs_mmode_avg);
+	}
+      }
       break;
     }
   default: { match = false; }
@@ -462,7 +480,7 @@ void ECA_CHAINSETUP_PARSER::interpret_processing_control (const string& argu)
 	{
 	  /* note! here we set the _maximum_ length of the chainsetup */
 	  csetup_repp->set_max_length_in_seconds(atof(kvu_get_argument_number(1, argu).c_str()));
-	  ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Set processing time to "
+	  ECA_LOG_MSG(ECA_LOGGER::info, "Set processing time to "
 		      + kvu_numtostr(csetup_repp->max_length_in_seconds_exact()) + ".");
 	  break;
 	}
@@ -471,9 +489,9 @@ void ECA_CHAINSETUP_PARSER::interpret_processing_control (const string& argu)
 	{
 	  csetup_repp->toggle_looping(true);
 	  if (csetup_repp->max_length_set() != true)
-	    ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Looping enabled. Length of input objects will be used to set the loop point.");
+	    ECA_LOG_MSG(ECA_LOGGER::info, "Looping enabled. Length of input objects will be used to set the loop point.");
 	  else
-	    ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Looping enabled.");
+	    ECA_LOG_MSG(ECA_LOGGER::info, "Looping enabled.");
 	  break;
 	}
       }
@@ -509,13 +527,13 @@ void ECA_CHAINSETUP_PARSER::interpret_chains (const string& argu)
       std::vector<string> schains = kvu_get_arguments(argu);
       if (std::find(schains.begin(), schains.end(), "all") != schains.end()) {
 	csetup_repp->select_all_chains();
-	ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup-parser) Selected all chains.");
+	ECA_LOG_MSG(ECA_LOGGER::system_objects, "Selected all chains.");
       }
       else {
 	csetup_repp->select_chains(schains);
 	csetup_repp->add_new_chains(schains);
 	MESSAGE_ITEM mtempa;
-	mtempa << "(eca-chainsetup-parser) Selected chain ids: ";
+	mtempa << "Selected chain ids: ";
 	for (std::vector<string>::const_iterator p = schains.begin(); p !=
 	       schains.end(); p++) { mtempa << *p << " "; }
 	ECA_LOG_MSG(ECA_LOGGER::system_objects, mtempa.to_string());
@@ -560,7 +578,7 @@ void ECA_CHAINSETUP_PARSER::interpret_audio_format (const string& argu)
       csetup_repp->set_default_audio_format(active_sinfo);
       
       MESSAGE_ITEM ftemp;
-      ftemp << "(eca-chainsetup-parser) Set active format to (bits/channels/srate/interleave): ";
+      ftemp << "Set active format to (bits/channels/srate/interleave): ";
       ftemp << csetup_repp->default_audio_format().format_string() 
 	    << "/" << csetup_repp->default_audio_format().channels() 
 	    << "/" << csetup_repp->default_audio_format().samples_per_second();
@@ -598,7 +616,7 @@ void ECA_CHAINSETUP_PARSER::interpret_effect_preset (const string& argu)
   switch(argu[1]) {
   case 'p':
     {
-      ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup-parser) Interpreting preset \"" + argu + "\".");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "Interpreting preset \"" + argu + "\".");
       CHAIN_OPERATOR* cop = 0;
 
       if (argu.size() < 3) return;  
@@ -628,14 +646,9 @@ void ECA_CHAINSETUP_PARSER::interpret_effect_preset (const string& argu)
       default: { }
       }
       if (cop != 0) {
-          MESSAGE_ITEM otemp;
           for(int n = 0; n < cop->number_of_params(); n++) {
               cop->set_parameter(n + 1, atof(kvu_get_argument_number(n + 2, argu).c_str()));
-              otemp << cop->get_parameter_name(n + 1) << " = ";
-              otemp << cop->get_parameter(n +1);
-              if (n + 1 < cop->number_of_params()) otemp << ", ";
           }
-          ECA_LOG_MSG(ECA_LOGGER::info, otemp.to_string());
           csetup_repp->add_chain_operator(cop);
       }
       break;
@@ -676,12 +689,12 @@ void ECA_CHAINSETUP_PARSER::interpret_audioio_device (const string& argu)
 	if ((audio_input->supported_io_modes() &
 	     AUDIO_IO::io_read) != AUDIO_IO::io_read) {
 	  interpret_set_result(false, 
-			       string("(eca-chainsetup-parser) Audio object '") + 
+			       string("Audio object '") + 
 			       tname +
 			       "' cannot be opened for input.");
 	}
 	else {
-	  ECA_LOG_MSG(ECA_LOGGER::system_objects,"(eca-chainsetup-parser) adding file \"" + tname + "\".");
+	  ECA_LOG_MSG(ECA_LOGGER::system_objects,"adding file \"" + tname + "\".");
 	  csetup_repp->add_input(audio_input);
 	  last_audio_add_vector_repp = &csetup_repp->inputs; /* for -y parsing */
 	}
@@ -709,11 +722,14 @@ void ECA_CHAINSETUP_PARSER::interpret_audioio_device (const string& argu)
 	    truncate = true;
 	  }
 	}
-	if ((audio_output->supported_io_modes() & mode_tmp != mode_tmp)) {
-	  interpret_set_result(false, string("(eca-chainsetup-parser) I/O-mode 'io_write' not supported by ") + audio_output->name());
+	else {
+	  truncate = true;
+	}
+	if (((audio_output->supported_io_modes() & mode_tmp) != mode_tmp)) {
+	  interpret_set_result(false, string("io_write/io_readwrite access modes not supported by output '") + audio_output->name() + "'.");
 	}
 	else {
-	  ECA_LOG_MSG(ECA_LOGGER::system_objects,"(eca-chainsetup-parser) adding file \"" + tname + "\".");
+	  ECA_LOG_MSG(ECA_LOGGER::system_objects,"adding file \"" + tname + "\".");
 	  csetup_repp->add_output(audio_output, truncate);
 	  last_audio_add_vector_repp = &csetup_repp->outputs; /* for -y parsing */
 	}
@@ -730,7 +746,7 @@ void ECA_CHAINSETUP_PARSER::interpret_audioio_device (const string& argu)
 
       if (last_audio_add_vector_repp == 0) {
 	ECA_LOG_MSG(ECA_LOGGER::info, 
-		    "Error! Non-existant last audio object.");
+		    "ERROR: Non-existant last audio object.");
       }
       else {
 	AUDIO_IO* last_object = (*last_audio_add_vector_repp).back();
@@ -744,7 +760,7 @@ void ECA_CHAINSETUP_PARSER::interpret_audioio_device (const string& argu)
 	  csetup_repp->output_start_pos[csetup_repp->output_start_pos.size() - 1] = last_object->position_in_seconds_exact();
 	}
 
-	ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) Setting starting position for audio object \""
+	ECA_LOG_MSG(ECA_LOGGER::info, "Setting starting position for audio object \""
 		    + last_object->label() 
 		    + "\": "
 		    + kvu_numtostr(last_object->position_in_seconds_exact()) 
@@ -760,7 +776,7 @@ void ECA_CHAINSETUP_PARSER::interpret_audioio_device (const string& argu)
 
   if (print_error == true) {
     interpret_set_result(false, 
-			 string("(eca-chainsetup-parser) Audio object '") +
+			 string("Audio object '") +
 			 tname + 
 			 "' does not match any of the known audio device types or "
 			 "file formats. You can check the list of supported "
@@ -829,7 +845,7 @@ void ECA_CHAINSETUP_PARSER::interpret_midi_device (const string& argu)
 	case 'd': 
 	  {
 	    string tname = kvu_get_argument_number(1, argu);
-	    ECA_LOG_MSG(ECA_LOGGER::system_objects,"(eca-chainsetup-parser) MIDI-config: Adding device \"" + tname + "\".");
+	    ECA_LOG_MSG(ECA_LOGGER::system_objects,"MIDI-config: Adding device \"" + tname + "\".");
 	    MIDI_IO* mdev = 0;
 	    mdev = ECA_OBJECT_FACTORY::create_midi_device(argu);
 	    if (mdev != 0) {
@@ -839,7 +855,7 @@ void ECA_CHAINSETUP_PARSER::interpret_midi_device (const string& argu)
 		csetup_repp->midi_server_needed_rep = true;
 	      }
 	      else {
-		ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup-parser) MIDI-config: Warning! I/O-mode 'io_readwrite' not supported by " + mdev->name());
+		ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: I/O-mode 'io_readwrite' not supported by MIDI-device " + mdev->name());
 	      }
 	    }
 	    break;
@@ -854,7 +870,7 @@ void ECA_CHAINSETUP_PARSER::interpret_midi_device (const string& argu)
 	      // FIXME: not implemented!
 	      int id = atoi(kvu_get_argument_number(1, argu).c_str());
 	      ECA_LOG_MSG(ECA_LOGGER::info, 
-			    "(eca-chainsetup-parser) MIDI-config: Receiving MMC messages with id  \"" + 
+			    "MIDI-config: Receiving MMC messages with id  \"" + 
 			    kvu_numtostr(id) +
 			    "\".");
 	      csetup_repp->midi_server_repp->set_mmc_receive_id(id);
@@ -867,7 +883,7 @@ void ECA_CHAINSETUP_PARSER::interpret_midi_device (const string& argu)
 	    {
 	      int id = atoi(kvu_get_argument_number(1, argu).c_str());
 	      ECA_LOG_MSG(ECA_LOGGER::info, 
-			    "(eca-chainsetup-parser) MIDI-config: Adding MMC-send to device id \"" + 
+			    "MIDI-config: Adding MMC-send to device id \"" + 
 			    kvu_numtostr(id) +
 			    "\".");
 	      csetup_repp->midi_server_repp->add_mmc_send_id(id);
@@ -886,7 +902,7 @@ void ECA_CHAINSETUP_PARSER::interpret_midi_device (const string& argu)
 	    {
 	      // FIXME: not implemented
 	      ECA_LOG_MSG(ECA_LOGGER::info, 
-			    "(eca-chainsetup-parser) MIDI-config: Receiving MIDI-sync.");
+			    "MIDI-config: Receiving MIDI-sync.");
 	      csetup_repp->midi_server_needed_rep = true;
 	      csetup_repp->midi_server_repp->toggle_midi_sync_receive(true);
 	      break;
@@ -896,7 +912,7 @@ void ECA_CHAINSETUP_PARSER::interpret_midi_device (const string& argu)
 	    {
 	      // FIXME: not implemented
 	      ECA_LOG_MSG(ECA_LOGGER::info, 
-			    "(eca-chainsetup-parser) MIDI-config: Sending MIDI-sync.");
+			    "MIDI-config: Sending MIDI-sync.");
 	      csetup_repp->midi_server_repp->toggle_midi_sync_send(true);
 	      csetup_repp->midi_server_needed_rep = true;
 	      break;
@@ -983,7 +999,7 @@ string ECA_CHAINSETUP_PARSER::general_options_to_string(void) const
 
   int setparams = csetup_repp->override_buffering_parameters().number_of_set();
   ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		"(eca-chainsetup-parser) genopts tostring - " + kvu_numtostr(setparams) +
+		"genopts tostring - " + kvu_numtostr(setparams) +
 		" overridden parameters.");
 
   if (setparams > 0) {
@@ -1058,13 +1074,18 @@ string ECA_CHAINSETUP_PARSER::general_options_to_string(void) const
   else
     t << " -z:nopsr";
 
+  if (csetup_repp->mix_mode() == ECA_CHAINSETUP::cs_mmode_avg)
+    t << " -z:mixmode,avg";
+  else
+    t << " -z:mixmode,sum";
+
   t.setprecision(3);
   if (csetup_repp->max_length_set()) {
     t << " -t:" << csetup_repp->max_length_in_seconds_exact();
   }
   if (csetup_repp->looping_enabled()) t << " -tl";
 
-  return(t.to_string());
+  return t.to_string();
 }
 
 string ECA_CHAINSETUP_PARSER::midi_to_string(void) const
@@ -1076,6 +1097,7 @@ string ECA_CHAINSETUP_PARSER::midi_to_string(void) const
   while (p < csetup_repp->midi_devices.size()) {
     t << "-Md:";
     for(int n = 0; n < csetup_repp->midi_devices[p]->number_of_params(); n++) {
+      // FIXME: should quote/escape possible commas and whitespace
       t << csetup_repp->midi_devices[p]->get_parameter(n + 1);
       if (n + 1 < csetup_repp->midi_devices[p]->number_of_params()) t << ",";
     }
@@ -1083,7 +1105,7 @@ string ECA_CHAINSETUP_PARSER::midi_to_string(void) const
     if (p < csetup_repp->midi_devices.size()) t << " ";
   }
 
-  return(t.to_string());
+  return t.to_string();
 }
 
 string ECA_CHAINSETUP_PARSER::inputs_to_string(void) const
@@ -1112,7 +1134,7 @@ string ECA_CHAINSETUP_PARSER::inputs_to_string(void) const
     if (p < csetup_repp->inputs.size()) t << "\n";
   }
 
-  return(t.to_string());
+  return t.to_string();
 }
 
 string ECA_CHAINSETUP_PARSER::outputs_to_string(void) const
@@ -1141,7 +1163,7 @@ string ECA_CHAINSETUP_PARSER::outputs_to_string(void) const
     if (p < csetup_repp->outputs.size()) t << "\n";
   }
 
-  return(t.to_string());
+  return t.to_string();
 }
 
 string ECA_CHAINSETUP_PARSER::chains_to_string(void) const
@@ -1159,7 +1181,7 @@ string ECA_CHAINSETUP_PARSER::chains_to_string(void) const
     ++p;
   }
 
-  return(t.to_string());
+  return t.to_string();
 }
 
 string ECA_CHAINSETUP_PARSER::audioio_to_string(const AUDIO_IO* aiod, const string& direction) const
@@ -1170,9 +1192,10 @@ string ECA_CHAINSETUP_PARSER::audioio_to_string(const AUDIO_IO* aiod, const stri
     aiod->channels() << ","  << aiod->samples_per_second();
   t << " -" << direction << ":";
   for(int n = 0; n < aiod->number_of_params(); n++) {
+    // FIXME: should quote/escape possible commas and whitespace
     t << aiod->get_parameter(n + 1);
     if (n + 1 < aiod->number_of_params()) t << ",";
   }
 
-  return(t.to_string());
+  return t.to_string();
 }
