@@ -1,6 +1,9 @@
 // ------------------------------------------------------------------------
 // eca-chainsetup.cpp: Class representing an ecasound chainsetup object.
-// Copyright (C) 1999-2003 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 1999-2004 Kai Vehmanen
+//
+// Attributes:
+//     eca-style-version: 3
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -61,6 +64,7 @@
 
 #include "eca-engine-driver.h"
 #include "eca-object-factory.h"
+#include "eca-object-map.h"
 
 #include "midiio.h"
 #include "midi-client.h"
@@ -183,7 +187,7 @@ ECA_CHAINSETUP::~ECA_CHAINSETUP(void)
 
   /* delete chain objects */
   for(vector<CHAIN*>::iterator q = chains.begin(); q != chains.end(); q++) {
-    ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-chainsetup) Deleting chain \"" + (*q)->name() + "\".");
+    ECA_LOG_MSG(ECA_LOGGER::user_objects, "Deleting chain \"" + (*q)->name() + "\".");
     delete *q;
     *q = 0;
   }
@@ -191,7 +195,7 @@ ECA_CHAINSETUP::~ECA_CHAINSETUP(void)
   /* take the garbage out (must be done before deleting input 
    * and output objects) */
   for(list<AUDIO_IO*>::iterator q = aobj_garbage_rep.begin(); q != aobj_garbage_rep.end(); q++) {
-    ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-chainsetup) Deleting garbage audio object \"" + (*q)->label() + "\".");
+    ECA_LOG_MSG(ECA_LOGGER::user_objects, "Deleting garbage audio object \"" + (*q)->label() + "\".");
     delete *q;
     *q = 0;
   }
@@ -199,7 +203,7 @@ ECA_CHAINSETUP::~ECA_CHAINSETUP(void)
   /* delete input db objects; reset all pointers to null */
   for(vector<AUDIO_IO*>::iterator q = inputs.begin(); q != inputs.end(); q++) {
     if (dynamic_cast<AUDIO_IO_DB_CLIENT*>(*q) != 0) {
-      ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-chainsetup) Deleting audio db-client \"" + (*q)->label() + "\".");
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Deleting audio db-client \"" + (*q)->label() + "\".");
       delete *q;
     }
     *q = 0;
@@ -208,7 +212,7 @@ ECA_CHAINSETUP::~ECA_CHAINSETUP(void)
   /* delete all actual audio input objects except loop devices; reset all pointers to null */
   for(vector<AUDIO_IO*>::iterator q = inputs_direct_rep.begin(); q != inputs_direct_rep.end(); q++) {
     if (dynamic_cast<LOOP_DEVICE*>(*q) == 0) { 
-      ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-chainsetup) Deleting audio object \"" + (*q)->label() + "\".");
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Deleting audio object \"" + (*q)->label() + "\".");
       delete *q;
     }
     *q = 0;
@@ -217,7 +221,7 @@ ECA_CHAINSETUP::~ECA_CHAINSETUP(void)
   /* delete output db objects; reset all pointers to null */
   for(vector<AUDIO_IO*>::iterator q = outputs.begin(); q != outputs.end(); q++) {
     if (dynamic_cast<AUDIO_IO_DB_CLIENT*>(*q) != 0) {
-      ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-chainsetup) Deleting audio db-client \"" + (*q)->label() + "\".");
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Deleting audio db-client \"" + (*q)->label() + "\".");
       delete *q;
     }
     *q = 0;
@@ -227,7 +231,7 @@ ECA_CHAINSETUP::~ECA_CHAINSETUP(void)
   for(vector<AUDIO_IO*>::iterator q = outputs_direct_rep.begin(); q != outputs_direct_rep.end(); q++) {
     // trouble with dynamic_cast with libecasoundc apps like ecalength?
     if (dynamic_cast<LOOP_DEVICE*>(*q) == 0) { 
-      ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-chainsetup) Deleting audio object \"" + (*q)->label() + "\".");
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Deleting audio object \"" + (*q)->label() + "\".");
       delete *q;
       *q = 0;
     }
@@ -235,14 +239,14 @@ ECA_CHAINSETUP::~ECA_CHAINSETUP(void)
 
   /* delete loop objects */
   for(map<int,LOOP_DEVICE*>::iterator q = loop_map.begin(); q != loop_map.end(); q++) {
-    ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-chainsetup) Deleting loop device \"" + q->second->label() + "\".");
+    ECA_LOG_MSG(ECA_LOGGER::user_objects, "Deleting loop device \"" + q->second->label() + "\".");
     delete q->second;
     q->second = 0;
   }
 
   /* delete aio manager objects */
   for(vector<AUDIO_IO_MANAGER*>::iterator q = aio_managers_rep.begin(); q != aio_managers_rep.end(); q++) {
-    ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-chainsetup) Deleting audio manager \"" + (*q)->name() + "\".");
+    ECA_LOG_MSG(ECA_LOGGER::user_objects, "Deleting audio manager \"" + (*q)->name() + "\".");
     delete *q;
     *q = 0;
   }
@@ -250,15 +254,6 @@ ECA_CHAINSETUP::~ECA_CHAINSETUP(void)
   delete impl_repp;
 }
 
-/**
- * Tests whether chainsetup is in a valid state.
- */
-bool ECA_CHAINSETUP::is_valid(void) const
-{
-  // FIXME: waiting for a better implementation...
-  return(is_valid_for_connection());
-}
-
 /**
  * Sets default values.
  *
@@ -280,7 +275,7 @@ void ECA_CHAINSETUP::set_defaults(void)
   if (kvu_check_for_mlockall() == true && 
       kvu_check_for_sched_fifo() == true) {
     rtcaps_rep = true;
-    ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) Rtcaps detected.");
+    ECA_LOG_MSG(ECA_LOGGER::system_objects, "Rtcaps detected.");
   }
   else 
     rtcaps_rep = false;
@@ -304,19 +299,20 @@ void ECA_CHAINSETUP::set_defaults(void)
   ECA_RESOURCES ecaresources;
   if (ecaresources.has_any() != true) {
     ECA_LOG_MSG(ECA_LOGGER::info, 
-		"(eca-chainsetup) Warning! Unable to read global resources. May result in incorrect behaviour.");
+		"WARNING: Unable to read global resources. May result in incorrect behaviour.");
   }
   
   set_default_midi_device(ecaresources.resource("midi-device"));
-
-  string aformat_temp = set_resource_helper(ecaresources,
-					    "default-audio-format", 
-					    ECA_CHAINSETUP::default_audio_format_const);
-  cparser_rep.interpret_object_option("-f:" + aformat_temp);
-
+  string rc_temp = set_resource_helper(ecaresources,
+				       "default-audio-format", 
+				       ECA_CHAINSETUP::default_audio_format_const);
+  cparser_rep.interpret_object_option("-f:" + rc_temp);
   set_samples_per_second(default_audio_format().samples_per_second());
-
   toggle_precise_sample_rates(ecaresources.boolean_resource("default-to-precise-sample-rates"));
+  rc_temp = set_resource_helper(ecaresources, 
+				"default-mix-mode", 
+				"avg");
+  cparser_rep.interpret_object_option("-z:mixmode," + rc_temp);
 
   impl_repp->bmode_nonrt_rep.set_all(set_resource_helper(ecaresources,
 							 "bmode-defaults-nonrt",
@@ -339,40 +335,103 @@ void ECA_CHAINSETUP::set_defaults(void)
 string ECA_CHAINSETUP::set_resource_helper(const ECA_RESOURCES& ecaresources, const string& tag, const string& alternative)
 {
   if (ecaresources.has(tag) == true) {
-    return(ecaresources.resource(tag));
+    return ecaresources.resource(tag);
   }
   else {
     ECA_LOG_MSG(ECA_LOGGER::system_objects,
 		"(eca-chaisetup) Using hardcoded defaults for '" +
 		tag + "'.");
-    return(alternative);
+    return alternative;
   }
 }
 
 /**
- * Checks whether chainsetup is valid for 
- * enabling/connecting.
+ * Tests whether chainsetup is in a valid state.
  */
-bool ECA_CHAINSETUP::is_valid_for_connection(void) const 
+bool ECA_CHAINSETUP::is_valid(void) const
 {
+  return is_valid_for_connection(false);
+}
+
+/**
+ * Checks whether chainsetup is valid for enabling/connecting. 
+ * If chainsetup is not valid and 'verbose' is true, detected
+ * errors are reported via the logging subsystem.
+ */
+bool ECA_CHAINSETUP::is_valid_for_connection(bool verbose) const 
+{
+  bool result = true;
+
   if (inputs.size() == 0) {
-    ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) No inputs in the current chainsetup.");
-    return(false);
+    if (verbose) ECA_LOG_MSG(ECA_LOGGER::info, 
+			     "Unable to connect: No inputs in the current chainsetup. (1.1-NO-INPUTS)");
+    result = false;
   }
-  if (outputs.size() == 0) {
-    ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) No outputs in the current chainsetup.");
-    return(false);
+  else if (outputs.size() == 0) {
+    if (verbose) ECA_LOG_MSG(ECA_LOGGER::info, 
+			     "Unable to connect: No outputs in the current chainsetup. (1.2-NO-OUTPUTS)");
+    result = false;
   }
-  if (chains.size() == 0) {
-    ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) No chains in the current chainsetup.");
-    return(false);
+  else if (chains.size() == 0) {
+    if (verbose) ECA_LOG_MSG(ECA_LOGGER::info, 
+			     "Unable to connect: No chains in the current chainsetup. (1.3-NO-CHAINS)");
+    result = false;
   }
-  for(vector<CHAIN*>::const_iterator q = chains.begin(); q != chains.end();
-      q++) {
-    // debug info printed in CHAIN::is_valid()
-    if ((*q)->is_valid() == false) return(false);
-  }
-  return(true);
+  else {
+    list<int> conn_inputs, conn_outputs;
+
+    for(vector<CHAIN*>::const_iterator q = chains.begin(); q != chains.end(); q++) {
+      /* log messages printed in CHAIN::is_valid() */
+
+      int id = (*q)->connected_input();
+      if (id > -1) 
+	conn_inputs.push_back(id);
+      
+      if ((*q)->is_valid() == false) {
+	result = false;
+	if (verbose) ECA_LOG_MSG(ECA_LOGGER::info, 
+				 "Unable to connect: Chain \"" + (*q)->name() + 
+				 "\" is not valid. Following errors were detected:");
+	if (verbose && id == -1) {
+	  ECA_LOG_MSG(ECA_LOGGER::info, 
+		      "Chain \"" + (*q)->name() + "\" is not connected to any input. "
+		      "All chains must have exactly one valid input. (2.1-NO-CHAIN-INPUT)");
+	}
+      }
+
+      id = (*q)->connected_output();
+      if (id > -1) 
+	conn_outputs.push_back(id);
+
+      if (verbose && (*q)->is_valid() == false) {
+	if (id == -1) {
+	  ECA_LOG_MSG(ECA_LOGGER::info, 
+		      "Chain \"" + (*q)->name() + "\" is not connected to any output. "
+		      "All chains must have exactly one valid output. (2.2-NO-CHAIN-OUTPUT)");
+	}
+      }
+    }
+
+    // FIXME: doesn't work yet
+
+    if (verbose) {
+      for(int n = 0; n < static_cast<int>(inputs.size()); n++) {
+	if (std::find(conn_inputs.begin(), conn_inputs.end(), n) == conn_inputs.end()) {
+	  ECA_LOG_MSG(ECA_LOGGER::info, 
+		      "WARNING: Input \"" + inputs[n]->label() + "\" is not connected to any chain. (3.1-DISCON-INPUT)");
+	}
+      }
+      
+      for(int n = 0; n < static_cast<int>(outputs.size()); n++) {
+	if (std::find(conn_outputs.begin(), conn_outputs.end(), n) == conn_outputs.end()) {
+	  ECA_LOG_MSG(ECA_LOGGER::info, 
+		      "WARNING: Output \"" + outputs[n]->label() + "\" is not connected to any chain. (3.2-DISCON-OUTPUT)");
+	}
+      }
+    }
+  } /* (verbose == true) */
+
+  return result;
 }
 
 void ECA_CHAINSETUP::set_buffering_mode(Buffering_mode_t value)
@@ -391,7 +450,7 @@ void ECA_CHAINSETUP::set_buffering_mode(Buffering_mode_t value)
 void ECA_CHAINSETUP::set_audio_io_manager_option(const string& mgrname, const string& optionstr)
 {
   ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-	      "(eca-chainsetup) Set manager '" +
+	      "Set manager '" +
 	      mgrname + "' option string to '" +
 	      optionstr + "'.");
 
@@ -421,7 +480,7 @@ void ECA_CHAINSETUP::select_active_buffering_mode(void)
 	number_of_non_realtime_inputs() > 0 && 
 	number_of_non_realtime_outputs() > 0 &&
 	chains.size() > 1))) {
-    ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup) Multitrack-mode enabled.");
+    ECA_LOG_MSG(ECA_LOGGER::info, "Multitrack-mode enabled.");
     multitrack_mode_rep = true;
   }
   else
@@ -436,14 +495,14 @@ void ECA_CHAINSETUP::select_active_buffering_mode(void)
       /* case 1: a multitrack setup */
       if (multitrack_mode_rep == true) {
 	active_buffering_mode_rep = ECA_CHAINSETUP::cs_bmode_rt;
-	ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) bmode-selection case-1");
+	ECA_LOG_MSG(ECA_LOGGER::system_objects, "bmode-selection case-1");
       }
 
       /* case 2: rt-objects without priviledges for rt-scheduling */
       else if (rtcaps_rep != true) {
 	toggle_raised_priority(false);
 	active_buffering_mode_rep = ECA_CHAINSETUP::cs_bmode_rt;
-	ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) bmode-selection case-2");
+	ECA_LOG_MSG(ECA_LOGGER::system_objects, "bmode-selection case-2");
       }
 
       /* case 3: no chain operators and "one-way rt-operation" */
@@ -451,25 +510,25 @@ void ECA_CHAINSETUP::select_active_buffering_mode(void)
 	       (number_of_realtime_inputs() == 0 || 
 		number_of_realtime_outputs() == 0)) {
 	active_buffering_mode_rep = ECA_CHAINSETUP::cs_bmode_rt;
-	ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) bmode-selection case-3");
+	ECA_LOG_MSG(ECA_LOGGER::system_objects, "bmode-selection case-3");
       }
 
       /* case 4: default for rt-setups */
       else {
 	active_buffering_mode_rep = ECA_CHAINSETUP::cs_bmode_rtlowlatency;
-	ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) bmode-selection case-4");
+	ECA_LOG_MSG(ECA_LOGGER::system_objects, "bmode-selection case-4");
       }
     }
     else { 
       /* case 5: no rt-objects */
       active_buffering_mode_rep = ECA_CHAINSETUP::cs_bmode_nonrt;
-      ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) bmode-selection case-5");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "bmode-selection case-5");
     }
   }
   else {
     /* user has explicitly selected the buffering mode */
     active_buffering_mode_rep = buffering_mode();
-    ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) bmode-selection explicit");
+    ECA_LOG_MSG(ECA_LOGGER::system_objects, "bmode-selection explicit");
   }
   
   switch(active_buffering_mode_rep) 
@@ -477,26 +536,26 @@ void ECA_CHAINSETUP::select_active_buffering_mode(void)
     case ECA_CHAINSETUP::cs_bmode_nonrt: { 
       impl_repp->bmode_active_rep = impl_repp->bmode_nonrt_rep;
       ECA_LOG_MSG(ECA_LOGGER::info, 
-		    "(eca-chainsetup) 'nonrt' buffering mode selected.");
+		    "'nonrt' buffering mode selected.");
       break; 
     }
     case ECA_CHAINSETUP::cs_bmode_rt: { 
       impl_repp->bmode_active_rep = impl_repp->bmode_rt_rep;
       ECA_LOG_MSG(ECA_LOGGER::info, 
-		    "(eca-chainsetup) 'rt' buffering mode selected.");
+		    "'rt' buffering mode selected.");
       break; 
     }
     case ECA_CHAINSETUP::cs_bmode_rtlowlatency: { 
       impl_repp->bmode_active_rep = impl_repp->bmode_rtlowlatency_rep;
       ECA_LOG_MSG(ECA_LOGGER::info, 
-		    "(eca-chainsetup) 'rtlowlatency' buffering mode selected.");
+		    "'rtlowlatency' buffering mode selected.");
       break;
     }
     default: { /* error! */ }
     }
 
   ECA_LOG_MSG(ECA_LOGGER::system_objects,
-		"(eca-chainsetup) Set buffering parameters to: \n--cut--" +
+		"Set buffering parameters to: \n--cut--" +
 		impl_repp->bmode_active_rep.to_string() +"\n--cut--");
 }
 
@@ -519,19 +578,19 @@ void ECA_CHAINSETUP::enable_active_buffering_mode(void)
   if (double_buffering() == true) {
     if (has_realtime_objects() != true) {
       ECA_LOG_MSG(ECA_LOGGER::system_objects,
-		    "(eca-chainsetup) No realtime objects; switching to direct mode.");
+		    "No realtime objects; switching to direct mode.");
       switch_to_direct_mode();
       impl_repp->bmode_active_rep.toggle_double_buffering(false);
     }
     else if (has_nonrealtime_objects() != true) {
       ECA_LOG_MSG(ECA_LOGGER::system_objects,
-		    "(eca-chainsetup) Only realtime objects; switching to direct mode.");
+		    "Only realtime objects; switching to direct mode.");
       switch_to_direct_mode();
       impl_repp->bmode_active_rep.toggle_double_buffering(false);
     }
     else if (db_clients_rep == 0) {
       ECA_LOG_MSG(ECA_LOGGER::system_objects,
-		    "(eca-chainsetup) Switching to db mode.");
+		    "Switching to db mode.");
       switch_to_db_mode();
     }
 
@@ -541,7 +600,7 @@ void ECA_CHAINSETUP::enable_active_buffering_mode(void)
     }
     else {
       ECA_LOG_MSG(ECA_LOGGER::info,
-		    "(eca-chainsetup) Warning! Buffersize set to 0.");
+		    "WARNING: Buffersize set to 0.");
       impl_repp->pserver_rep.set_buffer_defaults(0, 0);
     }
   }
@@ -549,7 +608,7 @@ void ECA_CHAINSETUP::enable_active_buffering_mode(void)
     /* double_buffering() != true */
     if (db_clients_rep > 0) {
       ECA_LOG_MSG(ECA_LOGGER::system_objects,
-		    "(eca-chainsetup) Switching to direct mode.");
+		    "Switching to direct mode.");
       switch_to_direct_mode();
     }
   }
@@ -614,14 +673,14 @@ void ECA_CHAINSETUP::lock_all_memory(void)
 {
 #ifdef HAVE_MLOCKALL
   if (::mlockall (MCL_CURRENT|MCL_FUTURE)) {
-    ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup) Warning! Couldn't lock all memory!");
+    ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: Couldn't lock all memory!");
   }
   else {
-    ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) Memory locked!");
+    ECA_LOG_MSG(ECA_LOGGER::system_objects, "Memory locked!");
     memory_locked_rep = true;
   }
 #else
-  ECA_LOG_MSG(ECA_LOGGER::info, "(eca-chainsetup) Memory locking not available.");
+  ECA_LOG_MSG(ECA_LOGGER::info, "Memory locking not available.");
 #endif
 }
 
@@ -633,15 +692,15 @@ void ECA_CHAINSETUP::unlock_all_memory(void)
 #ifdef HAVE_MUNLOCKALL
   if (memory_locked_rep == true) {
     if (::munlockall()) {
-      ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) Warning! Couldn't unlock all memory!");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "WARNING: Couldn't unlock all memory!");
     }
     else 
-      ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) Memory unlocked!");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "Memory unlocked!");
     memory_locked_rep = false;
   }
 #else
   memory_locked_rep = false;
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) Memory unlocking not available.");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "Memory unlocking not available.");
 #endif
 }
 
@@ -698,7 +757,7 @@ void ECA_CHAINSETUP::add_chain_helper(const string& name)
   chains.push_back(new CHAIN());
   chains.back()->name(name);
   chains.back()->set_samples_per_second(samples_per_second());
-  ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-chainsetup) Chain \"" + name + "\" created.");
+  ECA_LOG_MSG(ECA_LOGGER::user_objects, "Chain \"" + name + "\" created.");
 }
 
 /**
@@ -793,11 +852,11 @@ unsigned int ECA_CHAINSETUP::first_selected_chain(void) const
   while(o != schains.end()) {
     for(p = 0; p != chains.size(); p++) {
       if (chains[p]->name() == *o)
-	return(p);
+	return p;
     }
     ++o;
   }
-  return(p);
+  return p;
 }
 
 /**
@@ -848,12 +907,12 @@ void ECA_CHAINSETUP::toggle_chain_bypass(void)
 
 const ECA_CHAINSETUP_BUFPARAMS& ECA_CHAINSETUP::active_buffering_parameters(void) const 
 {
-  return(impl_repp->bmode_active_rep);
+  return impl_repp->bmode_active_rep;
 }
 
 const ECA_CHAINSETUP_BUFPARAMS& ECA_CHAINSETUP::override_buffering_parameters(void) const 
 {
-  return(impl_repp->bmode_override_rep);
+  return impl_repp->bmode_override_rep;
 }
 
 vector<string> ECA_CHAINSETUP::chain_names(void) const
@@ -864,7 +923,7 @@ vector<string> ECA_CHAINSETUP::chain_names(void) const
     result.push_back((*p)->name());
     ++p;
   }
-  return(result);
+  return result;
 }
 
 vector<string> ECA_CHAINSETUP::audio_input_names(void) const
@@ -875,7 +934,7 @@ vector<string> ECA_CHAINSETUP::audio_input_names(void) const
     result.push_back((*p)->label());
     ++p;
   }
-  return(result);
+  return result;
 }
 
 vector<string> ECA_CHAINSETUP::audio_output_names(void) const
@@ -886,7 +945,7 @@ vector<string> ECA_CHAINSETUP::audio_output_names(void) const
     result.push_back((*p)->label());
     ++p;
   }
-  return(result);
+  return result;
 }
 
 vector<string> ECA_CHAINSETUP::get_attached_chains_to_input(AUDIO_IO* aiod) const 
@@ -901,7 +960,7 @@ vector<string> ECA_CHAINSETUP::get_attached_chains_to_input(AUDIO_IO* aiod) cons
     ++q;
   }
   
-  return(res); 
+  return res; 
 }
 
 vector<string> ECA_CHAINSETUP::get_attached_chains_to_output(AUDIO_IO* aiod) const
@@ -931,7 +990,7 @@ int ECA_CHAINSETUP::number_of_attached_chains_to_input(AUDIO_IO* aiod) const
     ++q;
   }
 
-  return(count); 
+  return count; 
 }
 
 int ECA_CHAINSETUP::number_of_attached_chains_to_output(AUDIO_IO* aiod) const
@@ -946,7 +1005,7 @@ int ECA_CHAINSETUP::number_of_attached_chains_to_output(AUDIO_IO* aiod) const
     ++q;
   }
 
-  return(count); 
+  return count; 
 }
 
 /**
@@ -971,11 +1030,11 @@ bool ECA_CHAINSETUP::is_realtime_target_output(int output_id) const
     ++q;
   }
   if (output_found == true && result == true) 
-    ECA_LOG_MSG(ECA_LOGGER::system_objects,"(eca-chainsetup) slave output detected: " + outputs[output_id]->label());
+    ECA_LOG_MSG(ECA_LOGGER::system_objects,"slave output detected: " + outputs[output_id]->label());
   else
     result = false;
 
-  return(result);
+  return result;
 }
 
 vector<string> ECA_CHAINSETUP::get_attached_chains_to_iodev(const string& filename) const
@@ -985,17 +1044,17 @@ vector<string> ECA_CHAINSETUP::get_attached_chains_to_iodev(const string& filena
   p = 0;
   while (p < inputs.size()) {
     if (inputs[p]->label() == filename)
-      return(get_attached_chains_to_input(inputs[p]));
+      return get_attached_chains_to_input(inputs[p]);
     ++p;
   }
 
   p = 0;
   while (p < outputs.size()) {
     if (outputs[p]->label() == filename)
-      return(get_attached_chains_to_output(outputs[p]));
+      return get_attached_chains_to_output(outputs[p]);
     ++p;
   }
-  return(vector<string> (0));
+  return vector<string> (0);
 }
 
 /**
@@ -1009,7 +1068,7 @@ int ECA_CHAINSETUP::number_of_chain_operators(void) const
     cops += (*q)->number_of_chain_operators();
     ++q;
   }
-  return(cops);
+  return cops;
 }
 
 /**
@@ -1020,9 +1079,9 @@ bool ECA_CHAINSETUP::has_realtime_objects(void) const
 {
   if (number_of_realtime_inputs() > 0 ||
       number_of_realtime_outputs() > 0) 
-    return(true);
+    return true;
 
-  return(false);
+  return false;
 }
 
 /**
@@ -1033,9 +1092,9 @@ bool ECA_CHAINSETUP::has_nonrealtime_objects(void) const
 {
   if (static_cast<int>(inputs_direct_rep.size() + outputs_direct_rep.size()) >
       number_of_realtime_inputs() + number_of_realtime_outputs())
-    return(true);
+    return true;
   
-  return(false);
+  return false;
 }
 
 /**
@@ -1045,7 +1104,7 @@ bool ECA_CHAINSETUP::has_nonrealtime_objects(void) const
  */
 string ECA_CHAINSETUP::options_to_string(void) const
 {
-  return(cparser_rep.general_options_to_string());
+  return cparser_rep.general_options_to_string();
 }
 
 /**
@@ -1058,7 +1117,7 @@ int ECA_CHAINSETUP::number_of_realtime_inputs(void) const
     AUDIO_IO_DEVICE* p = dynamic_cast<AUDIO_IO_DEVICE*>(inputs_direct_rep[n]);
     if (p != 0) res++;
   }
-  return(res);
+  return res;
 }
 
 /**
@@ -1071,7 +1130,7 @@ int ECA_CHAINSETUP::number_of_realtime_outputs(void) const
     AUDIO_IO_DEVICE* p = dynamic_cast<AUDIO_IO_DEVICE*>(outputs_direct_rep[n]);
     if (p != 0) res++;
   }
-  return(res);
+  return res;
 }
 
 /**
@@ -1079,7 +1138,7 @@ int ECA_CHAINSETUP::number_of_realtime_outputs(void) const
  */
 int ECA_CHAINSETUP::number_of_non_realtime_inputs(void) const
 {
-  return(inputs.size() - number_of_realtime_inputs());
+  return inputs.size() - number_of_realtime_inputs();
 }
 
 /**
@@ -1087,7 +1146,7 @@ int ECA_CHAINSETUP::number_of_non_realtime_inputs(void) const
  */
 int ECA_CHAINSETUP::number_of_non_realtime_outputs(void) const
 {
-  return(outputs.size() - number_of_realtime_outputs());
+  return outputs.size() - number_of_realtime_outputs();
 }
 
 /**
@@ -1100,15 +1159,15 @@ AUDIO_IO_MANAGER* ECA_CHAINSETUP::get_audio_object_manager(AUDIO_IO* aio) const
   for(vector<AUDIO_IO_MANAGER*>::const_iterator q = aio_managers_rep.begin(); q != aio_managers_rep.end(); q++) {
     if ((*q)->get_object_id(aio) != -1) {
       ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		    "(eca-chainsetup) Found object manager '" +
+		    "Found object manager '" +
 		    (*q)->name() + 
 		    "' for aio '" +
 		    aio->label() + "'.");
       
-      return(*q);
+      return *q;
     }
   }
-  return(0);
+  return 0;
 }
 
 /**
@@ -1122,15 +1181,15 @@ AUDIO_IO_MANAGER* ECA_CHAINSETUP::get_audio_object_type_manager(AUDIO_IO* aio) c
   for(vector<AUDIO_IO_MANAGER*>::const_iterator q = aio_managers_rep.begin(); q != aio_managers_rep.end(); q++) {
     if ((*q)->is_managed_type(aio) == true) {
       ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		    "(eca-chainsetup) Found object manager '" +
+		    "Found object manager '" +
 		    (*q)->name() + 
 		    "' for aio type '" +
 		    aio->name() + "'.");
       
-      return(*q);
+      return *q;
     }
   }
-  return(0);
+  return 0;
 }
 
 /**
@@ -1144,7 +1203,7 @@ void ECA_CHAINSETUP::register_engine_driver(AUDIO_IO_MANAGER* amgr)
   if (driver != 0) {
     engine_driver_repp = driver;
     ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		  "(eca-chainsetup) Registered audio i/o manager '" +
+		  "Registered audio i/o manager '" +
 		  amgr->name() +
 		  "' as the current engine driver.");
   }
@@ -1162,7 +1221,7 @@ void ECA_CHAINSETUP::register_audio_object_to_manager(AUDIO_IO* aio)
     mgr = aio->create_object_manager();
     if (mgr != 0) {
       ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		    "(eca-chainsetup) Creating object manager '" +
+		    "Creating object manager '" +
 		    mgr->name() + 
 		    "' for aio '" +
 		    aio->name() + "'.");
@@ -1189,7 +1248,7 @@ void ECA_CHAINSETUP::unregister_audio_object_from_manager(AUDIO_IO* aio)
     int id = mgr->get_object_id(aio);
     if (id != -1) {
       ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		    "(eca-chainsetup) Unregistering object '" +
+		    "Unregistering object '" +
 		    aio->name() + 
 		    "' from manager '" +
 		    mgr->name() + "'.");
@@ -1213,7 +1272,7 @@ void ECA_CHAINSETUP::propagate_audio_io_manager_options(void)
       for(int n = 0; n < numparams; n++) {
 	(*q)->set_parameter(n + 1, kvu_get_argument_number(n + 1, optstring));
 	ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		    "(eca-chainsetup) Manager '" +
+		    "Manager '" +
 		    (*q)->name() + "', " + 
 		    kvu_numtostr(n + 1) + ". parameter set to '" +
 		    (*q)->get_parameter(n + 1) + "'.");
@@ -1240,7 +1299,7 @@ AUDIO_IO* ECA_CHAINSETUP::add_audio_object_helper(AUDIO_IO* aio)
     retobj = new AUDIO_IO_DB_CLIENT(&impl_repp->pserver_rep, aio, false);
     ++db_clients_rep;
   }
-  return(retobj);
+  return retobj;
 }
 
 /** 
@@ -1356,7 +1415,7 @@ void ECA_CHAINSETUP::remove_audio_input(const string& label)
 
   for(size_t n = 0; n < inputs.size(); n++) {
     if (inputs[n]->label() == label) {
-      ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-chainsetup) Removing input " + label + ".");
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Removing input " + label + ".");
 
       remove_audio_object_helper(inputs[n]);
 
@@ -1412,7 +1471,7 @@ void ECA_CHAINSETUP::remove_audio_output(const string& label)
 
   for(size_t n = 0; n < outputs.size(); n++) {
     if (outputs[n]->label() == label) {
-      ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-chainsetup) Removing output " + label + ".");
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Removing output " + label + ".");
 
       remove_audio_object_helper(outputs[n]);
 
@@ -1465,7 +1524,7 @@ void ECA_CHAINSETUP::audio_object_info(const AUDIO_IO* aio)
   DBC_REQUIRE(aio != 0);
   // --------
 
-  string temp = "(eca-chainsetup) Audio object \"" + aio->label();
+  string temp = "Audio object \"" + aio->label();
   temp += "\", mode \"";
   if (aio->io_mode() == AUDIO_IO::io_read) temp += "read";
   if (aio->io_mode() == AUDIO_IO::io_write) temp += "write";
@@ -1526,7 +1585,7 @@ const CHAIN* ECA_CHAINSETUP::get_chain_with_name(const string& name) const
     if ((*p)->name() == name) return(*p);
     ++p;
   }
-  return(0);
+  return 0;
 }
 
 /**
@@ -1551,7 +1610,7 @@ void ECA_CHAINSETUP::attach_input_to_selected_chains(const AUDIO_IO* obj)
 	  (*q)->disconnect_input();
 	}
       }
-      temp += "(eca-chainsetup) Assigning file to chains:";
+      temp += "Assigning file to chains:";
       for(vector<string>::const_iterator p = selected_chainids.begin(); p!= selected_chainids.end(); p++) {
 	for(vector<CHAIN*>::iterator q = chains.begin(); q != chains.end(); q++) {
 	  if (*p == (*q)->name()) {
@@ -1587,7 +1646,7 @@ void ECA_CHAINSETUP::attach_output_to_selected_chains(const AUDIO_IO* obj)
 	  (*q)->disconnect_output();
 	}
       }
-      temp += "(eca-chainsetup) Assigning file to chains:";
+      temp += "Assigning file to chains:";
       for(vector<string>::const_iterator p = selected_chainids.begin(); p!= selected_chainids.end(); p++) {
 	for(vector<CHAIN*>::iterator q = chains.begin(); q != chains.end(); q++) {
 	  if (*p == (*q)->name()) {
@@ -1611,7 +1670,7 @@ bool ECA_CHAINSETUP::ok_audio_object(const AUDIO_IO* aobj) const
   if (ok_audio_object_helper(aobj, inputs) == true ||
       ok_audio_object_helper(aobj, outputs) == true ) return(true);
 
-  return(false);
+  return false;
   
 }
 
@@ -1621,7 +1680,7 @@ bool ECA_CHAINSETUP::ok_audio_object_helper(const AUDIO_IO* aobj,
   for(size_t n = 0; n < aobjs.size(); n++) {
     if (aobjs[n] == aobj) return(true);
   }
-  return(false);
+  return false;
 }
 
 void ECA_CHAINSETUP::check_object_samplerate(const AUDIO_IO* obj,
@@ -1762,14 +1821,14 @@ void ECA_CHAINSETUP::enable(void) throw(ECA_ERROR&)
   }
   catch(AUDIO_IO::SETUP_ERROR& e) {
     ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		"(eca-chainsetup) Connecting chainsetup failed, throwing an SETUP_ERROR exception.");
+		"Connecting chainsetup failed, throwing an SETUP_ERROR exception.");
     throw(ECA_ERROR("ECA-CHAINSETUP", 
 		    string("Enabling chainsetup: ")
 		    + e.message()));
   }
   catch(...) { 
     ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-		"(eca-chainsetup) Connecting chainsetup failed, throwing a generic exception.");
+		"Connecting chainsetup failed, throwing a generic exception.");
     throw; 
   }
 
@@ -1802,18 +1861,18 @@ void ECA_CHAINSETUP::disable(void)
   if (is_enabled_rep == true) {
     ECA_LOG_MSG(ECA_LOGGER::system_objects, "Closing chainsetup \"" + name() + "\"");
     for(vector<AUDIO_IO*>::iterator q = inputs.begin(); q != inputs.end(); q++) {
-      ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) Closing audio device/file \"" + (*q)->label() + "\".");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "Closing audio device/file \"" + (*q)->label() + "\".");
       if ((*q)->is_open() == true) (*q)->close();
     }
     
     for(vector<AUDIO_IO*>::iterator q = outputs.begin(); q != outputs.end(); q++) {
-      ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) Closing audio device/file \"" + (*q)->label() + "\".");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "Closing audio device/file \"" + (*q)->label() + "\".");
       if ((*q)->is_open() == true) (*q)->close();
     }
 
     if (impl_repp->midi_server_rep.is_enabled() == true) impl_repp->midi_server_rep.disable();
     for(vector<MIDI_IO*>::iterator q = midi_devices.begin(); q != midi_devices.end(); q++) {
-      ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) Closing midi device \"" + (*q)->label() + "\".");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "Closing midi device \"" + (*q)->label() + "\".");
       if ((*q)->is_open() == true) (*q)->close();
     }
 
@@ -1888,7 +1947,7 @@ long int ECA_CHAINSETUP::check_for_locked_buffersize(void) const
       result = static_cast<long int>(jack_get_buffer_size(client));
       
       ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		"(eca-chainsetup) jackd buffersize check returned " +
+		"jackd buffersize check returned " +
 		  kvu_numtostr(result) + ".");
       
       jack_client_close(client);
@@ -1897,13 +1956,13 @@ long int ECA_CHAINSETUP::check_for_locked_buffersize(void) const
     }
     else {
       ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		  "(eca-chainsetup) unable to perform jackd buffersize check.");
+		  "unable to perform jackd buffersize check.");
     }
     
     DBC_CHECK(client == 0);
   }
 #endif
-  return(result);
+  return result;
 }
 
 /**
@@ -1915,7 +1974,7 @@ void ECA_CHAINSETUP::set_samples_per_second(SAMPLE_SPECS::sample_rate_t new_valu
   DBC_CHECK(is_locked() != true);
 
   ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		"(eca-chainsetup) sample rate change, chainsetup " +
+		"sample rate change, chainsetup " +
 		name() +
 		" to rate " + 
 		kvu_numtostr(new_value) + ".");
@@ -1947,7 +2006,7 @@ void ECA_CHAINSETUP::seek_position(void)
   // --------
 
   ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		"(eca-chainsetup) seek position, chainsetup '" +
+		"seek position, chainsetup '" +
 		name() +
 		"' to pos in sec " + 
 		kvu_numtostr(position_in_seconds()) + ".");
@@ -2050,82 +2109,82 @@ void ECA_CHAINSETUP::interpret_options(vector<string>& opts)
 
 void ECA_CHAINSETUP::set_buffersize(long int value)
 {
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) overriding buffersize.");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "overriding buffersize.");
   impl_repp->bmode_override_rep.set_buffersize(value); 
 }
 
 void ECA_CHAINSETUP::toggle_raised_priority(bool value) { 
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) overriding raised priority.");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "overriding raised priority.");
   impl_repp->bmode_override_rep.toggle_raised_priority(value); 
 }
 
 void ECA_CHAINSETUP::set_sched_priority(int value)
 {
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) sched_priority.");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "sched_priority.");
   impl_repp->bmode_override_rep.set_sched_priority(value); 
 }
 
 void ECA_CHAINSETUP::toggle_double_buffering(bool value)
 {
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) overriding doublebuffering.");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "overriding doublebuffering.");
   impl_repp->bmode_override_rep.toggle_double_buffering(value); 
 }
 
 void ECA_CHAINSETUP::set_double_buffer_size(long int v)
 {
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) overriding db-size.");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "overriding db-size.");
   impl_repp->bmode_override_rep.set_double_buffer_size(v); 
 }
 
 void ECA_CHAINSETUP::toggle_max_buffers(bool v)
 {
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) overriding max_buffers.");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "overriding max_buffers.");
   impl_repp->bmode_override_rep.toggle_max_buffers(v); 
 }
 
 long int ECA_CHAINSETUP::buffersize(void) const
 {
   if (impl_repp->bmode_override_rep.is_set_buffersize() == true)
-    return(impl_repp->bmode_override_rep.buffersize());
+    return impl_repp->bmode_override_rep.buffersize();
   
-  return(impl_repp->bmode_active_rep.buffersize()); 
+  return impl_repp->bmode_active_rep.buffersize(); 
 }
 
 bool ECA_CHAINSETUP::raised_priority(void) const
 {
   if (impl_repp->bmode_override_rep.is_set_raised_priority() == true)
-    return(impl_repp->bmode_override_rep.raised_priority());
+    return impl_repp->bmode_override_rep.raised_priority();
 
-  return(impl_repp->bmode_active_rep.raised_priority()); 
+  return impl_repp->bmode_active_rep.raised_priority(); 
 }
 
 int ECA_CHAINSETUP::get_sched_priority(void) const
 {
   if (impl_repp->bmode_override_rep.is_set_sched_priority() == true)
-    return(impl_repp->bmode_override_rep.get_sched_priority());
+    return impl_repp->bmode_override_rep.get_sched_priority();
 
-  return(impl_repp->bmode_active_rep.get_sched_priority()); 
+  return impl_repp->bmode_active_rep.get_sched_priority(); 
 }
 
 bool ECA_CHAINSETUP::double_buffering(void) const { 
   if (impl_repp->bmode_override_rep.is_set_double_buffering() == true)
-    return(impl_repp->bmode_override_rep.double_buffering());
+    return impl_repp->bmode_override_rep.double_buffering();
 
-  return(impl_repp->bmode_active_rep.double_buffering()); 
+  return impl_repp->bmode_active_rep.double_buffering(); 
 }
 
 long int ECA_CHAINSETUP::double_buffer_size(void) const { 
   if (impl_repp->bmode_override_rep.is_set_double_buffer_size() == true)
-    return(impl_repp->bmode_override_rep.double_buffer_size());
+    return impl_repp->bmode_override_rep.double_buffer_size();
 
-  return(impl_repp->bmode_active_rep.double_buffer_size()); 
+  return impl_repp->bmode_active_rep.double_buffer_size(); 
 }
 
 bool ECA_CHAINSETUP::max_buffers(void) const { 
   if (impl_repp->bmode_override_rep.is_set_max_buffers() == true)
-    return(impl_repp->bmode_override_rep.max_buffers());
+    return impl_repp->bmode_override_rep.max_buffers();
 
-  return(impl_repp->bmode_active_rep.max_buffers()); 
+  return impl_repp->bmode_active_rep.max_buffers(); 
 }
 
 void ECA_CHAINSETUP::set_default_audio_format(ECA_AUDIO_FORMAT& value) { 
@@ -2134,7 +2193,7 @@ void ECA_CHAINSETUP::set_default_audio_format(ECA_AUDIO_FORMAT& value) {
 
 const ECA_AUDIO_FORMAT& ECA_CHAINSETUP::default_audio_format(void) const
 { 
-  return(impl_repp->default_audio_format_rep); 
+  return impl_repp->default_audio_format_rep; 
 }
 
 /**
@@ -2235,10 +2294,11 @@ void ECA_CHAINSETUP::add_default_output(void)
   // --------
 
   if (inputs.size() > 0 && outputs.size() == 0) {
+
     // No -o[:] options specified; let's use the default output
+    
     select_all_chains();
-    ECA_RESOURCES ecaresources;
-    interpret_object_option(string("-o:" + ecaresources.resource("default-output")));
+    interpret_object_option(string("-o:") + ECA_OBJECT_FACTORY::probe_default_output_device());
   }
 }
 
@@ -2278,9 +2338,11 @@ void ECA_CHAINSETUP::load_from_file(const string& filename,
     if (temp.size() > 0 && temp[0] == '#') {
       continue;
     }
+    // FIXME: we should add quoting when saving the chainsetup or
+    // give on quoting altogether...
     vector<string> words = kvu_string_to_tokens_quoted(temp);
     for(unsigned int n = 0; n < words.size(); n++) {
-      ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-chainsetup) Adding \"" + words[n] + "\" to options (loaded from \"" + filename + "\".");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "Adding \"" + words[n] + "\" to options (loaded from \"" + filename + "\".");
       options.push_back(words[n]);
     }
   }
