@@ -1,6 +1,10 @@
 #ifndef INCLUDED_ECA_CONTROL_BASE_H
 #define INCLUDED_ECA_CONTROL_BASE_H
 
+#include <list>
+#include <string>
+#include <vector>
+
 #include <pthread.h>
 
 #include <kvu_locks.h>
@@ -27,6 +31,7 @@ class ECA_CONTROL_BASE {
   bool req_batchmode_rep;
   pthread_t th_cqueue_rep;
   ATOMIC_INTEGER engine_exited_rep;
+  int float_to_string_precision_rep;
 
   static void* start_normal_thread(void *ptr);
 
@@ -36,6 +41,8 @@ class ECA_CONTROL_BASE {
  protected:
 
   void close_engine(void);
+  void set_float_to_string_precision(int precision);
+  std::string float_to_string(double n) const;
 
   ECA_ENGINE* engine_repp;
   ECA_SESSION* session_repp;
@@ -71,6 +78,7 @@ class ECA_CONTROL_BASE {
  protected:
 
   void set_last_string_list(const std::vector<std::string>& s);
+  void set_last_string(const std::list<std::string>& s);
   void set_last_string(const std::string& s);
   void set_last_float(double v);
   void set_last_integer(int v);
@@ -114,6 +122,8 @@ class ECA_CONTROL_BASE {
    */
   std::string resource_value(const std::string& key) const;
 
+  int float_to_string_precision(void) const { return(float_to_string_precision_rep); }
+
   // -------------------------------------------------------------------
   // Session status
   // -------------------------------------------------------------------
@@ -123,11 +133,8 @@ class ECA_CONTROL_BASE {
   bool is_selected(void) const;
   bool is_finished(void) const;
   bool is_valid(void) const;
+  bool is_engine_started(void) const;
 
-  /**
-   * Returns true if engine has been started. 
-   */
-  bool is_engine_started(void) const { return(engine_repp != 0); }
   std::string engine_status(void) const;
 
   ECA_CONTROL_BASE (ECA_SESSION* psession);
