@@ -1,6 +1,9 @@
 // ------------------------------------------------------------------------
 // eca-control.cpp: Class for controlling the whole ecasound library
-// Copyright (C) 1999-2001 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 1999-2004 Kai Vehmanen
+//
+// Attributes:
+//     eca-style-version: 3
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -117,7 +120,7 @@ void ECA_CONTROL::command(const string& cmd)
 	string first = *p;
 	++p;
 	if (p != cmds.end()) {
-	  set_action_argument(vector<string> (p, cmds.end()));
+	  set_action_argument(kvu_vector_to_string(vector<string> (p, cmds.end()), " "));
 	}
 	action(action_id);
       }
@@ -125,7 +128,7 @@ void ECA_CONTROL::command(const string& cmd)
   }
 }
 
-void ECA_CONTROL::set_action_argument(const vector<string>& s)
+void ECA_CONTROL::set_action_argument(const string& s)
 {
   action_args_rep = s;
   action_arg_f_set_rep = false;
@@ -139,30 +142,34 @@ void ECA_CONTROL::set_action_argument(double v)
 
 void ECA_CONTROL::clear_action_arguments(void)
 {
-  action_args_rep.clear();
+  // use resize() instead of clear(); clear() was a late
+  // addition to C++ standard and not supported by all
+  // compilers (for example egcs-2.91.66) 
+  action_args_rep.resize(0);
+
   action_arg_f_rep = 0.0f;
   action_arg_f_set_rep = false;
 }
 
-double ECA_CONTROL::first_argument_as_float(void) const
+double ECA_CONTROL::first_action_argument_as_float(void) const
 {
   if (action_arg_f_set_rep == true)
-    return(action_arg_f_rep);
+    return action_arg_f_rep;
 
-  return(atof(action_args_rep[0].c_str()));
+  return atof(action_args_rep.c_str());
 }
 
-long int ECA_CONTROL::first_argument_as_long_int(void) const
+long int ECA_CONTROL::first_action_argument_as_long_int(void) const
 {
-  return(atol(action_args_rep[0].c_str()));
+  return atol(action_args_rep.c_str());
 }
 
-SAMPLE_SPECS::sample_pos_t ECA_CONTROL::first_argument_as_samples(void) const
+SAMPLE_SPECS::sample_pos_t ECA_CONTROL::first_action_argument_as_samples(void) const
 {
 #ifdef HAVE_ATOLL
-  return(atoll(action_args_rep[0].c_str()));
+  return atoll(action_args_rep.c_str());
 #else
-  return(atol(action_args_rep[0].c_str()));
+  return atol(action_args_rep.c_str());
 #endif
 }
 
@@ -200,7 +207,7 @@ void ECA_CONTROL::chainsetup_option(const string& cmd)
       set_last_error("When adding controllers, only one chain can be selected.");
   }
   else {
-    set_action_argument(kvu_string_to_tokens_quoted(cmd));
+    set_action_argument(cmd);
     action(ec_cs_option);
   }
 }
@@ -246,25 +253,25 @@ void ECA_CONTROL::check_action_preconditions(int action_id)
       action_ok = false;
     }
     else {
-      set_last_error("Warning! No chainsetup selected. Connected chainsetup will be selected.");
+      ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: No chainsetup selected. Connected chainsetup will be selected.");
       select_chainsetup(connected_chainsetup());
     }
   }
   /* case 5: action requires a connected chainsetup, but none connected */
   else if (is_connected() == false &&
-      action_requires_connected(action_id)) {
+	   action_requires_connected(action_id)) {
     if (!is_selected()) {
       set_last_error("Can't perform requested action; no chainsetup connected.");
       action_ok = false;
     }
     else {
-      set_last_error("Warning! No chainsetup connected. Trying to connect currently selected chainsetup.");
       if (is_valid() == true) {
+	ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: No chainsetup connected. Trying to connect currently selected chainsetup.");
 	connect_chainsetup();
       }
       if (is_connected() != true) {
 	/* connect_chainsetup() sets last_error() so we just add to it */
-	set_last_error(last_error() + "\nWarning! Selected chainsetup cannot be connected. Can't perform requested action. ");
+	set_last_error(last_error() + " Selected chainsetup cannot be connected. Can't perform requested action. ");
 	action_ok = false;
       }
     }
@@ -273,7 +280,7 @@ void ECA_CONTROL::check_action_preconditions(int action_id)
    *         but selected chainsetup is also connected */
   else if (selected_chainsetup() == connected_chainsetup() &&
 	   action_requires_selected_not_connected(action_id)) {
-    set_last_error("Warning! This operation requires that chainsetup is disconnected. Temporarily disconnecting...");
+    ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: This operation requires that chainsetup is disconnected. Temporarily disconnecting...");
     if (is_running()) action_restart = true;
     disconnect_chainsetup();
     action_reconnect = true;
@@ -283,8 +290,9 @@ void ECA_CONTROL::check_action_preconditions(int action_id)
 void ECA_CONTROL::action(int action_id, 
 			 const vector<string>& args) 
 {
+  ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: ECA_CONTROL::action() method is obsolete.\n");
   clear_action_arguments();
-  set_action_argument(args);
+  set_action_argument(kvu_vector_to_string(args, " "));
   action(action_id);
 }
 
@@ -305,14 +313,14 @@ void ECA_CONTROL::action(int action_id)
   case ec_start: 
     { 
       if (is_running() != true) start();
-      // ECA_LOG_MSG(ECA_LOGGER::info, "(eca-control) Can't perform requested action; no chainsetup connected.");
+      // ECA_LOG_MSG(ECA_LOGGER::info, "Can't perform requested action; no chainsetup connected.");
       break; 
     }
   case ec_stop: { if (is_running() == true) stop(); break; }
   case ec_run: { run(); break; }
   case ec_debug:
     {
-      int level = atoi((action_args_rep[0]).c_str());
+      int level = atoi(action_args_rep.c_str());
       ECA_LOGGER::instance().set_log_level_bitmask(level);
       set_last_string("Debug level set to " + kvu_numtostr(level) + ".");
       break;
@@ -323,23 +331,23 @@ void ECA_CONTROL::action(int action_id)
     // ---
   case ec_cs_add:
     {
-      add_chainsetup(action_args_rep[0]);
+      add_chainsetup(action_args_rep);
       break;
     }
   case ec_cs_remove: { remove_chainsetup(); break; }
   case ec_cs_list: { set_last_string_list(chainsetup_names()); break; }
-  case ec_cs_select: { select_chainsetup(action_args_rep[0]); break; }
+  case ec_cs_select: { select_chainsetup(action_args_rep); break; }
   case ec_cs_selected: { set_last_string(selected_chainsetup()); break; }
   case ec_cs_index_select: { 
-    if (action_args_rep[0].empty() != true) {
-      select_chainsetup_by_index(atoi(action_args_rep[0].c_str()));
+    if (action_args_rep.size() > 0) {
+      select_chainsetup_by_index(atoi(action_args_rep.c_str()));
     }
     break; 
   }
   case ec_cs_edit: { edit_chainsetup(); break; }
-  case ec_cs_load: { load_chainsetup(action_args_rep[0]); break; }
+  case ec_cs_load: { load_chainsetup(action_args_rep); break; }
   case ec_cs_save: { save_chainsetup(""); break; }
-  case ec_cs_save_as: { save_chainsetup(action_args_rep[0]); break; }
+  case ec_cs_save_as: { save_chainsetup(action_args_rep); break; }
   case ec_cs_is_valid: { 
     if (is_valid() == true) 
       set_last_integer(1);
@@ -359,34 +367,35 @@ void ECA_CONTROL::action(int action_id)
     }
   case ec_cs_connected: { set_last_string(connected_chainsetup()); break; }
   case ec_cs_disconnect: { disconnect_chainsetup(); break; }
-  case ec_cs_set_param: { set_chainsetup_parameter(action_args_rep[0]); break; }
-  case ec_cs_set_audio_format: { set_chainsetup_sample_format(action_args_rep[0]); break; }
+  case ec_cs_set_param: { set_chainsetup_parameter(action_args_rep); break; }
+  case ec_cs_set_audio_format: { set_chainsetup_sample_format(action_args_rep); break; }
   case ec_cs_status: { 
     set_last_string(chainsetup_status()); 
     break; 
   }
-  case ec_cs_rewind: { change_chainsetup_position(-first_argument_as_float()); break; }
-  case ec_cs_forward: { change_chainsetup_position(first_argument_as_float()); break; }
-  case ec_cs_set_position: { set_chainsetup_position(first_argument_as_float()); break; }
-  case ec_cs_set_position_samples: { set_chainsetup_position_samples(first_argument_as_samples()); break; }
+  case ec_cs_rewind: { change_chainsetup_position(-first_action_argument_as_float()); break; }
+  case ec_cs_forward: { change_chainsetup_position(first_action_argument_as_float()); break; }
+  case ec_cs_set_position: { set_chainsetup_position(first_action_argument_as_float()); break; }
+  case ec_cs_set_position_samples: { set_chainsetup_position_samples(first_action_argument_as_samples()); break; }
   case ec_cs_get_position: { set_last_float(position_in_seconds_exact()); break; }
   case ec_cs_get_position_samples: { set_last_long_integer(selected_chainsetup_repp->position_in_samples()); break; }
   case ec_cs_get_length: { set_last_float(length_in_seconds_exact()); break; }
   case ec_cs_get_length_samples: { set_last_long_integer(length_in_samples()); break; }
   case ec_cs_set_length: 
     { 
-      set_chainsetup_processing_length_in_seconds(first_argument_as_float()); 
+      set_chainsetup_processing_length_in_seconds(first_action_argument_as_float()); 
       break; 
     }
   case ec_cs_set_length_samples:
     {
-      set_chainsetup_processing_length_in_samples(first_argument_as_samples());
+      set_chainsetup_processing_length_in_samples(first_action_argument_as_samples());
       break;
     }
   case ec_cs_toggle_loop: { toggle_chainsetup_looping(); break; } 
   case ec_cs_option: 
     {
-      selected_chainsetup_repp->interpret_options(action_args_rep);
+      vector<string> temp = kvu_string_to_tokens_quoted(action_args_rep);
+      selected_chainsetup_repp->interpret_options(temp);
       if (selected_chainsetup_repp->interpret_result() != true) {
 	set_last_error(selected_chainsetup_repp->interpret_result_verbose());
       }
@@ -396,16 +405,16 @@ void ECA_CONTROL::action(int action_id)
   // ---
   // Chains
   // ---
-  case ec_c_add: { add_chains(kvu_string_to_vector(action_args_rep[0], ',')); break; }
+  case ec_c_add: { add_chains(kvu_string_to_vector(action_args_rep, ',')); break; }
   case ec_c_remove: { remove_chains(); break; }
   case ec_c_list: { set_last_string_list(chain_names()); break; }
-  case ec_c_select: { select_chains(kvu_string_to_vector(action_args_rep[0], ',')); break; }
+  case ec_c_select: { select_chains(kvu_string_to_vector(action_args_rep, ',')); break; }
   case ec_c_selected: { set_last_string_list(selected_chains()); break; }
-  case ec_c_index_select: { select_chains_by_index(kvu_string_to_int_vector(action_args_rep[0], ',')); break; }
-  case ec_c_deselect: { deselect_chains(kvu_string_to_vector(action_args_rep[0], ',')); break; }
+  case ec_c_index_select: { select_chains_by_index(kvu_string_to_int_vector(action_args_rep, ',')); break; }
+  case ec_c_deselect: { deselect_chains(kvu_string_to_vector(action_args_rep, ',')); break; }
   case ec_c_select_add: 
     { 
-      select_chains(kvu_string_to_vector(action_args_rep[0] + "," +
+      select_chains(kvu_string_to_vector(action_args_rep + "," +
 					 kvu_vector_to_string(selected_chains(), ","), ',')); 
       break; 
     }
@@ -417,7 +426,7 @@ void ECA_CONTROL::action(int action_id)
 	set_last_error("When renaming chains, only one chain canbe selected.");
       }
       else {
-	rename_chain(action_args_rep[0]); 
+	rename_chain(action_args_rep); 
       }
       break;
     }
@@ -444,14 +453,14 @@ void ECA_CONTROL::action(int action_id)
     // ---
     // Audio input objects
     // ---
-  case ec_ai_add: { add_audio_input(action_args_rep[0]); break; }
+  case ec_ai_add: { add_audio_input(action_args_rep); break; }
   case ec_ai_remove: { remove_audio_input(); break; }
   case ec_ai_list: { set_last_string_list(audio_input_names()); break; }
-  case ec_ai_select: { select_audio_input(action_args_rep[0]); break; }
+  case ec_ai_select: { select_audio_input(action_args_rep); break; }
   case ec_ai_selected: { set_last_string(get_audio_input()->label()); break; }
   case ec_ai_index_select: { 
-    if (action_args_rep[0].empty() != true) {
-	select_audio_input_by_index(atoi(action_args_rep[0].c_str()));
+    if (action_args_rep.size() > 0) {
+	select_audio_input_by_index(atoi(action_args_rep.c_str()));
     }
     break; 
   }
@@ -459,17 +468,17 @@ void ECA_CONTROL::action(int action_id)
   case ec_ai_forward: 
     { 
       audio_input_as_selected();
-      forward_audio_object(first_argument_as_float()); 
+      forward_audio_object(first_action_argument_as_float()); 
       break; 
     }
   case ec_ai_rewind: 
     { 
       audio_input_as_selected();
-      rewind_audio_object(first_argument_as_float()); 
+      rewind_audio_object(first_action_argument_as_float()); 
       break; 
     }
-  case ec_ai_set_position: { audio_input_as_selected(); set_audio_object_position(first_argument_as_float()); break; }
-  case ec_ai_set_position_samples: { audio_input_as_selected(); set_audio_object_position_samples(first_argument_as_long_int()); break; }
+  case ec_ai_set_position: { audio_input_as_selected(); set_audio_object_position(first_action_argument_as_float()); break; }
+  case ec_ai_set_position_samples: { audio_input_as_selected(); set_audio_object_position_samples(first_action_argument_as_long_int()); break; }
   case ec_ai_get_position: { set_last_float(get_audio_input()->position().seconds()); break; }
   case ec_ai_get_position_samples: { set_last_long_integer(get_audio_input()->position().samples()); break; }
   case ec_ai_get_length: { set_last_float(get_audio_input()->length().seconds()); break; }
@@ -487,15 +496,15 @@ void ECA_CONTROL::action(int action_id)
     // ---
     // Audio output objects
     // ---
-  case ec_ao_add: { if (action_args_rep.size() == 0) add_default_output(); else add_audio_output(action_args_rep[0]); break; }
+  case ec_ao_add: { if (action_args_rep.size() == 0) add_default_output(); else add_audio_output(action_args_rep); break; }
   case ec_ao_add_default: { add_default_output(); break; }
   case ec_ao_remove: { remove_audio_output(); break; }
   case ec_ao_list: { set_last_string_list(audio_output_names()); break; }
-  case ec_ao_select: { select_audio_output(action_args_rep[0]); break; }
+  case ec_ao_select: { select_audio_output(action_args_rep); break; }
   case ec_ao_selected: { set_last_string(get_audio_output()->label()); break; }
   case ec_ao_index_select: { 
-    if (action_args_rep[0].empty() != true) {
-      select_audio_output_by_index(atoi(action_args_rep[0].c_str()));
+    if (action_args_rep.size() > 0) {
+      select_audio_output_by_index(atoi(action_args_rep.c_str()));
     }
     break; 
   }
@@ -503,17 +512,17 @@ void ECA_CONTROL::action(int action_id)
   case ec_ao_forward: 
     { 
       audio_output_as_selected();
-      forward_audio_object(first_argument_as_float()); 
+      forward_audio_object(first_action_argument_as_float()); 
       break; 
     }
   case ec_ao_rewind: 
     { 
       audio_output_as_selected();
-      rewind_audio_object(first_argument_as_float()); 
+      rewind_audio_object(first_action_argument_as_float()); 
       break; 
     }
-  case ec_ao_set_position: { audio_output_as_selected(); set_audio_object_position(first_argument_as_float()); break; }
-  case ec_ao_set_position_samples: { audio_output_as_selected(); set_audio_object_position_samples(first_argument_as_long_int()); break; }
+  case ec_ao_set_position: { audio_output_as_selected(); set_audio_object_position(first_action_argument_as_float()); break; }
+  case ec_ao_set_position_samples: { audio_output_as_selected(); set_audio_object_position_samples(first_action_argument_as_long_int()); break; }
   case ec_ao_get_position: { set_last_float(get_audio_output()->position().seconds()); break; }
   case ec_ao_get_position_samples: { set_last_long_integer(get_audio_output()->position().samples()); break; }
   case ec_ao_get_length: { set_last_float(get_audio_output()->length().seconds()); break; }
@@ -529,14 +538,14 @@ void ECA_CONTROL::action(int action_id)
     // ---
     // Chain operators
     // ---
-  case ec_cop_add: { add_chain_operator(action_args_rep[0]); break; }
+  case ec_cop_add: { add_chain_operator(action_args_rep); break; }
   case ec_cop_remove: { remove_chain_operator(); break; }
   case ec_cop_list: { set_last_string_list(chain_operator_names()); break; }
-  case ec_cop_select: { select_chain_operator(atoi((action_args_rep[0]).c_str())); break; }
+  case ec_cop_select: { select_chain_operator(atoi((action_args_rep).c_str())); break; }
   case ec_cop_selected: { set_last_integer(selected_chain_operator()); break; }
   case ec_cop_set: 
     { 
-      vector<string> a = kvu_string_to_vector(action_args_rep[0], ',');
+      vector<string> a = kvu_string_to_vector(action_args_rep, ',');
       if (a.size() < 3) {
 	set_last_error("Not enough parameters!");
 	break;
@@ -564,18 +573,18 @@ void ECA_CONTROL::action(int action_id)
     // Chain operator parameters
     // ---
   case ec_copp_list: { set_last_string_list(chain_operator_parameter_names()); break; }
-  case ec_copp_select: { select_chain_operator_parameter(atoi((action_args_rep[0]).c_str())); break; }
+  case ec_copp_select: { select_chain_operator_parameter(atoi((action_args_rep).c_str())); break; }
   case ec_copp_selected: { set_last_integer(selected_chain_operator_parameter()); break; }
-  case ec_copp_set: { set_chain_operator_parameter(first_argument_as_float()); break; }
+  case ec_copp_set: { set_chain_operator_parameter(first_action_argument_as_float()); break; }
   case ec_copp_get: { set_last_float(get_chain_operator_parameter()); break; }
 
     // ---
     // Controllers
     // ---
-  case ec_ctrl_add: { add_controller(action_args_rep[0]); break; }
+  case ec_ctrl_add: { add_controller(action_args_rep); break; }
   case ec_ctrl_remove: { remove_controller(); break; }
   case ec_ctrl_list: { set_last_string_list(controller_names()); break; }
-  case ec_ctrl_select: { select_controller(atoi((action_args_rep[0]).c_str())); break; }
+  case ec_ctrl_select: { select_controller(atoi((action_args_rep).c_str())); break; }
   case ec_ctrl_selected: { set_last_integer(selected_controller()); break; }
   case ec_ctrl_status: 
     { 
@@ -590,7 +599,8 @@ void ECA_CONTROL::action(int action_id)
 
   case ec_map_cop_list: { cop_descriptions(); break; }
   case ec_map_preset_list: { preset_descriptions(); break; }
-  case ec_map_ladspa_list: { ladspa_descriptions(); break; }
+  case ec_map_ladspa_list: { ladspa_descriptions(false); break; }
+  case ec_map_ladspa_id_list: { ladspa_descriptions(true); break; }
   case ec_map_ctrl_list: { ctrl_descriptions(); break; }
 
   // ---
@@ -604,11 +614,14 @@ void ECA_CONTROL::action(int action_id)
   // Internal commands
   // ---
   case ec_int_cmd_list: { set_last_string_list(registered_commands_list()); break; }
+  case ec_int_log_history: { set_last_string(ECA_LOGGER::instance().log_history()); break; }
   case ec_int_output_mode_wellformed: { 
     ECA_LOGGER::attach_logger(new ECA_LOGGER_WELLFORMED()); 
     wellformed_mode_rep = true;
     break; 
   }
+  case ec_int_set_float_to_string_precision: { set_float_to_string_precision(atoi(action_args_rep.c_str())); break; }
+  case ec_int_set_log_history_length: { ECA_LOGGER::instance().set_log_history_length(atoi(action_args_rep.c_str())); break; }
   case ec_int_version_string: { set_last_string(ecasound_library_version); break; }
   case ec_int_version_lib_current: { set_last_integer(ecasound_library_version_current); break; }
   case ec_int_version_lib_revision: { set_last_integer(ecasound_library_version_revision); break; }
@@ -617,7 +630,7 @@ void ECA_CONTROL::action(int action_id)
   // ---
   // Dump commands
   // ---
-  case ec_dump_target: { ctrl_dump_rep.set_dump_target(action_args_rep[0]); break; }
+  case ec_dump_target: { ctrl_dump_rep.set_dump_target(action_args_rep); break; }
   case ec_dump_status: { ctrl_dump_rep.dump_status(); break; }
   case ec_dump_position: { ctrl_dump_rep.dump_position(); break; }
   case ec_dump_length: { ctrl_dump_rep.dump_length(); break; }
@@ -633,9 +646,10 @@ void ECA_CONTROL::action(int action_id)
   case ec_dump_ao_open_state: { ctrl_dump_rep.dump_audio_output_open_state(); break; }
   case ec_dump_cop_value: 
     { 
-      if (action_args_rep.size() > 1) {
-	ctrl_dump_rep.dump_chain_operator_value(atoi(action_args_rep[0].c_str()),
-						atoi(action_args_rep[1].c_str())); 
+      vector<string> temp = kvu_string_to_tokens_quoted(action_args_rep);
+      if (temp.size() > 1) {
+	ctrl_dump_rep.dump_chain_operator_value(atoi(temp[0].c_str()),
+						atoi(temp[1].c_str()));
       }
       break; 
     }
@@ -671,16 +685,17 @@ string ECA_CONTROL::last_value_to_string(void)
   }
   else if (type == "s")
     result = last_string();
-  else if (type == "S")
-    result = kvu_vector_to_string(last_string_list(), ",");
+  else if (type == "S") {
+    result = kvu_vector_to_string(kvu_vector_search_and_replace(last_string_list(), ",", "\\,"), ",");
+  }
   else if (type == "i")
     result = kvu_numtostr(last_integer());
   else if (type == "li")
     result = kvu_numtostr(last_long_integer());
   else if (type == "f")
-    result = kvu_numtostr(last_float(), 3);
+    result = ECA_CONTROL_BASE::float_to_string(last_float());
 
-  return(result);
+  return result;
 }
 
 void ECA_CONTROL::print_last_value(void)
@@ -689,7 +704,7 @@ void ECA_CONTROL::print_last_value(void)
   string result;
 
   if (type == "e") {
-    result += "(eca-control) ERROR: ";
+    result += "ERROR: ";
   }
 
   result += last_value_to_string();
@@ -714,6 +729,9 @@ string ECA_CONTROL::chainsetup_details_to_string(const ECA_CHAINSETUP* cs) const
   result += " outputs, " + kvu_numtostr(cs->chains.size());
   result += " chains";
 
+  // FIXME: add explanations on why the chainsetup cannot be
+  //        connected
+
   if (cs->is_valid()) 
     result += "\n -> State:   valid (can be connected)";
   else
@@ -722,7 +740,7 @@ string ECA_CONTROL::chainsetup_details_to_string(const ECA_CHAINSETUP* cs) const
   result += "\n -> Options: ";
   result += cs->options_to_string();
   
-  return(result);
+  return result;
 }
 
 string ECA_CONTROL::chainsetup_status(void) const 
@@ -742,7 +760,7 @@ string ECA_CONTROL::chainsetup_status(void) const
     if (cs_citer != session_repp->chainsetups_rep.end()) result += "\n";
   }
 
-  return(result);
+  return result;
 }
 
 string ECA_CONTROL::chain_status(void) const
@@ -771,7 +789,7 @@ string ECA_CONTROL::chain_status(void) const
     if (chain_citer != selected_chainsetup_repp->chains.end()) mitem << "\n";
   }
 
-  return(mitem.to_string());
+  return mitem.to_string();
 }
 
 string ECA_CONTROL::chain_operator_status(void) const
@@ -798,7 +816,7 @@ string ECA_CONTROL::chain_operator_status(void) const
 	msg << "[" << n + 1 << "] ";
 	msg << cop->get_parameter_name(n + 1);
 	msg << " ";
-	msg << kvu_numtostr(cop->get_parameter(n + 1));
+	msg << ECA_CONTROL_BASE::float_to_string(cop->get_parameter(n + 1));
 	if (n + 1 < cop->number_of_params()) msg <<  ", ";
       }
       st_info_string = cop->status();
@@ -810,7 +828,7 @@ string ECA_CONTROL::chain_operator_status(void) const
     ++chain_citer;
     if (chain_citer != selected_chainsetup_repp->chains.end()) msg << "\n";
   }
-  return(msg.to_string());
+  return msg.to_string();
 }
 
 string ECA_CONTROL::controller_status(void) const
@@ -836,7 +854,7 @@ string ECA_CONTROL::controller_status(void) const
 	mitem << "\n\t\t[" << n + 1 << "] ";
 	mitem << gtrl->get_parameter_name(n + 1);
 	mitem << " ";
-	mitem << kvu_numtostr(gtrl->get_parameter(n + 1));
+	mitem << ECA_CONTROL_BASE::float_to_string(gtrl->get_parameter(n + 1));
 	if (n + 1 < gtrl->number_of_params()) mitem <<  ", ";
       }
       st_info_string = gtrl->status();
@@ -848,7 +866,7 @@ string ECA_CONTROL::controller_status(void) const
     ++chain_citer;
     if (chain_citer != selected_chainsetup_repp->chains.end()) mitem << "\n";
   }
-  return(mitem.to_string());
+  return mitem.to_string();
 }
 
 string ECA_CONTROL::aio_status(void) const
@@ -909,7 +927,7 @@ string ECA_CONTROL::aio_status(void) const
     ++adev_citer;
     if (adev_sizet < selected_chainsetup_repp->outputs.size()) st_info_string += "\n";
   }
-  return(st_info_string);
+  return st_info_string;
 }
 
 void ECA_CONTROL::aio_register(void)
@@ -956,7 +974,7 @@ static string eca_aio_register_sub(ECA_OBJECT_MAP& objmap)
 
     ++p;
   }
-  return(result);
+  return result;
 }
 
 void ECA_CONTROL::cop_register(void)
@@ -1093,6 +1111,8 @@ void ECA_CONTROL::operator_descriptions_helper(const ECA_OBJECT_MAP& arg, string
     string temp;
     const OPERATOR* q = dynamic_cast<const OPERATOR*>(arg.object(*p));
     if (q != 0) {
+      /* FIXME: prefer backslash escaped over '_' to handle commas */
+      
       /* 1. keyword */
       *result += kvu_string_search_and_replace(*p, ',', '_');
       /* 2. name */
@@ -1115,15 +1135,15 @@ void ECA_CONTROL::operator_descriptions_helper(const ECA_OBJECT_MAP& arg, string
 	/* 5.2 description */
 	*result += "," + kvu_string_search_and_replace(pd.description, ',', '_');
 	/* 5.3 default value */
-	*result += "," + kvu_numtostr(pd.default_value);
+	*result += "," + ECA_CONTROL_BASE::float_to_string(pd.default_value);
 	/* 5.4 is bounded above (1=yes, 0=no) */
 	*result += ",above=" + kvu_numtostr(static_cast<int>(pd.bounded_above));
 	/* 5.5 upper bound */
-	*result += ",upper=" + kvu_numtostr(pd.upper_bound);
+	*result += ",upper=" + ECA_CONTROL_BASE::float_to_string(pd.upper_bound);
 	/* 5.6 is bounded below (1=yes, 0=no) */
 	*result += ",below=" + kvu_numtostr(static_cast<int>(pd.bounded_below));
 	/* 5.7 lower bound */
-	*result += ",lower=" + kvu_numtostr(pd.lower_bound);
+	*result += ",lower=" + ECA_CONTROL_BASE::float_to_string(pd.lower_bound);
 	/* 5.8. is toggled (1=yes, 0=no) */
 	*result += "," + kvu_numtostr(static_cast<int>(pd.toggled));
 	/* 5.9. is integer value (1=yes, 0=no) */
@@ -1171,10 +1191,15 @@ void ECA_CONTROL::preset_descriptions(void)
  * Prints the description of all LADSPA plugins and 
  * their parameters.
  */
-void ECA_CONTROL::ladspa_descriptions(void)
+void ECA_CONTROL::ladspa_descriptions(bool use_id)
 {
   string result;
-  operator_descriptions_helper(ECA_OBJECT_FACTORY::ladspa_plugin_map(), &result);
+  if (use_id) {
+    operator_descriptions_helper(ECA_OBJECT_FACTORY::ladspa_plugin_id_map(), &result);
+  }
+  else {
+    operator_descriptions_helper(ECA_OBJECT_FACTORY::ladspa_plugin_map(), &result);
+  }
   set_last_string(result);
 }
 
