@@ -1,6 +1,9 @@
 // ------------------------------------------------------------------------
 // eca-engine.cpp: Main processing engine
-// Copyright (C) 1999-2003 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 1999-2004 Kai Vehmanen
+//
+// Attributes:
+//     eca-style-version: 3
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -70,6 +73,13 @@ using std::vector;
 #define PROFILE_ENGINE_STATEMENT(x) ((void)0)
 #endif
 
+/**
+ * Prototypes of static functions
+ */
+
+static void mix_to_outputs_divide_helper(const SAMPLE_BUFFER *from, SAMPLE_BUFFER *to, int divide_by, bool first_time);
+static void mix_to_outputs_sum_helper(const SAMPLE_BUFFER *from, SAMPLE_BUFFER *to, bool first_time);
+
 /**
  * Implementations of non-static functions
  */
@@ -78,7 +88,7 @@ using std::vector;
  * Driver implementation
  **********************************************************************/
 
-void ECA_ENGINE_DEFAULT_DRIVER::exec(ECA_ENGINE* engine, ECA_CHAINSETUP* csetup)
+int ECA_ENGINE_DEFAULT_DRIVER::exec(ECA_ENGINE* engine, ECA_CHAINSETUP* csetup)
 {
   engine_repp = engine;
 
@@ -113,6 +123,8 @@ void ECA_ENGINE_DEFAULT_DRIVER::exec(ECA_ENGINE* engine, ECA_CHAINSETUP* csetup)
   }
 
   if (engine_repp->is_prepared() == true) engine_repp->stop_operation();
+
+  return 0;
 }
 
 void ECA_ENGINE_DEFAULT_DRIVER::start(void)
@@ -160,7 +172,8 @@ ECA_ENGINE::ECA_ENGINE(ECA_CHAINSETUP* csetup)
     running_rep(false),
     edit_lock_rep(false),
     finished_rep(false),
-    outputs_finished_rep(false),
+    outputs_finished_rep(0),
+    driver_errors_rep(0),
     csetup_repp(csetup),
     mixslot_repp(0)
 {
@@ -169,7 +182,7 @@ ECA_ENGINE::ECA_ENGINE(ECA_CHAINSETUP* csetup)
   DBC_REQUIRE(csetup->is_enabled() == true);
   // --
 
-  ECA_LOG_MSG(ECA_LOGGER::system_objects,"(eca-engine) Engine/Initializing");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "Engine/Initializing");
 
   csetup_repp->toggle_locked_state(true);
 
@@ -193,7 +206,7 @@ ECA_ENGINE::ECA_ENGINE(ECA_CHAINSETUP* csetup)
  */
 ECA_ENGINE::~ECA_ENGINE(void)
 {
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-engine) ECA_ENGINE destructor!");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "ECA_ENGINE destructor!");
 
   if (csetup_repp != 0) {
     command(ECA_ENGINE::ep_exit, 0.0f);
@@ -255,14 +268,23 @@ void ECA_ENGINE::exec(bool batch_mode)
 
   ECA_LOG_MSG(ECA_LOGGER::subsystems, "Engine init - Driver start");
 
-  driver_repp->exec(this, csetup_repp);
+  int res = driver_repp->exec(this, csetup_repp);
+  if (res < 0) {
+    ++driver_errors_rep;
+    ECA_LOG_MSG(ECA_LOGGER::info, 
+		"WARNING: Engine has raised an error! "
+		"Possible causes: connection lost to system services, unable to adapt "
+		"to changes in operating environment, etc.");
+  }
 
   csetup_repp->toggle_locked_state(false);
 
   signal_exit();
 
   if (outputs_finished_rep > 0) 
-    ECA_LOG_MSG(ECA_LOGGER::info, "(eca-engine) Warning! An output object has raised an error! Out of disk space, permission denied, etc?");
+    ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: An output object has raised an error! "
+		"Possible causes: Out of disk space, permission denied, unable to launch external "
+		"applications needed in procesing, etc.");
 
   DBC_CHECK(status() == ECA_ENGINE::engine_status_stopped ||
 	    status() == ECA_ENGINE::engine_status_finished ||
@@ -271,7 +293,7 @@ void ECA_ENGINE::exec(bool batch_mode)
   cleanup();
 
   ECA_LOG_MSG(ECA_LOGGER::user_objects, 
-		"(eca-engine) Engine state when finishing: " + 
+		"Engine state when finishing: " + 
 		kvu_numtostr(static_cast<int>(status())));
 
   // --
@@ -308,7 +330,7 @@ void ECA_ENGINE::wait_for_stop(int timeout)
 				   &impl_repp->ecasound_stop_cond_repp, 
 				   timeout);
   ECA_LOG_MSG(ECA_LOGGER::system_objects, 
-	      kvu_pthread_timed_wait_result(ret, "(eca-engine) wait_for_stop"));
+	      kvu_pthread_timed_wait_result(ret, "wait_for_stop"));
 }
 
 /**
@@ -354,10 +376,10 @@ bool ECA_ENGINE::is_valid(void) const
       outputs_repp == 0 ||
       outputs_repp->size() == 0) {
 
-    return(false);
+    return false;
   }
   
-  return(true);
+  return true;
 }
 
 /**
@@ -371,10 +393,10 @@ bool ECA_ENGINE::is_finite_length(void) const
 
   if (csetup_repp->max_length_set() == true ||
       csetup_repp->number_of_realtime_inputs() == 0) {
-    return(true);
+    return true;
   }
 
-  return(false);
+  return false;
 }
 
 /**
@@ -386,22 +408,23 @@ bool ECA_ENGINE::is_finite_length(void) const
 ECA_ENGINE::Engine_status_t ECA_ENGINE::status(void) const
 { 
   if (csetup_repp == 0) 
-    return(ECA_ENGINE::engine_status_notready);
+    return ECA_ENGINE::engine_status_notready;
 
   /* calculated in update_engine_status() */
   if (finished_rep == true)
-    return(ECA_ENGINE::engine_status_finished);
+    return ECA_ENGINE::engine_status_finished;
   
-  if (outputs_finished_rep > 0) 
-    return(ECA_ENGINE::engine_status_error);
+  if (outputs_finished_rep > 0 ||
+      driver_errors_rep > 0) 
+    return ECA_ENGINE::engine_status_error;
 
   if (is_running() == true) 
-    return(ECA_ENGINE::engine_status_running);
+    return ECA_ENGINE::engine_status_running;
 
   if (is_prepared() == true) 
-    return(ECA_ENGINE::engine_status_stopped);
+    return ECA_ENGINE::engine_status_stopped;
 
-  return(ECA_ENGINE::engine_status_stopped);
+  return ECA_ENGINE::engine_status_stopped;
 }
 
 /**********************************************************************
@@ -476,7 +499,7 @@ void ECA_ENGINE::update_engine_state(void)
       outputs_finished_rep == 0 && 
       finished_rep != true) {
     if (is_running() == true) {
-      ECA_LOG_MSG(ECA_LOGGER::system_objects,"(eca-engine) all inputs finished - stop");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects,"all inputs finished - stop");
       // FIXME: this is still wrong, command() is not fully rt-safe
       // we are not allowed to call request_stop here
       command(ECA_ENGINE::ep_stop, 0.0f);
@@ -489,7 +512,7 @@ void ECA_ENGINE::update_engine_state(void)
 
   if (status() == ECA_ENGINE::engine_status_error) {
     if (is_running() == true) {
-      ECA_LOG_MSG(ECA_LOGGER::system_objects,"(eca-engine) output error - stop");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects,"output error - stop");
       // FIXME: this is still wrong, command() is not fully rt-safe
       // we are not allowed to call request_stop here
       command(ECA_ENGINE::ep_stop, 0.0f);
@@ -566,9 +589,9 @@ void ECA_ENGINE::prepare_operation(void)
   /* 2. enable rt-scheduling */
   if (csetup_repp->raised_priority() == true) {
     if (kvu_set_thread_scheduling(SCHED_FIFO, csetup_repp->get_sched_priority()) != 0)
-      ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-engine) Unable to change scheduling policy!");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "Unable to change scheduling policy!");
     else
-      ECA_LOG_MSG(ECA_LOGGER::info, "(eca-engine) Using realtime-scheduling (SCHED_FIFO).");
+      ECA_LOG_MSG(ECA_LOGGER::info, "Using realtime-scheduling (SCHED_FIFO).");
   }
 
   /* 3. reinitialize chains if necessary */
@@ -620,7 +643,7 @@ void ECA_ENGINE::start_operation(void)
   DBC_REQUIRE(is_running() != true);
   // ---
 
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-engine) starting engine operation!");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "starting engine operation!");
 
   start_realtime_objects();
   running_rep = true;
@@ -652,7 +675,7 @@ void ECA_ENGINE::stop_operation(void)
   DBC_REQUIRE(is_prepared() == true);
   // ---
 
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-engine) stopping engine operation!");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "stopping engine operation!");
 
   running_rep = false;
 
@@ -674,9 +697,9 @@ void ECA_ENGINE::stop_operation(void)
   /* lower priority back to normal */
   if (csetup_repp->raised_priority() == true) {
     if (kvu_set_thread_scheduling(SCHED_OTHER, 0) != 0)
-      ECA_LOG_MSG(ECA_LOGGER::info, "(eca-engine) Unable to change scheduling back to SCHED_OTHER!");
+      ECA_LOG_MSG(ECA_LOGGER::info, "Unable to change scheduling back to SCHED_OTHER!");
     else
-      ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-engine) Changed back to non-realtime scheduling SCHED_OTHER.");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects, "Changed back to non-realtime scheduling SCHED_OTHER.");
   }
 
   /* release chainsetup lock */
@@ -699,7 +722,7 @@ void ECA_ENGINE::stop_operation(void)
  */
 bool ECA_ENGINE::is_prepared(void) const
 {
-  return(prepared_rep);
+  return prepared_rep;
 }
 
 /**
@@ -710,7 +733,7 @@ bool ECA_ENGINE::is_prepared(void) const
  */
 bool ECA_ENGINE::is_running(void) const
 {
-  return(running_rep);
+  return running_rep;
 }
 
 /**********************************************************************
@@ -720,7 +743,7 @@ bool ECA_ENGINE::is_running(void) const
 long int ECA_ENGINE::buffersize(void) const
 {
   DBC_CHECK(csetup_repp != 0);
-  return(csetup_repp->buffersize());
+  return csetup_repp->buffersize();
 }
 
 int ECA_ENGINE::max_channels(void) const
@@ -734,7 +757,7 @@ int ECA_ENGINE::max_channels(void) const
     if (csetup_repp->outputs[n]->channels() > result)
       result = csetup_repp->outputs[n]->channels();
   }
-  return(result);
+  return result;
 }
 
 /**********************************************************************
@@ -757,7 +780,7 @@ void ECA_ENGINE::request_start(void)
   DBC_REQUIRE(status() != engine_status_running);
   // ---
 
-  ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-engine) Request start");
+  ECA_LOG_MSG(ECA_LOGGER::user_objects, "Request start");
 
   // --
   // start the driver
@@ -782,7 +805,7 @@ void ECA_ENGINE::request_stop(void)
 	      status() == engine_status_finished);
   // ---
 
-  ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-engine) Request stop");
+  ECA_LOG_MSG(ECA_LOGGER::user_objects, "Request stop");
 
   driver_repp->stop();
 }
@@ -798,7 +821,7 @@ void ECA_ENGINE::request_stop(void)
 void ECA_ENGINE::signal_stop(void)
 {
   pthread_mutex_lock(&impl_repp->ecasound_stop_mutex_repp);
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-engine) Signaling stop");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "Signaling stop");
   pthread_cond_broadcast(&impl_repp->ecasound_stop_cond_repp);
   pthread_mutex_unlock(&impl_repp->ecasound_stop_mutex_repp);
 }
@@ -814,7 +837,7 @@ void ECA_ENGINE::signal_stop(void)
 void ECA_ENGINE::signal_exit(void)
 {
   pthread_mutex_lock(&impl_repp->ecasound_exit_mutex_repp);
-  ECA_LOG_MSG(ECA_LOGGER::system_objects, "(eca-engine) Signaling exit");
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, "Signaling exit");
   pthread_cond_broadcast(&impl_repp->ecasound_exit_cond_repp);
   pthread_mutex_unlock(&impl_repp->ecasound_exit_mutex_repp);
 }
@@ -846,7 +869,7 @@ void ECA_ENGINE::conditional_start(void)
 void ECA_ENGINE::conditional_stop(void)
 {
   if (status() == ECA_ENGINE::engine_status_running) {
-    ECA_LOG_MSG(ECA_LOGGER::system_objects,"(eca-engine) conditional stop");
+    ECA_LOG_MSG(ECA_LOGGER::system_objects,"conditional stop");
     was_running_rep = true;
     // don't call request_stop(), as it would signal that we are 
     // stopping completely (JACK transport stop will be sent  to all)
@@ -859,9 +882,9 @@ void ECA_ENGINE::start_servers(void)
 {
   if (csetup_repp->double_buffering() == true) {
     csetup_repp->pserver_repp->start();
-    ECA_LOG_MSG(ECA_LOGGER::info, "(eca-engine) Prefilling i/o buffers.");
+    ECA_LOG_MSG(ECA_LOGGER::info, "Prefilling i/o buffers.");
     csetup_repp->pserver_repp->wait_for_full();
-    ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-engine) i/o buffers prefilled.");
+    ECA_LOG_MSG(ECA_LOGGER::user_objects, "i/o buffers prefilled.");
   }
   
   if (use_midi_rep == true) {
@@ -896,7 +919,7 @@ void ECA_ENGINE::prepare_realtime_objects(void)
 	  prefill_threshold_rep * buffersize()) {
 
 	ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		    "(eca-engine) audio output '" + 
+		    "audio output '" + 
 		    realtime_outputs_rep[n]->name() + 
 		    "' only offers " +
 		    kvu_numtostr(realtime_outputs_rep[n]->prefill_space()) +
@@ -906,7 +929,7 @@ void ECA_ENGINE::prepare_realtime_objects(void)
       }
       
       ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		  "(eca-engine) prefilling rt-outputs with " +
+		  "prefilling rt-outputs with " +
 		  kvu_numtostr(prefill_threshold_rep) +
 		  " blocks.");
 
@@ -933,7 +956,7 @@ void ECA_ENGINE::reset_realtime_devices(void)
   for (size_t n = 0; n < realtime_objects_rep.size(); n++) {
     if (realtime_objects_rep[n]->is_open() == true) {
       ECA_LOG_MSG(ECA_LOGGER::user_objects, 
-		    "(eca-engine) Reseting rt-object " + 
+		    "Reseting rt-object " + 
 		    realtime_objects_rep[n]->label());
       realtime_objects_rep[n]->close();
     }
@@ -950,12 +973,12 @@ void ECA_ENGINE::reset_realtime_devices(void)
 
 SAMPLE_SPECS::sample_pos_t ECA_ENGINE::current_position_in_samples(void) const
 {
-  return(csetup_repp->position_in_samples());
+  return csetup_repp->position_in_samples();
 }
 
 double ECA_ENGINE::current_position_in_seconds_exact(void) const
 {
-  return(csetup_repp->position_in_seconds_exact());
+  return csetup_repp->position_in_seconds_exact();
 }
 
 // FIXME: remove
@@ -963,8 +986,8 @@ double ECA_ENGINE::current_position_in_seconds_exact(void) const
 double ECA_ENGINE::current_position_chain(void) const
 {
   AUDIO_IO* ptr = (*inputs_repp)[(*chains_repp)[csetup_repp->active_chain_index_rep]->connected_input()]; 
-    return(ptr->position_in_seconds_exact());
-  return(0.0f);
+    return ptr->position_in_seconds_exact();
+  return 0.0f;
 }
 #endif
 
@@ -1055,7 +1078,7 @@ void ECA_ENGINE::posthandle_control_position(void)
   if (csetup_repp->max_length_set() == true &&
       csetup_repp->is_over_max_length() == true) {
     if (csetup_repp->looping_enabled() == true) {
-      ECA_LOG_MSG(ECA_LOGGER::system_objects,"(eca-engine) loop point reached");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects,"loop point reached");
       inputs_not_finished_rep = 1;
       csetup_repp->seek_position_in_samples(0);
       for(unsigned int adev_sizet = 0; adev_sizet < non_realtime_inputs_rep.size(); adev_sizet++) {
@@ -1063,7 +1086,7 @@ void ECA_ENGINE::posthandle_control_position(void)
       }
     }
     else {
-      ECA_LOG_MSG(ECA_LOGGER::system_objects,"(eca-engine) posthandle_c_p over_max - stop");
+      ECA_LOG_MSG(ECA_LOGGER::system_objects,"posthandle_c_p over_max - stop");
       if (status() == ECA_ENGINE::engine_status_running ||
 	  status() == ECA_ENGINE::engine_status_finished) {
 	command(ECA_ENGINE::ep_stop, 0.0f);
@@ -1092,7 +1115,16 @@ void ECA_ENGINE::posthandle_control_position(void)
 void ECA_ENGINE::interpret_queue(void)
 {
   while(impl_repp->command_queue_rep.is_empty() != true) {
-    std::pair<int,double> item = impl_repp->command_queue_rep.front();
+    const std::pair<int,double>* item_p = impl_repp->command_queue_rep.front();
+    std::pair<int,double> item;
+    if (item_p != impl_repp->command_queue_rep.invalid_item()) {
+      item = *item_p;
+    }
+    else {
+      /* queue temporarily unavailable, unable to continue processing
+       * messages */
+      break;
+    }
 
     switch(item.first) 
       {
@@ -1105,7 +1137,7 @@ void ECA_ENGINE::interpret_queue(void)
 	  if (status() == engine_status_running || 
 	      status() == engine_status_finished) request_stop();
 	  while(impl_repp->command_queue_rep.is_empty() == false) impl_repp->command_queue_rep.pop_front();
-	  ECA_LOG_MSG(ECA_LOGGER::system_objects,"(eca-engine) ecasound_queue: exit!");
+	  ECA_LOG_MSG(ECA_LOGGER::system_objects,"ecasound_queue: exit!");
 	  driver_repp->exit();
 	  return;
 	}
@@ -1231,7 +1263,7 @@ void ECA_ENGINE::init_prefill(void)
     prefill_threshold_rep = ECA_ENGINE::prefill_blocks_constant;
   
   ECA_LOG_MSG(ECA_LOGGER::system_objects,
-		"(eca-engine) Prefill loops: " +
+		"Prefill loops: " +
 		kvu_numtostr(prefill_threshold_rep) +
 		" (blocksize " + 
 		kvu_numtostr(buffersize()) + ").");
@@ -1245,7 +1277,7 @@ void ECA_ENGINE::init_servers(void)
 {
   if (csetup_repp->midi_devices.size() > 0) {
     use_midi_rep = true;
-    ECA_LOG_MSG(ECA_LOGGER::info, "(eca-engine) Initializing MIDI-server.");
+    ECA_LOG_MSG(ECA_LOGGER::info, "Initializing MIDI-server.");
     csetup_repp->midi_server_repp->init();
   }
 }
@@ -1333,12 +1365,12 @@ void ECA_ENGINE::update_cache_latency_values(void)
       else {
 	if (in_latency != realtime_inputs_rep[n]->latency()) {
 	  ECA_LOG_MSG(ECA_LOGGER::info, 
-			"(eca-engine) Warning! Latency mismatch between input objects!");
+			"WARNING: Latency mismatch between input objects!");
 	}
       }
 
       ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		  "(eca-engine) Input latency for '" +
+		  "Input latency for '" +
 		  realtime_inputs_rep[n]->name() + 
 		  "' is " + kvu_numtostr(in_latency) + ".");
 
@@ -1347,8 +1379,13 @@ void ECA_ENGINE::update_cache_latency_values(void)
     long int out_latency = -1;
     for(unsigned int n = 0; n < realtime_outputs_rep.size(); n++) {
       if (out_latency == -1) {
-	if (realtime_outputs_rep[n]->prefill_space() > 0)
-	  out_latency = (prefill_threshold_rep * buffersize()) + realtime_outputs_rep[n]->latency();
+	if (realtime_outputs_rep[n]->prefill_space() > 0) {
+	  long int max_prefill = prefill_threshold_rep * buffersize();
+	  if (max_prefill > realtime_outputs_rep[n]->prefill_space()) {
+	    max_prefill = realtime_outputs_rep[n]->prefill_space();
+	  }
+	  out_latency = max_prefill + realtime_outputs_rep[n]->latency();
+	}
 	else
 	  out_latency = realtime_outputs_rep[n]->latency();
       }
@@ -1358,12 +1395,12 @@ void ECA_ENGINE::update_cache_latency_values(void)
 	    realtime_outputs_rep[n]->prefill_space() == 0 &&
 	    out_latency != realtime_outputs_rep[n]->latency()) {
 	  ECA_LOG_MSG(ECA_LOGGER::info, 
-			"(eca-engine) Warning! Latency mismatch between output objects!");
+			"WARNING: Latency mismatch between output objects!");
 	}
       }
 
       ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		  "(eca-engine) Output latency for '" +
+		  "Output latency for '" +
 		  realtime_outputs_rep[n]->name() + 
 		  "' is " + kvu_numtostr(out_latency) + ".");
     }
@@ -1373,11 +1410,11 @@ void ECA_ENGINE::update_cache_latency_values(void)
     
     if (recording_offset_rep % buffersize()) {
       ECA_LOG_MSG(ECA_LOGGER::info, 
-		    "(eca-engine) Warning! Recording offset not divisible with chainsetup buffersize.");
+		    "WARNING: Recording offset not divisible with chainsetup buffersize.");
     }
     
     ECA_LOG_MSG(ECA_LOGGER::user_objects,
-		  "(eca-engine) recording offset is " +
+		  "recording offset is " +
 		  kvu_numtostr(recording_offset_rep) +
 		  	" samples.");
   }
@@ -1445,7 +1482,7 @@ void ECA_ENGINE::dump_profile_info(void)
                             impl_repp->looptimer_rep.events_under_lower_bound() -
                             impl_repp->looptimer_rep.events_over_upper_bound();
 
-  cerr << "(eca-engine) *** profile begin ***" << endl;
+  cerr << "*** profile begin ***" << endl;
   cerr << "Loops faster than realtime: "  << kvu_numtostr(impl_repp->looptimer_rep.events_under_lower_bound());
   cerr << " (<" << kvu_numtostr(impl_repp->looptimer_low_rep * 1000, 1) << " msec)" << endl;
   cerr << "Loops slower than realtime: "  << kvu_numtostr(slower_than_rt);
@@ -1462,7 +1499,7 @@ void ECA_ENGINE::dump_profile_info(void)
   cerr << "/";
   cerr << kvu_numtostr(impl_repp->looptimer_rep.average_duration_seconds() * 1000, 1);
   cerr << " msec." << endl;
-  cerr << "(eca-engine) *** profile end   ***" << endl;
+  cerr << "*** profile end   ***" << endl;
 }
 
 /**********************************************************************
@@ -1527,6 +1564,35 @@ void ECA_ENGINE::process_chains(void)
   }
 }
 
+void mix_to_outputs_divide_helper(const SAMPLE_BUFFER *from, SAMPLE_BUFFER *to, int divide_by, bool first_time)
+{
+  if (first_time == true) {
+    // this is the first output connected to this chain
+    if (from->number_of_channels() < to->number_of_channels()) {
+      to->make_silent();
+    }
+    to->copy(*from);
+    to->divide_by(divide_by);
+  }
+  else {
+    to->add_with_weight(*from, divide_by);
+  }
+}
+
+void mix_to_outputs_sum_helper(const SAMPLE_BUFFER *from, SAMPLE_BUFFER *to, bool first_time)
+{
+  if (first_time == true) {
+    // this is the first output connected to this chain
+    if (from->number_of_channels() < to->number_of_channels()) {
+      to->make_silent();
+    }
+    to->copy(*from);
+  }
+  else {
+    to->add(*from);
+  }
+}
+
 /**
  * context: J-level-1
  */
@@ -1536,7 +1602,7 @@ void ECA_ENGINE::mix_to_outputs(bool skip_realtime_target_outputs)
     if (skip_realtime_target_outputs == true) {
       if (csetup_repp->is_realtime_target_output(outputnum) == true) {
 	ECA_LOG_MSG(ECA_LOGGER::system_objects,
-		    "(eca-engine) Skipping rt-target output " +
+		    "Skipping rt-target output " +
 		    (*outputs_repp)[outputnum]->label() + ".");
 	continue;
       }
@@ -1573,21 +1639,12 @@ void ECA_ENGINE::mix_to_outputs(bool skip_realtime_target_outputs)
 	}
 	else {
 	  ++count;
-	  if (count == 1) {
-	    // -- 
-	    // this is the first output connected to this chain
-	    // --
-	    if (cslots_rep[n]->number_of_channels() < mixslot_repp->number_of_channels()) {
-	      mixslot_repp->make_silent();
-	    }
-	    mixslot_repp->copy(*cslots_rep[n]);
-	    mixslot_repp->divide_by(output_chain_count_rep[outputnum]);
-	  }
-	  else {
-	    mixslot_repp->add_with_weight(*cslots_rep[n],
-					  output_chain_count_rep[outputnum]);
-	  }
-	  
+
+	  if (csetup_repp->mix_mode() == ECA_CHAINSETUP::cs_mmode_avg) 	  
+	    mix_to_outputs_divide_helper(cslots_rep[n], mixslot_repp, output_chain_count_rep[outputnum], (count == 1));
+ 	  else
+	    mix_to_outputs_sum_helper(cslots_rep[n], mixslot_repp, (count == 1));
+
 	  if (count == output_chain_count_rep[outputnum]) {
 	    (*outputs_repp)[outputnum]->write_buffer(mixslot_repp);
 	    if ((*outputs_repp)[outputnum]->finished() == true) outputs_finished_rep++;
