@@ -1,7 +1,7 @@
 // ------------------------------------------------------------------------
 // eca-fileio-stream.cpp: File-I/O and buffering routines using normal
 //                        file streams.
-// Copyright (C) 1999-2002 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 1999-2002 Kai Vehmanen
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -157,7 +157,7 @@ void ECA_FILE_IO_STREAM::set_file_position(off_t newpos)
     }
 #else
     DBC_CHECK(sizeof(long int) == sizeof(off_t));
-    std::fseek(f1, curpos_rep, SEEK_SET);
+    std::fseek(f1, static_cast<long>(curpos_rep), SEEK_SET);
 #endif
   }
 }
