@@ -137,7 +137,10 @@ class ECA_IAMODE_PARSER_COMMANDS {
     ec_ctrlp_get,      /* FIXME: not implemented */
     // --
     ec_int_cmd_list,
+    ec_int_log_history,
     ec_int_output_mode_wellformed,
+    ec_int_set_float_to_string_precision,
+    ec_int_set_log_history_length,
     ec_int_version_string,
     ec_int_version_lib_current,
     ec_int_version_lib_revision,
@@ -146,6 +149,7 @@ class ECA_IAMODE_PARSER_COMMANDS {
     ec_map_cop_list,
     ec_map_preset_list,
     ec_map_ladspa_list,
+    ec_map_ladspa_id_list,
     ec_map_ctrl_list,
     // --
     ec_dump_target,
