@@ -1,7 +1,10 @@
 // ------------------------------------------------------------------------
 // eca-object-factory.cpp: Abstract factory for creating libecasound 
 //                         objects.
-// Copyright (C) 2000-2002 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 2000-2004 Kai Vehmanen
+//
+// Attributes:
+//     eca-style-version: 3
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -27,6 +30,14 @@
 #include <string>
 #include <pthread.h>
 
+#include <sys/types.h>
+#include <unistd.h>
+#include <stdlib.h>
+
+#ifdef ECA_COMPILE_JACK
+#include <jack/jack.h>
+#endif
+
 #include <kvu_dbc.h>
 #include "kvu_locks.h"
 #include <kvu_numtostr.h>
@@ -41,6 +52,7 @@
 #include "eca-object-map.h"
 #include "eca-preset-map.h"
 #include "eca-object-factory.h"
+#include "eca-resources.h"
 #include "eca-logger.h"
 
 /**
@@ -88,7 +100,7 @@ ECA_OBJECT_MAP& ECA_OBJECT_FACTORY::audio_io_rt_map(void)
       ECA_STATIC_OBJECT_MAPS::register_audio_io_rt_objects(audio_io_rt_map_repp);
     }
   }
-  return(*audio_io_rt_map_repp);
+  return *audio_io_rt_map_repp;
 }
 
 /**
@@ -106,7 +118,7 @@ ECA_OBJECT_MAP& ECA_OBJECT_FACTORY::audio_io_nonrt_map(void)
       ECA_STATIC_OBJECT_MAPS::register_audio_io_nonrt_objects(audio_io_nonrt_map_repp);
     }
   }
-  return(*audio_io_nonrt_map_repp);
+  return *audio_io_nonrt_map_repp;
 }
 
 /**
@@ -124,7 +136,7 @@ ECA_OBJECT_MAP& ECA_OBJECT_FACTORY::chain_operator_map(void)
       ECA_STATIC_OBJECT_MAPS::register_chain_operator_objects(chain_operator_map_repp);
     }
   }
-  return(*chain_operator_map_repp);
+  return *chain_operator_map_repp;
 }
 
 /**
@@ -145,7 +157,7 @@ ECA_OBJECT_MAP& ECA_OBJECT_FACTORY::ladspa_plugin_map(void)
       ECA_STATIC_OBJECT_MAPS::register_ladspa_plugin_objects(ladspa_plugin_map_repp);
     }
   }
-  return(*ladspa_plugin_map_repp);
+  return *ladspa_plugin_map_repp;
 }
 
 /**
@@ -166,7 +178,7 @@ ECA_OBJECT_MAP& ECA_OBJECT_FACTORY::ladspa_plugin_id_map(void)
       ECA_STATIC_OBJECT_MAPS::register_ladspa_plugin_id_objects(ladspa_plugin_id_map_repp);
     }
   }
-  return(*ladspa_plugin_id_map_repp);
+  return *ladspa_plugin_id_map_repp;
 }
 
 /**
@@ -184,7 +196,7 @@ ECA_PRESET_MAP& ECA_OBJECT_FACTORY::preset_map(void)
       ECA_STATIC_OBJECT_MAPS::register_preset_objects(preset_map_repp);
     }
   }
-  return(*preset_map_repp);
+  return *preset_map_repp;
 }
 
 /**
@@ -202,7 +214,7 @@ ECA_OBJECT_MAP& ECA_OBJECT_FACTORY::controller_map(void)
       ECA_STATIC_OBJECT_MAPS::register_controller_objects(controller_map_repp);
     }
   }
-  return(*controller_map_repp);
+  return *controller_map_repp;
 }
 
 /**
@@ -221,7 +233,7 @@ ECA_OBJECT_MAP& ECA_OBJECT_FACTORY::midi_device_map(void)
       ECA_STATIC_OBJECT_MAPS::register_midi_device_objects(midi_device_map_repp);
     }
   }
-  return(*midi_device_map_repp);
+  return *midi_device_map_repp;
 }
 
 /**
@@ -234,7 +246,8 @@ ECA_OBJECT_MAP& ECA_OBJECT_FACTORY::midi_device_map(void)
  *
  * @pre arg.empty() != true
  */
-AUDIO_IO* ECA_OBJECT_FACTORY::create_audio_object(const string& arg) {
+AUDIO_IO* ECA_OBJECT_FACTORY::create_audio_object(const string& arg)
+{
   // --
   DBC_REQUIRE(arg.empty() != true);
   // --
@@ -253,12 +266,12 @@ AUDIO_IO* ECA_OBJECT_FACTORY::create_audio_object(const string& arg) {
   AUDIO_IO* new_file = 0;
   if (main_file != 0) {
     new_file = main_file->new_expr();
-    ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-object-factory) Object \"" + arg + "\" created, type \"" + new_file->name() + "\". Has " + kvu_numtostr(new_file->number_of_params()) + " parameter(s).");
+    ECA_LOG_MSG(ECA_LOGGER::user_objects, "Object \"" + arg + "\" created, type \"" + new_file->name() + "\". Has " + kvu_numtostr(new_file->number_of_params()) + " parameter(s).");
     for(int n = 0; n < new_file->number_of_params(); n++) {
       new_file->set_parameter(n + 1, kvu_get_argument_number(n + 1, arg));
     }
   }
-  return(new_file);
+  return new_file;
 }
 
 /**
@@ -286,12 +299,12 @@ MIDI_IO* ECA_OBJECT_FACTORY::create_midi_device(const string& arg)
   MIDI_IO* new_device = 0;
   if (device != 0) {
     new_device = device->new_expr();
-    ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-object-factory) Object \"" + arg + "\" created, type \"" + new_device->name() + "\". Has " + kvu_numtostr(new_device->number_of_params()) + " parameter(s).");
+    ECA_LOG_MSG(ECA_LOGGER::user_objects, "Object \"" + arg + "\" created, type \"" + new_device->name() + "\". Has " + kvu_numtostr(new_device->number_of_params()) + " parameter(s).");
     for(int n = 0; n < new_device->number_of_params(); n++) {
       new_device->set_parameter(n + 1, kvu_get_argument_number(n + 1, arg));
     }
   }
-  return(new_device);
+  return new_device;
 }
 
 /**
@@ -305,7 +318,8 @@ MIDI_IO* ECA_OBJECT_FACTORY::create_midi_device(const string& arg)
  * @pre argu.empty() != true
  */
 AUDIO_IO* ECA_OBJECT_FACTORY::create_loop_input(const string& argu,
-						 map<int,LOOP_DEVICE*>* loop_map) {
+						map<int,LOOP_DEVICE*>* loop_map)
+{
   // --------
   DBC_REQUIRE(argu.empty() != true);
   // --------
@@ -324,7 +338,7 @@ AUDIO_IO* ECA_OBJECT_FACTORY::create_loop_input(const string& argu,
     p->register_input();
   }
   
-  return(p);
+  return p;
 }
 
 /**
@@ -338,7 +352,8 @@ AUDIO_IO* ECA_OBJECT_FACTORY::create_loop_input(const string& argu,
  * @pre argu.empty() != true
  */
 AUDIO_IO* ECA_OBJECT_FACTORY::create_loop_output(const string& argu,
-						 map<int,LOOP_DEVICE*>* loop_map) {
+						 map<int,LOOP_DEVICE*>* loop_map)
+{
   // --------
   DBC_REQUIRE(argu.empty() != true);
   // --------
@@ -357,7 +372,7 @@ AUDIO_IO* ECA_OBJECT_FACTORY::create_loop_output(const string& argu,
     p->register_output();
   }
   
-  return(p);
+  return p;
 }
 
 /**
@@ -394,9 +409,9 @@ CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_ladspa_plugin (const string& argu)
       new_cop = dynamic_cast<CHAIN_OPERATOR*>(cop->new_expr());
 
       ECA_LOG_MSG(ECA_LOGGER::user_objects, 
-		  "(eca-object-factory) Creating LADSPA-plugin \"" + new_cop->name() + "\"");
+		  "Creating LADSPA-plugin \"" + new_cop->name() + "\"");
 
-      otemp << "(eca-object-factory) Setting parameters: ";
+      otemp << "Setting parameters: ";
       for(int n = 0; n < new_cop->number_of_params(); n++) {
 	new_cop->set_parameter(n + 1, atof(kvu_get_argument_number(n + 2, argu).c_str()));
 	otemp << new_cop->get_parameter_name(n + 1) << " = ";
@@ -405,9 +420,14 @@ CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_ladspa_plugin (const string& argu)
       }
       ECA_LOG_MSG(ECA_LOGGER::user_objects, otemp.to_string());
     }
-    return(new_cop);
+    else {
+      ECA_LOG_MSG(ECA_LOGGER::info, 
+		  "ERROR: Unable to find LADSPA plugin \"" + unique + "\"");
+
+    }
+    return new_cop;
   }
-  return(0);
+  return 0;
 }
 
 /**
@@ -423,7 +443,8 @@ CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_ladspa_plugin (const string& argu)
  *        (distribution of VST-headers is not
  *        allowed).
  */
-CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_vst_plugin (const string& argu) {
+CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_vst_plugin (const string& argu)
+{
   // --------
   DBC_REQUIRE(argu.size() > 0);
   DBC_REQUIRE(argu[0] == '-');
@@ -438,8 +459,8 @@ CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_vst_plugin (const string& argu) {
   CHAIN_OPERATOR* new_cop = 0;
   if (cop != 0) {
     
-    ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-object-factory) Creating VST-plugin \"" + new_cop->name() + "\"");
-    otemp << "(eca-object-factory) Setting parameters: ";
+    ECA_LOG_MSG(ECA_LOGGER::user_objects, "Creating VST-plugin \"" + new_cop->name() + "\"");
+    otemp << "Setting parameters: ";
     for(int n = 0; n < new_cop->number_of_params(); n++) {
       new_cop->set_parameter(n + 1, atof(kvu_get_argument_number(n + 1, argu).c_str()));
       otemp << new_cop->get_parameter_name(n + 1) << " = ";
@@ -448,7 +469,7 @@ CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_vst_plugin (const string& argu) {
     }
     ECA_LOG_MSG(ECA_LOGGER::user_objects, otemp.to_string());
   }
-  return(new_cop);
+  return new_cop;
 }
 #endif /* VST ifdef 0 */
 
@@ -463,7 +484,8 @@ CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_vst_plugin (const string& argu) {
  * @pre argu.size() > 0
  * @pre argu[0] == '-'
  */
-CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_chain_operator (const string& argu) {
+CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_chain_operator (const string& argu)
+{
   // --------
   DBC_REQUIRE(argu.size() > 0);
   DBC_REQUIRE(argu[0] == '-');
@@ -479,10 +501,10 @@ CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_chain_operator (const string& argu) {
   if (cop != 0) {
     new_cop = dynamic_cast<CHAIN_OPERATOR*>(cop->new_expr());
 
-    ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-object-factory) Creating chain operator \"" +
+    ECA_LOG_MSG(ECA_LOGGER::user_objects, "Creating chain operator \"" +
 		  new_cop->name() + "\"");
     //    otemp << "(eca-chainsetup) Adding effect " << new_cop->name();
-    otemp << "(eca-object-factory) Setting parameters: ";
+    otemp << "Setting parameters: ";
     for(int n = 0; n < new_cop->number_of_params(); n++) {
       new_cop->set_parameter(n + 1, atof(kvu_get_argument_number(n + 1, argu).c_str()));
       otemp << new_cop->get_parameter_name(n + 1) << " = ";
@@ -490,9 +512,9 @@ CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_chain_operator (const string& argu) {
       if (n + 1 < new_cop->number_of_params()) otemp << ", ";
     }
     ECA_LOG_MSG(ECA_LOGGER::user_objects, otemp.to_string());
-    return(new_cop);
+    return new_cop;
   }
-  return(0);
+  return 0;
 }
 
 /**
@@ -506,13 +528,14 @@ CHAIN_OPERATOR* ECA_OBJECT_FACTORY::create_chain_operator (const string& argu) {
  * @pre argu.size() > 0
  * @pre argu[0] == '-'
  */
-GENERIC_CONTROLLER* ECA_OBJECT_FACTORY::create_controller (const string& argu) {
+GENERIC_CONTROLLER* ECA_OBJECT_FACTORY::create_controller (const string& argu)
+{
   // --------
   DBC_REQUIRE(argu.size() > 0);
   DBC_REQUIRE(argu[0] == '-');
   // --------
 
-  if (argu.size() > 0 && argu[0] != '-') return(0);
+  if (argu.size() > 0 && argu[0] != '-') return 0;
   string prefix = kvu_get_argument_prefix(argu);
 
   const GENERIC_CONTROLLER* gcontroller = 
@@ -528,11 +551,11 @@ GENERIC_CONTROLLER* ECA_OBJECT_FACTORY::create_controller (const string& argu) {
       }
       new_gcontroller->assign_source(new_csource);
 
-      ECA_LOG_MSG(ECA_LOGGER::user_objects, "(eca-object-factory) Creating controller source \"" +  new_gcontroller->name() + "\"");
+      ECA_LOG_MSG(ECA_LOGGER::user_objects, "Creating controller source \"" +  new_gcontroller->name() + "\"");
 
       MESSAGE_ITEM otemp;
       otemp.setprecision(3);
-      otemp << "(eca-object-factory) Setting parameters: ";
+      otemp << "Setting parameters: ";
       int numparams = new_gcontroller->number_of_params();
       for(int n = 0; n < numparams; n++) {
 	new_gcontroller->set_parameter(n + 1, atof(kvu_get_argument_number(n + 1, argu).c_str()));
@@ -543,8 +566,79 @@ GENERIC_CONTROLLER* ECA_OBJECT_FACTORY::create_controller (const string& argu) {
       }
       ECA_LOG_MSG(ECA_LOGGER::user_objects, otemp.to_string());
 
-      return(new_gcontroller);
+      return new_gcontroller;
     }
   }
-  return(0);
+  return 0;
+}
+
+/**
+ * Returns a EOS-compatible string describing the default 
+ * output device. This device is determined based on 
+ * ecasoundrc settings, available resources, 
+ * compile-time options, etc.
+ */
+std::string ECA_OBJECT_FACTORY::probe_default_output_device(void)
+{
+    ECA_RESOURCES ecaresources;
+    string default_output = ecaresources.resource("default-output");
+    bool output_selected = true;
+
+    if (default_output == "autodetect") {
+      
+      output_selected = false;
+
+#ifdef ECA_COMPILE_JACK
+      /* phase 1: check for JACK */
+
+      int pid = getpid();
+      string cname = "ecasound-autodetect-" + kvu_numtostr(pid);
+
+      bool env_changed = false;
+      char* oldenv = getenv("JACK_NO_START_SERVER");
+      if (oldenv == NULL) {
+	env_changed = true;
+	setenv("JACK_NO_START_SERVER", "yes", 1);
+      }
+      
+      jack_client_t *client = jack_client_new (cname.c_str());
+      if (client != 0) {
+	jack_client_close(client);
+	
+	default_output = "jack_alsa";
+	output_selected = true;
+      }
+      
+      if (env_changed == true) {
+	unsetenv("JACK_NO_START_SERVER");
+      }
+#endif
+      
+      /* phase 2: check for ALSA support */
+      if (output_selected != true) {
+	const ECA_OBJECT *obj = ECA_OBJECT_FACTORY::audio_io_rt_map().object("alsa");
+	if (obj != 0) {
+	  default_output = "alsa,default";
+	  output_selected = true;
+	}
+      }
+      
+      /* phase 3: check for OSS support */
+      if (output_selected != true) {
+	const ECA_OBJECT *obj = ECA_OBJECT_FACTORY::audio_io_rt_map().object("/dev/dsp");
+	if (obj != 0) {
+	  default_output = "/dev/dsp";
+	  output_selected = true;
+	}
+      }
+      
+      /* phase 4: fallback to rtnull */
+      if (output_selected != true) {
+	ECA_LOG_MSG(ECA_LOGGER::info,
+		    "WARNING: No default output available. Using 'rtnull' as a fallback.");
+	  default_output = "rtnull";
+      }
+    }
+
+    return default_output;
 }
