@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // midi-cc.cpp: Interface to MIDI continuous controllers
-// Copyright (C) 1999,2001 Kai Vehmanen (kaiv@wakkanet.fi)
+// Copyright (C) 1999,2001-2002 Kai Vehmanen
 //
 // This program is fre software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,34 +20,30 @@
 #include <string>
 #include <iostream>
 
-#include <kvutils/message_item.h>
+#include <kvu_message_item.h>
 
 //  #include "eca-midi.h"
 #include "midi-client.h"
 #include "midi-server.h"
 #include "midi-cc.h"
 
-#include "eca-debug.h"
+#include "eca-logger.h"
 
-CONTROLLER_SOURCE::parameter_type MIDI_CONTROLLER::value(void) {
-//    if (midi_in_queue.update_controller_value()) {
-//      value_rep =
-//        static_cast<double>(midi_in_queue.last_controller_value(static_cast<int>(channel), static_cast<int>(controller)));
-//      value_rep /= 127.0;
-//    }
+CONTROLLER_SOURCE::parameter_t MIDI_CONTROLLER::value(void)
+{
   if (trace_request_rep == true) {
     if (server() != 0) {
-      server()->add_controller_trace(channel, 
-				     controller);
+      server()->add_controller_trace(channel_rep, 
+				     controller_rep);
     }
     else {
-      cerr << "(midi-cc) Warning! No MIDI-server found!" << endl;
+      std::cerr << "(midi-cc) WARNING: No MIDI-server found!" << std::endl;
     }
     trace_request_rep = false;
   }
   if (server() != 0)
     value_rep =
-      static_cast<double>(server()->last_controller_value(channel, controller));
+      static_cast<double>(server()->last_controller_value(channel_rep, controller_rep));
       
   value_rep /= 127.0;
   return(value_rep);
@@ -55,41 +51,56 @@ CONTROLLER_SOURCE::parameter_type MIDI_CONTROLLER::value(void) {
 
 MIDI_CONTROLLER::MIDI_CONTROLLER(int controller_number, 
 				 int midi_channel) 
-  : controller(controller_number), 
-    channel(midi_channel),
+  : controller_rep(controller_number), 
+    channel_rep(midi_channel),
     value_rep(0.0),
     trace_request_rep(false) 
-{ }
-
-void MIDI_CONTROLLER::init(CONTROLLER_SOURCE::parameter_type phasestep) {
-//      init_midi_queues();
+{
+}
 
+void MIDI_CONTROLLER::init(void)
+{
     MESSAGE_ITEM otemp;
     otemp << "(midi-cc) MIDI-controller initialized using controller ";
     otemp.setprecision(0);
-    otemp << controller << " and channel " << channel << ".";
-    ecadebug->msg(ECA_DEBUG::user_objects, otemp.to_string());
-
+    otemp << controller_rep << " and channel " << channel_rep << ".";
+    ECA_LOG_MSG(ECA_LOGGER::user_objects, otemp.to_string());
 }
 
-void MIDI_CONTROLLER::set_parameter(int param, CONTROLLER_SOURCE::parameter_type value) {
+void MIDI_CONTROLLER::set_parameter(int param, CONTROLLER_SOURCE::parameter_t value)
+{
   switch (param) {
   case 1: 
-    controller = static_cast<int>(value);
+    controller_rep = static_cast<int>(value);
+    if (controller_rep < 0 ||
+	controller_rep > 127) {
+      controller_rep = 1;
+      ECA_LOG_MSG(ECA_LOGGER::info, 
+		  "(midi-cc) Controller number must be a number between 0 and 127. Defaulting to controller 0");
+    }
     break;
   case 2: 
-    channel = static_cast<int>(value);
+    channel_rep = static_cast<int>(value);
+    if (channel_rep < 1 ||
+	channel_rep > 16) {
+      channel_rep = 1;
+      ECA_LOG_MSG(ECA_LOGGER::info, 
+		  "(midi-cc) MIDI-channel must be a number between 1 and 16. Defaulting to channel 1.");
+    }
+    --channel_rep; /* map from 1...16 -> 0...15 */
     break;
   }
   trace_request_rep = true;
 }
 
-CONTROLLER_SOURCE::parameter_type MIDI_CONTROLLER::get_parameter(int param) const { 
+CONTROLLER_SOURCE::parameter_t MIDI_CONTROLLER::get_parameter(int param) const
+{
   switch (param) {
   case 1: 
-    return(static_cast<parameter_type>(controller));
+    return(static_cast<parameter_t>(controller_rep));
   case 2: 
-    return(static_cast<parameter_type>(channel));
+    /* map from 0...15 -> 1...16 */
+    return(static_cast<parameter_t>(channel_rep + 1));
   }
   return(0.0);
 }
