@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // midi-server.cpp: MIDI i/o engine serving generic clients.
-// Copyright (C) 2001-2002 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 2001-2002 Kai Vehmanen
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -221,7 +221,7 @@ void MIDI_SERVER::enable(void)
 #endif
     }
 #else
-	ECA_LOG_MSG(ECA_LOGGER::info, "Warning! Functions to query scheduler settings not available!");
+	ECA_LOG_MSG(ECA_LOGGER::info, "WARNING: Functions to query scheduler settings not available!");
 #endif
     thread_running_rep = true;
   }
