@@ -1,6 +1,12 @@
 // ------------------------------------------------------------------------
 // samplebuffer.cpp: Class representing a buffer of audio samples.
-// Copyright (C) 1999-2003 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
+// Copyright (C) 1999-2005 Kai Vehmanen
+//
+// Attributes:
+//     eca-style-version: 3
+//
+// References:
+//     http://www.mega-nerd.com/SRC/
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -80,17 +86,21 @@ SAMPLE_BUFFER::SAMPLE_BUFFER (buf_size_t buffersize, channel_size_t channels)
 
   impl_repp->rt_lock_rep = false;
   impl_repp->lockref_rep = 0;
-  impl_repp->quality_rep = 50;
   impl_repp->old_buffer_repp = 0;
+#ifdef ECA_COMPILE_SAMPLERATE
+  impl_repp->quality_rep = 50;
+  impl_repp->src_state_rep.resize(channels);
+#else
+  impl_repp->quality_rep = 5;
+#endif
  
   ECA_LOG_MSG(ECA_LOGGER::functions, 
-		"(samplebuffer) Buffer created, channels: " +
+		"Buffer created, channels: " +
 		kvu_numtostr(buffer.size()) + ", length-samples: " +
 		kvu_numtostr(buffersize_rep) + ".");
 
   // ---
   DBC_ENSURE(buffer.size() == static_cast<size_t>(channel_count_rep));
-  DBC_ENSURE(buffersize_rep == reserved_samples_rep);
   // ---
 }
 
@@ -120,7 +130,7 @@ SAMPLE_BUFFER::SAMPLE_BUFFER (const SAMPLE_BUFFER& x)
   impl_repp->old_buffer_repp = 0;
 
   ECA_LOG_MSG(ECA_LOGGER::functions, 
-		"(samplebuffer) Buffer copy-constructed, channels: " +
+		"Buffer copy-constructed, channels: " +
 		kvu_numtostr(buffer.size()) + ", length-samples: " +
 		kvu_numtostr(buffersize_rep) + ".");
 
@@ -196,6 +206,15 @@ SAMPLE_BUFFER::~SAMPLE_BUFFER (void)
     impl_repp->old_buffer_repp = 0;
   }
 
+#ifdef ECA_COMPILE_SAMPLERATE
+  for(size_t n = 0; n < impl_repp->src_state_rep.size(); n++) {
+    if (impl_repp->src_state_rep[n] != 0) {
+      src_delete(impl_repp->src_state_rep[n]);
+      impl_repp->src_state_rep[n] = 0;
+    }
+  }
+#endif
+
   delete impl_repp;
 }
 
@@ -359,33 +378,55 @@ void SAMPLE_BUFFER::limit_values(void)
  * Resamples samplebuffer contents. Resampling
  * changes buffer length by 'to_rate/from_rate'.
  *
- * @post std::fabs(to_rate / from_rate * old_length_in_samples - length_in_samples()) <= 1
+ * @post to_rate / from_rate * old_length_in_samples - length_in_samples() >= -1
  */
 void SAMPLE_BUFFER::resample(SAMPLE_SPECS::sample_rate_t from_rate,
 			     SAMPLE_SPECS::sample_rate_t to_rate)
 {
+#ifndef ECA_COMPILE_SAMPLERATE
   DBC_DECLARE(buf_size_t old_length_in_samples = length_in_samples());
-
-#ifdef ECA_COMPILE_SAMPLERATE
-  resample_secret_rabbit_code(from_rate, to_rate);
-#else
-  resample_with_memory(from_rate, to_rate); 
 #endif
 
-  /* FIXME: fails all the fime... */
-  DBC_CHECK(std::fabs((static_cast<double>(to_rate) / from_rate * old_length_in_samples - length_in_samples())) <= 1.0f);
-  // DBC_CHECK(length_in_samples() == std::floor((static_cast<double>(to_rate) / from_rate) * static_cast<double>(old_length_in_samples) + 0.5f));
-  // fprintf(stderr, "resample fail len_samples=%ld, to=%ld, from=%ld, old_len=%ld, round=%f.\n", length_in_samples(), to_rate, from_rate, old_length_in_samples, std::floor((static_cast<double>(to_rate) / from_rate) * static_cast<double>(old_length_in_samples) + 0.5f));
+#ifdef ECA_COMPILE_SAMPLERATE
+  if (impl_repp->quality_rep > 5) {
+    resample_secret_rabbit_code(from_rate, to_rate);
+  }
+  else 
+#endif
+    {
+      DBC_CHECK(impl_repp->quality_rep <= 5);
+      resample_with_memory(from_rate, to_rate); 
+    }
+
+#ifndef ECA_COMPILE_SAMPLERATE
+  /* with libsamplerate, the output sample count can vary from call to call */
+  DBC_CHECK((static_cast<double>(to_rate) / from_rate * old_length_in_samples - length_in_samples()) >= -1);
+#endif
 }
 
 /**
  * Set resampling quality. 
  *
+ * Depending on build options, not all quality levels
+ * are necessarily supported. If the requested quality level 
+ * exceeds the available resamplers, the quality setting 
+ * is set to the highest available algorithm. You can use
+ * the get_resample_quality() function to query the current level.
+ *
  * @param quality value between 0 (lowest) to 100 (highest)
  */
 void SAMPLE_BUFFER::resample_set_quality(int quality)
 {
+#ifdef ECA_COMPILE_SAMPLERATE
   impl_repp->quality_rep = quality;
+#else
+  if (quality > 10) {
+    ECA_LOG_MSG(ECA_LOGGER::info, 
+		"WARNING: Libsamplerate is required for high-quality resampling. "
+		"Using the internal resampler instead.");
+    impl_repp->quality_rep = 5;
+  }
+#endif
 }
 
 /**
@@ -395,7 +436,7 @@ void SAMPLE_BUFFER::resample_set_quality(int quality)
  */
 int SAMPLE_BUFFER::resample_get_quality(void) const
 {
-  return(impl_repp->quality_rep);
+  return impl_repp->quality_rep;
 }
 
 void SAMPLE_BUFFER::export_helper(unsigned char* obuffer, 
@@ -499,7 +540,7 @@ void SAMPLE_BUFFER::export_helper(unsigned char* obuffer,
     
   default: 
     { 
-      ECA_LOG_MSG(ECA_LOGGER::info, "(samplebuffer) Unknown sample format! [1].");
+      ECA_LOG_MSG(ECA_LOGGER::info, "Unknown sample format! [1].");
     }
   }
 }
@@ -536,7 +577,6 @@ void SAMPLE_BUFFER::export_interleaved(unsigned char* target,
       else if (stemp < SAMPLE_SPECS::impl_min_value) stemp = SAMPLE_SPECS::impl_min_value;
 
       SAMPLE_BUFFER::export_helper(target, &osize, stemp, fmt);
-      
     }
   }
   
@@ -603,7 +643,7 @@ void SAMPLE_BUFFER::import_helper(const unsigned char *ibuffer,
     {
       // little endian: (LSB, MSB) (Intel)
       // big endian: (MSB, LSB) (Motorola)
-      if (SAMPLE_SPECS::is_system_littleendian) {
+      if (is_system_littleendian) {
 	a[0] = ibuffer[(*iptr)++];
 	a[1] = ibuffer[(*iptr)++];
       }
@@ -617,7 +657,7 @@ void SAMPLE_BUFFER::import_helper(const unsigned char *ibuffer,
 
   case ECA_AUDIO_FORMAT::sfmt_s16_be:
     {
-      if (!SAMPLE_SPECS::is_system_littleendian) {
+      if (!is_system_littleendian) {
 	a[0] = ibuffer[(*iptr)++];
 	a[1] = ibuffer[(*iptr)++];
       }
@@ -631,7 +671,7 @@ void SAMPLE_BUFFER::import_helper(const unsigned char *ibuffer,
 
   case ECA_AUDIO_FORMAT::sfmt_s24_le:
     {
-      if (SAMPLE_SPECS::is_system_littleendian) {
+      if (is_system_littleendian) {
 	b[0] = 0; /* LSB */
 	b[1] = ibuffer[(*iptr)++];
 	b[2] = ibuffer[(*iptr)++];
@@ -649,7 +689,7 @@ void SAMPLE_BUFFER::import_helper(const unsigned char *ibuffer,
 
   case ECA_AUDIO_FORMAT::sfmt_s24_be:
     {
-      if (SAMPLE_SPECS::is_system_littleendian) {
+      if (is_system_littleendian) {
 	b[3] = ibuffer[(*iptr)++];
 	b[2] = ibuffer[(*iptr)++];
 	b[1] = ibuffer[(*iptr)++];
@@ -667,7 +707,7 @@ void SAMPLE_BUFFER::import_helper(const unsigned char *ibuffer,
 
   case ECA_AUDIO_FORMAT::sfmt_s32_le:
     {
-      if (SAMPLE_SPECS::is_system_littleendian) {
+      if (is_system_littleendian) {
 	b[0] = ibuffer[(*iptr)++];
 	b[1] = ibuffer[(*iptr)++];
 	b[2] = ibuffer[(*iptr)++];
@@ -685,7 +725,7 @@ void SAMPLE_BUFFER::import_helper(const unsigned char *ibuffer,
 
   case ECA_AUDIO_FORMAT::sfmt_s32_be:
     {
-      if (SAMPLE_SPECS::is_system_littleendian) {
+      if (is_system_littleendian) {
 	b[3] = ibuffer[(*iptr)++];
 	b[2] = ibuffer[(*iptr)++];
 	b[1] = ibuffer[(*iptr)++];
@@ -703,7 +743,7 @@ void SAMPLE_BUFFER::import_helper(const unsigned char *ibuffer,
 
   case ECA_AUDIO_FORMAT::sfmt_f32_le:
     {
-      if (SAMPLE_SPECS::is_system_littleendian) {
+      if (is_system_littleendian) {
 	b[0] = ibuffer[(*iptr)++];
 	b[1] = ibuffer[(*iptr)++];
 	b[2] = ibuffer[(*iptr)++];
@@ -721,7 +761,7 @@ void SAMPLE_BUFFER::import_helper(const unsigned char *ibuffer,
 
   case ECA_AUDIO_FORMAT::sfmt_f32_be:
     {
-      if (SAMPLE_SPECS::is_system_littleendian) {
+      if (is_system_littleendian) {
 	b[3] = ibuffer[(*iptr)++];
 	b[2] = ibuffer[(*iptr)++];
 	b[1] = ibuffer[(*iptr)++];
@@ -739,7 +779,7 @@ void SAMPLE_BUFFER::import_helper(const unsigned char *ibuffer,
 
   default: 
     { 
-      ECA_LOG_MSG(ECA_LOGGER::info, "(samplebuffer) Unknown sample format! [4].");
+      ECA_LOG_MSG(ECA_LOGGER::info, "Unknown sample format! [4].");
     }
   }
 }
@@ -818,7 +858,7 @@ void SAMPLE_BUFFER::number_of_channels(channel_size_t len)
     for(channel_size_t n = old_size; n < len; n++) {
       buffer[n] = new sample_t [reserved_samples_rep];
     }
-    ECA_LOG_MSG(ECA_LOGGER::functions, "(samplebuffer<>) Increasing channel-count (1).");    
+    ECA_LOG_MSG(ECA_LOGGER::functions, "Increasing channel-count (1).");    
   }
 
   /* note! channel_count_rep and buffer.size() necessarily
@@ -831,7 +871,7 @@ void SAMPLE_BUFFER::number_of_channels(channel_size_t len)
 	buffer[n][m] = SAMPLE_SPECS::silent_value;
       }
     }
-    // ECA_LOG_MSG(ECA_LOGGER::system_objects, "(samplebuffer<>) Increasing channel-count (2).");
+    // ECA_LOG_MSG(ECA_LOGGER::system_objects, "Increasing channel-count (2).");
   }
 
   channel_count_rep = len;
@@ -858,7 +898,7 @@ void SAMPLE_BUFFER::length_in_samples(buf_size_t len)
 
     if (impl_repp->old_buffer_repp != 0) {
       delete[] impl_repp->old_buffer_repp;
-      impl_repp->old_buffer_repp = 0;
+      impl_repp->old_buffer_repp = new sample_t [reserved_samples_rep];
     }
   }
 
@@ -884,29 +924,56 @@ void SAMPLE_BUFFER::length_in_samples(buf_size_t len)
 void SAMPLE_BUFFER::resample_init_memory(SAMPLE_SPECS::sample_rate_t from_srate,
 					 SAMPLE_SPECS::sample_rate_t to_srate)
 {
+#ifdef ECA_COMPILE_SAMPLERATE
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, 
+		"Resampler selected: libsamplerate (Secret Rabbit Code).");
+#else
+  ECA_LOG_MSG(ECA_LOGGER::system_objects, 
+		"Resampler selected: internal resampler.");
+#endif
+
   double step = 1.0;
   if (from_srate != 0) { step = static_cast<double>(to_srate) / from_srate; }
-  buf_size_t new_buffer_size = static_cast<buf_size_t>((step * buffersize_rep) + 1.0f);
+
+  /* add at least one word of extra space */
+  buf_size_t new_buffer_size = static_cast<buf_size_t>((step * buffersize_rep)) + sizeof(buf_size_t);
 
   if (new_buffer_size > reserved_samples_rep) {
     reserved_samples_rep = new_buffer_size * 2;
 
+#ifdef ECA_DEBUG_MODE
     DBC_CHECK(impl_repp->rt_lock_rep != true);
     DBC_CHECK(impl_repp->lockref_rep == 0);
+#endif
 
     for(int c = 0; c < channel_count_rep; c++) {
       delete[] buffer[c];
       buffer[c] = new sample_t [reserved_samples_rep];
-    } 
+    }
   }
 
+#ifdef ECA_COMPILE_SAMPLERATE
+  impl_repp->src_state_rep.resize(channel_count_rep);
+  for(int c = 0; c < channel_count_rep; c++) {
+    if (impl_repp->src_state_rep[c] == 0) {
+      int error;
+      impl_repp->src_state_rep[c] = src_new((impl_repp->quality_rep > 75) ? SRC_SINC_BEST_QUALITY : SRC_SINC_MEDIUM_QUALITY, 1, &error);
+      DBC_CHECK(impl_repp->src_state_rep[c] != 0);
+    }
+  }
+#endif
+
   if (impl_repp->old_buffer_repp == 0) {
+#ifdef ECA_DEBUG_MODE
     DBC_CHECK(impl_repp->rt_lock_rep != true);
+#endif
     impl_repp->old_buffer_repp = new sample_t [reserved_samples_rep];
   }
 
   if (impl_repp->resample_memory_rep.size() < static_cast<size_t>(channel_count_rep)) {
+#ifdef ECA_DEBUG_MODE
     DBC_CHECK(impl_repp->rt_lock_rep != true);
+#endif
     impl_repp->resample_memory_rep.resize(channel_count_rep, 0.0f);
   }
 }
@@ -979,9 +1046,10 @@ void SAMPLE_BUFFER::resample_nofilter(SAMPLE_SPECS::sample_rate_t from,
   double step = static_cast<double>(to) / from;
   buf_size_t old_buffer_size = buffersize_rep;
 
-  length_in_samples(static_cast<buf_size_t>(std::floor(step * buffersize_rep + 0.5f)));
+  // truncate, not round, to integer
+  length_in_samples(static_cast<buf_size_t>(std::floor(step * buffersize_rep)));
 
-  DEBUG_RESAMPLING_STATEMENT(std::cerr << "(samplebuffer) resample_no_f from " << from << " to " << to << "." << std::endl);
+  DEBUG_RESAMPLING_STATEMENT(std::cerr << "resample_no_f from " << from << " to " << to << "." << std::endl);
 
   DBC_CHECK(impl_repp->old_buffer_repp != 0);
 
@@ -1032,7 +1100,8 @@ void SAMPLE_BUFFER::resample_with_memory(SAMPLE_SPECS::sample_rate_t from,
   double step = (double)to / from;
   buf_size_t old_buffer_size = buffersize_rep;
 
-  length_in_samples(static_cast<buf_size_t>(std::floor(step * buffersize_rep + 0.5f)));
+  // truncate, not round, to integer
+  length_in_samples(static_cast<buf_size_t>(std::floor(step * buffersize_rep)));
 
   DBC_CHECK(impl_repp->old_buffer_repp != 0);
   DEBUG_RESAMPLING_STATEMENT(std::cerr << "(samplebuffer) resample_w_m from " << from << " to " << to << "." << std::endl); 
@@ -1091,7 +1160,8 @@ void SAMPLE_BUFFER::resample_secret_rabbit_code(SAMPLE_SPECS::sample_rate_t from
   double step = static_cast<double>(to_srate) / from_srate;
   buf_size_t old_buffer_size = buffersize_rep;
 
-  length_in_samples(static_cast<buf_size_t>(std::floor(step * buffersize_rep + 0.5f)));
+  /* modify buffersize_rep (size of dst buffer) */
+  length_in_samples(static_cast<buf_size_t>(std::floor(step * buffersize_rep)));
 
   DBC_CHECK(impl_repp->old_buffer_repp != 0);
   DEBUG_RESAMPLING_STATEMENT(std::cerr << "(samplebuffer) resample_s_r_c from " << from_srate << " to " << to_srate << "." << std::endl); 
@@ -1113,36 +1183,57 @@ void SAMPLE_BUFFER::resample_secret_rabbit_code(SAMPLE_SPECS::sample_rate_t from
     // Maximum number of frames pointer to by data_out.
     // note: was 'reserved_samples_rep' but this led 
     //       to corrupted output
-    params.output_frames = reserved_samples_rep; /* buffersize_rep */
+    params.output_frames = reserved_samples_rep; /* buffersize_rep; */
 
     // Equal to output_sample_rate / input_sample_rate.
     params.src_ratio = step;
+    params.end_of_input = 0;
+
+    // update the step
+    int ret = src_set_ratio(impl_repp->src_state_rep[c], step);
+    DBC_CHECK(ret == 0);
 
     // Perform the sample rate conversion
-    int ret = src_simple(&params, 
-			 (impl_repp->quality_rep > 75) ? SRC_SINC_BEST_QUALITY : SRC_SINC_MEDIUM_QUALITY, 1);
-
+    ret = src_process(impl_repp->src_state_rep[c], &params);
     DBC_CHECK(ret == 0);
-    DBC_CHECK(std::labs(params.input_frames_used - old_buffer_size) <= 1);
+    if (ret) {
+      /* make sure we avoid segfault in all cases */
+      params.output_frames_gen = 0;
+      params.input_frames_used = old_buffer_size;
+    }
+    DBC_CHECK(std::labs(params.input_frames_used - old_buffer_size) == 0);
+#ifdef ECA_DEBUG_MODE
+    /* make sure all input samples have been used */
     if (old_buffer_size != params.input_frames_used) { std::cerr << "input_frames_over=" << old_buffer_size - params.input_frames_used << ".\n"; }
 
+    /* check that all channels are processed in the same way */
     if (c == 0) ch_out_count = params.output_frames_gen;
     DBC_CHECK(ch_out_count == params.output_frames_gen);
-
-    // if (buffersize_rep != params.output_frames_gen) { std::cerr << "output_frame_space_left=" << params.output_frames_gen - buffersize_rep << ".\n"; }
+#endif /* ECA_DEBUG_MODE */
   }
 
-  DBC_CHECK(std::labs(params.output_frames_gen - buffersize_rep) <= 1);
+  DEBUG_RESAMPLING_STATEMENT(std::cerr << "(samplebuffer) src_src input=" << old_buffer_size 
+			     << ", target=" << buffersize_rep 
+			     << ", processed=" << params.input_frames_used 
+			     << ", space_for=" << reserved_samples_rep 
+			     << ", out=" << params.output_frames_gen << std::endl); 
 
-  /* FIXME: this is incorrect, but what to do!? */
-  // DBC_CHECK(std::labs(params.output_frames_gen - buffersize_rep) <= 1);
-  if (params.output_frames_gen != buffersize_rep) {
-    /* danger! we set buffersize_rep directly, but in this case 
-     * it is safe as we have used 'reserved_samples_rep' 
-     * as the upper limit */
-    buffersize_rep = params.output_frames_gen;
+#ifdef ECA_DEBUG_MODE
+  if ((params.output_frames_gen - buffersize_rep) > 1) {
+    std::cerr << "(samplebuffer) src_src input=" << old_buffer_size 
+	      << ", target=" << buffersize_rep 
+	      << ", processed=" << params.input_frames_used 
+	      << ", space_for=" << reserved_samples_rep 
+	      << ", out=" << params.output_frames_gen << std::endl; 
   }
 #endif
+  if (params.output_frames_gen != buffersize_rep) {
+    /* note: we set buffersize_rep directly and bypass
+     * length_in_samples(), but in this case it is safe as 
+     * we have used 'reserved_samples_rep' as the upper limit */
+    buffersize_rep = params.output_frames_gen;
+  }
+#endif /* ECA_COMPILE_SAMPLERATE */
 }
 
 void SAMPLE_BUFFER::resample_extfilter(SAMPLE_SPECS::sample_rate_t from_srate,
