@@ -13,8 +13,8 @@
 /** ------------------------------------------------------------------------
  * ecasoundc.cpp: Standalone C implementation of the 
  *                ecasound control interface
- * Copyright (C) 2000-2002 Kai Vehmanen (kai.vehmanen@wakkanet.fi)
- * Copyright (C) 2001 Aymeric Jeanneau (ajeanneau@cvf.fr)
+ * Copyright (C) 2000-2005 Kai Vehmanen
+ * Copyright (C) 2001 Aymeric Jeanneau
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,7 +31,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * -------------------------------------------------------------------------
- * History: 
+ * History of major changes:
  *
  * 2002-10-04 Kai Vehmanen
  *     - Rewritten as a standalone implementation.
@@ -53,6 +53,7 @@
 #include <sys/stat.h>     /* POSIX: stat() */
 #include <sys/types.h>    /* POSIX: fork() */
 #include <sys/wait.h>     /* POSIX: wait() */
+#include <signal.h>       /* POSIIX: signal handling */
 
 #include "ecasoundc.h"
 
@@ -98,10 +99,10 @@
 #define ECI_RETURN_TYPE_LOGLEVEL   256
 
 #ifdef ECI_ENABLE_DEBUG
-#define ECI_DEBUG(x) printf(x)
-#define ECI_DEBUG_1(x,y) printf(x,y)
-#define ECI_DEBUG_2(x,y,z) printf(x,y,z)
-#define ECI_DEBUG_3(x,y,z,t) printf(x,y,z,t)
+#define ECI_DEBUG(x) fprintf(stderr,x)
+#define ECI_DEBUG_1(x,y) fprintf(stderr,x,y)
+#define ECI_DEBUG_2(x,y,z) fprintf(stderr,x,y,z)
+#define ECI_DEBUG_3(x,y,z,t) fprintf(stderr,x,y,z,t)
 #else
 #define ECI_DEBUG(x) ((void) 0)
 #define ECI_DEBUG_1(x,y) ((void) 0)
@@ -265,12 +266,24 @@ eci_handle_t eci_init_r(void)
 
     eci_rep->pid_of_child_rep = fork();
     if (eci_rep->pid_of_child_rep == 0) { 
-      /* child */
+      /* first child */
 
       /* -c = interactive mode, -D = direct prompts and banners to stderr */
       const char* args[4] = { NULL, "-c", "-D", NULL };
       int res = 0;
+      struct sigaction sa;
+      pid_t pid;
 
+      sa.sa_handler=SIG_IGN;
+      sigemptyset(&sa.sa_mask);
+      sa.sa_flags=0;
+      sigaction(SIGHUP, &sa, NULL);
+      setsid();
+      if (fork() != 0)
+	  exit(0);    /* first child terminates here */
+
+      /* second child continues */
+     
       args[0] = ecasound_exec;
 
       /* close all unused descriptors */
@@ -287,6 +300,10 @@ eci_handle_t eci_init_r(void)
       close(cmd_send_pipe[1]);
 
       freopen("/dev/null", "w", stderr);
+
+      /* write second child's pid to the (grand) parent */
+      pid = getpid();
+      write(1, &pid, sizeof(pid));
       
       /* notify the parent that we're up */
       res = write(1, args, 1); 
@@ -300,15 +317,32 @@ eci_handle_t eci_init_r(void)
       exit(res);
       ECI_DEBUG("(ecasoundc_sa) You shouldn't see this!\n");
     }
-    else if (eci_rep->pid_of_child_rep > 0) { 
+    else { 
       /* parent */
       int res;
       char buf[1];
+      int status;
+      int pid;
 
+      /*
+	waits for first child to prevent the zombie
+	read grand child prozess id from pipe
+      */
+      waitpid(eci_rep->pid_of_child_rep, &status, 0);
+      res = read(cmd_receive_pipe[0], &pid, sizeof(pid));
+      if ( res != sizeof(pid) ) {
+	  ECI_DEBUG_1("(ecasoundc_sa) fork() of %s FAILED!\n", ecasound_exec);
+	  free(eci_rep->parser_repp);
+	  free(eci_rep);
+	  eci_rep = NULL;
+      }
+      eci_rep->pid_of_child_rep = pid;
       eci_rep->pid_of_parent_rep = getpid();
 
       eci_rep->cmd_read_fd_rep = cmd_receive_pipe[0];
+      close(cmd_receive_pipe[1]);
       eci_rep->cmd_write_fd_rep = cmd_send_pipe[1];
+      close(cmd_send_pipe[0]);
 
       /* switch to non-blocking mode for read */
       fcntl(eci_rep->cmd_read_fd_rep, F_SETFL, O_NONBLOCK);
@@ -322,7 +356,8 @@ eci_handle_t eci_init_r(void)
 	eci_rep = NULL;
       }
       else {
-	write(eci_rep->cmd_write_fd_rep, "debug 259\n", strlen("debug 259\n"));
+	write(eci_rep->cmd_write_fd_rep, "debug 256\n", strlen("debug 256\n"));
+	write(eci_rep->cmd_write_fd_rep, "int-set-float-to-string-precision 17\n", strlen("int-set-float-to-string-precision 17\n"));
 	write(eci_rep->cmd_write_fd_rep, "int-output-mode-wellformed\n", strlen("int-output-mode-wellformed\n"));
 	eci_rep->commands_counter_rep ++;
       
@@ -373,6 +408,10 @@ void eci_cleanup_r(eci_handle_t ptr)
   ECI_DEBUG("(ecasoundc_sa) child exit signalled\n");
 
   if(eci_rep != 0) {
+    /* close descriptors */
+    close(eci_rep->cmd_read_fd_rep);
+    close(eci_rep->cmd_write_fd_rep);
+
     /* if (eci_rep->eci != 0) {} */
     /* delete eci_rep->eci; */
     free(eci_rep->parser_repp);
@@ -648,7 +687,8 @@ void eci_impl_clean_last_values(struct eci_parser* parser)
   DBC_CHECK(parser != 0);
 
   memset(parser->last_s_repp, 0, ECI_MAX_STRING_SIZE);
-  parser->last_los_repp = NULL;
+  eci_impl_los_list_clear(&parser->last_los_repp);
+  DBC_CHECK(parser->last_los_repp == NULL);
   parser->last_i_rep = 0;
   parser->last_li_rep = 0;
   parser->last_f_rep = 0.0f;
@@ -744,6 +784,7 @@ void eci_impl_los_list_add_item(struct eci_los_list** headptr, char* stmp, int l
 
 void eci_impl_los_list_alloc_item(struct eci_los_list **ptr)
 {
+  /* ECI_DEBUG("(ecasoundc_sa) list alloc item\n"); */
   *ptr = (struct eci_los_list*)malloc(sizeof(struct eci_los_list*));
   DBC_CHECK(*ptr != NULL);
   (*ptr)->data_repp = (char*)malloc(ECI_MAX_STRING_SIZE);
@@ -755,15 +796,21 @@ void eci_impl_los_list_clear(struct eci_los_list **ptr)
 {
   struct eci_los_list *i = *ptr;
 
+  ECI_DEBUG_1("(ecasoundc_sa) clearing list, i=%p\n", (void*)i);
+
   while(i != NULL) {
+    /* ECI_DEBUG_1("(ecasoundc_sa) freeing list item %p\n", (void*)i); */
     struct eci_los_list* next = i->next_repp;
 
     if (i->data_repp) {
       {
-	if (next->data_repp != NULL) {
-	  /* ECI_DEBUG_1("(ecasoundc_sa) removing item '%s' from los list\n", i->data_repp); */
-	}
+	  if ( next != NULL ) {
+	      if (next->data_repp != NULL) {
+		  /* ECI_DEBUG_1("(ecasoundc_sa) removing item '%s' from los list\n", i->data_repp); */
+	      }
+	  }
       }
+      /* ECI_DEBUG("(ecasoundc_sa) freeing list item\n"); */
       free(i->data_repp);
     }
     free(i);
@@ -832,9 +879,10 @@ void eci_impl_set_last_los_value(struct eci_parser* parser)
   DBC_CHECK(parser != 0);
   DBC_CHECK(parser->state_rep == ECI_STATE_COMMON_LF_3);
 
-  /* ECI_DEBUG_2("(ecasoundc_sa) parsing a list '%s' (count=%d)\n", parser->buffer_repp, parser->buffer_current_rep); */
+  ECI_DEBUG_2("(ecasoundc_sa) parsing a list '%s' (count=%d)\n", parser->buffer_repp, parser->buffer_current_rep);
 
   eci_impl_los_list_clear(i);
+  /* eci_impl_los_list_clear(&parser->last_los_repp); */
 
   for(n = 0; n < parser->buffer_current_rep && n < parser->msgsize_rep; n++) {
     char c = parser->buffer_repp[n];
@@ -892,7 +940,7 @@ void eci_impl_set_last_values(struct eci_parser* parser)
       break;
 
     case 'l':
-      parser->last_i_rep = atol(parser->buffer_repp);
+      parser->last_li_rep = atol(parser->buffer_repp);
       break;
 
     case 'f':
@@ -997,7 +1045,7 @@ void eci_impl_update_state(struct eci_parser* parser, char c)
 	int len = (parser->buffer_current_rep < ECI_MAX_RETURN_TYPE_SIZE) ? parser->buffer_current_rep : (ECI_MAX_RETURN_TYPE_SIZE - 1);
 	parser->buffer_repp[parser->buffer_current_rep] = 0;
 	memcpy(parser->last_type_repp, parser->buffer_repp, len);
-	parser->last_type_repp[ECI_MAX_RETURN_TYPE_SIZE - 1] = 0;
+	parser->last_type_repp[len] = 0;
 	
 	/* ECI_DEBUG_2("(ecasoundc_sa) found rettype '%s' (len=%d).\n", parser->last_type_repp, parser->buffer_current_rep); */
 
